function out = defineExtraEventCodes(in)
% This function creates predefined event codes and stores them in a variable.
% This variable can be used together with the sendEvent function to conveniently
% send indivisual events to e.g. electrophysiology hardware.
%
% If these functions are used, the following agreements must be respected in order
% to achieve compatible events across multiple experiments:
%
% The first four pin numbers defined in myHardwareSetup.m are reserved on all systems!
% They are used to send standard event codes that are always the same in all setups and
% experiments. The following 5 to n (where n depends on your setup) pins defined in
% myHardwareSetup.m are available for free use.
%
% This function checks whether the specified events can be transmitted with the existing
% hardware (as specified in myHardwareSetup.m).
%
% Usage: event = defineExtraEventCodes(decimalNumber)
%
% Examples:
%
% image(1) = defineExtraEventCodes(1);  	% Individual event for the first image
%
% image(2) = defineExtraEventCodes(435);	% Individual event for image 435 (for systems with a lot
%                                           % of event pins, at least 16 bit)
%
% image = defineExtraEventCodes(1:320);     % Returns a vector with event codes for all 320 images
%                                           % if the system is able to send these codes
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x

% Tobias Otto
% 1.0
% 27.05.2021

% 27.05.2021, Tobias: first draft

%% Init variables
global SETUP

if(~isfield(SETUP, 'events'))
    disp(' ================================================================================');
    disp(' Make sure to call initOTBR with a proper edited myHardwareSetup.m before using');
    disp(' this function! During the computation of the events this function will check');
    disp(' whether the desired number can be created at all with the existing event pins.');
    disp(' There is now way we can continue here. Please read the documentation in mySetup.m');
    disp(' carefully and change your entries');
    disp(' ================================================================================');
    error(' This is a permanent error until you change it ;)');
end

if(isempty(SETUP.events.pinsLen))
    disp(' ================================================================================');
    disp(' No event pins defined in myHardwareSetup.m! There is no way I can compute events');
    disp(' ================================================================================');
    error(' This is a permanent error until you change it ;)');
end

%% Compute events and check if we can handle the numbers
% Check if the required number can be creatwd using the individual event bits available
indivBits   = SETUP.events.pinsLen - 4;   % the first 4 bits are standard bits and can't be used!
maxNum      = pow2(indivBits)-1;

for i=1:length(in)
    if(in(i) > maxNum)
        disp(['Event ' num2str(in(i)) ' can''t be send with only ' num2str(SETUP.events.pinsLen-4) ' individual event pin(s) !!!']);
        disp('Either increase the number of event pins in your system and change ');
        disp('the settings in myHardwareSetup.m or use smaller event numbers! ');
        disp(['With the given data your max event number is: ' num2str(maxNum)]);
        error('This is a permanent error until you change it. I give up.');
    else
        % Add max standard event value to the given number
        out(i) = in(i)+15;
    end
end
