function [tme, decEvents] = sendEvent(stdEvents, extraEvents, camEvents)
% Use this function to send events to your external hardware (e.g. recording
% hardware or electrophysiology hardware).
%
% This function expects two input arguments:
% First argument is a string that defines the standard events:
% --> itiOn, stimOn, off, bhv, rwd, pun, end, trlMtch, indiv (description below)
%
% | decimal | binary    | NAME      | meaning                                           |
% | ------- | ------    |------     |---------------------------------------------------|
% | 0       | 0000      | reset     | reset all pins to zero                            |
% | 1       | 0001      | itiOn     | start of the trial                                |
% | 2       | 0010      | stimOn    | any stimulus on: auditory/ visual/ neural ...)    |
% | 3       | 0011      | off       | anything off: stimulus/ reward/ punishment ...)   |
% | 4       | 0100      | bhv       | any behavior: peck/ fixation/ location ...)       |
% | 5       | 0101      | rwd       | reward on                                         |
% | 6       | 0110      | pun       | punishiment on                                    |
% | 7       | 0111      | end       | trial end                                         |
% | 8-15    | 1000-1111 | trlMtch   | A fail safe to match data. A randomNumber (8:15)  |
%
% Second argument is a number for individual events.
%
% Usage: binVec = sendEvent(StringOfStdEvents, extraEvents)
%
% Examples:
% --------
% sendEvent('stimOn');          % Sends the standard event: stimulus on
%
% [tme, decEvents] = sendEvent('stimOn', stim46);
%                               % Sends the standard event: stimulus on and the individual
%                               % event for stimulus 46
%                               % decEvents is the decimal of all pins, so
%                               % it combines 'stimOn' and stim46
%
% sendEvent('bhv', peck2);      % Sends the standard event: behavioral and indicates with the
%                               % individual event that it was a peck3 event
%
% sendEvent('indiv', trialNum);	% Sends only the individual event of variable trialNum
%
% sendEvent('trlMtch');         % Sends a randomized pattern of numbers between 8 and 15 to
%                               % indicate e.g. the start of a trial
%
% sendEvent('stimOn', stim3, 1);	% Third optional input camEvents.
%                               % Set true to write an event to memory
%                               % mapped files associated with cameras.
%                               % !! This will work only if the field
%                               % SETUP.tracker.datInHand.Data exists !!
%                               % Will write to all camera trackers.
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x

% Tobias Otto
% 1.8
% 15.06.2022

% 27.05.2021, Tobias: first draft
% 14.06.2021, Jonas: - changed output to [time, decEvents]
%                      decEvent is the decimal of the events on all pins
%                    - changed behavior on trlMtch to allow extraEvents
%                    - added optional argument to write to memory mapped
%                      files in camera tracking
% 16.08.2021, Sara : - Correct formula used to convert decimals to binary
%           & Lukas  - Define SETUP.events.powersOf2 , SETUP.events.pinsLen
%                      and assign the convention values to the standard events
%                      (done in myParadigmSetup). Necessary for this function to work.
%
% 10.09.2021, Aylin: - changed saving to memory mapped file (timestamp +
%                      decimal event) to prevent imperfect saving (sometimes
%                      correct timestamp is saved with not yet updated event)
%
% 15.10.2021, Sara : - added 'reset' to standard events. The deuteron's neural logger
%                      only registers the changes in current of a pin, when an event code
%                      is sent. It proved to be relevant to set the output of Network Box
%                      to off? prior to the start of the trial, in order to have a clear
%                      refence of the status of all pins.
%
% 10.11.2021, Aylin: - Accommodate to the Deuteron system:
%            & Sara    It was stipulated an event code would correspond to a rising edge
%                      in only one pin. It was also agreed that, in an experiment, a trial
%                      would start with the event code itiOn, which the logger would
%                      register as the binary 0001. All the following events would be coded
%                      as an iteration of pins 2 to 4. Whenever a new event code is sent,
%                      the pin that was previously on is set to off.
%                      Requires the addition of the following fields, to the trigger pins
%                      section of myHardwareSetup.m
%                      SETUP.events.setup = 'Deuteron'; (with the neural recording system in use)
%                      SETUP.events.lastPin = 1;   	(in order to accommodate to the Deuteron system)
% 12.04.2022, Aylin: - changed specification of ephys setup
% 02.05.2022, Aylin: - corrected sending of event codes for Intan system
%            & Lukas
% 15.06.2022, Tobias: replaced pause with Waitsecs; replaced strings with ID and corrected send
%                     event codes for all other systems

%% Init variables
global SETUP

if(nargin == 1)
    extraEvents  = 0;
    camEvents    = false;
elseif nargin==2
    camEvents    = false;
end

%% Compute vector
if(strcmpi(stdEvents, 'trlmtch'))
    % Events 1 to 7 are defined standard events, events 8 to 15 are used for trial match
    % Only if we have at least 4 bits available
    stdEvents   = randperm(8,1)+7;
    binVec      = rem(floor((stdEvents+extraEvents) ./ SETUP.events.powersOf2),2) & SETUP.events.pinsLen > 3;

elseif(strcmpi(stdEvents, 'reset'))
    % make sure all pins are off before the trial starts
    binVec = zeros(1,SETUP.events.pinsLen);
else
    % Use only standard events: 1 to 7
    binVec = rem(floor((SETUP.events.stdEvents.(stdEvents)+extraEvents) ./ SETUP.events.powersOf2),2); %
end

%% Set event
if SETUP.ephysSetupID == 1 && ~strcmpi(stdEvents, 'itiOn')      % SETUP.ephysSetupID=1 --> deuteron
    % turn pin on + off for deuteron to detect rising edge
    if SETUP.events.lastPin == 4
        bIO(SETUP.events.pins(mod(SETUP.events.lastPin,4) + 2),1);
    else
        bIO(SETUP.events.pins(SETUP.events.lastPin+1),1);
    end

    bIO(SETUP.events.pins(SETUP.events.lastPin),0);

    % update which pin was modified last
    if SETUP.events.lastPin == 4
        SETUP.events.lastPin = mod(SETUP.events.lastPin,4) + 2;
    else
        SETUP.events.lastPin = mod(SETUP.events.lastPin,4) + 1;
    end

elseif SETUP.ephysSetupID == 1 && strcmpi(stdEvents, 'itiOn')       % SETUP.ephysSetupID = 1 --> deuteron
    bIO(SETUP.events.pins, binVec(1:length(SETUP.events.pins)));    % same as previous event code architecture
    SETUP.events.lastPin = 1;

elseif SETUP.ephysSetupID == 2                                      % SETUP.ephysSetupID = 2 --> intan
    % corrected for new Intan system:
    pinsToChange = SETUP.events.pins(logical(binVec));  % which pins need to be turned on/off
    bIO(pinsToChange,ones(1,length(pinsToChange)))      % turn pins on
    WaitSecs(0.002)                                     % wait dur
    bIO(pinsToChange,zeros(1,length(pinsToChange)))     % turn pins off

else
    % All other systems
    bIO(SETUP.events.pins, binVec(1:length(SETUP.events.pins))); % same as previous event code architecture
end

%% write full decimal event and timestamp to memory mapped file of each camera
if camEvents
    for i=1:length(SETUP.tracker.camCtrl)
        SETUP.tracker.camCtrl{i}.Data(3:4) = [tme, decEvents]; % change Aylin
    end
end

%% generate the output
tme         = GetSecs;
decEvents   = binVec * (2 .^ (0:SETUP.events.pinsLen-1))'; % convert the binary to decimal integer