function simpleChoice
% This is a simple example program where you can distinguish between stimuli presenting humans and
% stimuli presenting other objects.

% Tobias Otto
% 1.0
% 16.11.2021

% 16.01.2021, Tobias: first draft

%% Init toolbox and experiment
initOTBR;
startExperiment;    % Not necessary for human experiments yet, but it doesn't hurt

%% Init variables
% We recommend to initialize all variables in this section and to program constantly with
% variables. This makes changes (changing presentation times, etc.) easier and this would be the
% only place where changes have to be done.

% Init variables
p.subject       = inputdlg({'Subject'});
p.fixCross1     = 5;    % Time in seconds for the first fixation cross
p.fixCross2     = 1;    % Time in seconds for fixation cross in loop
p.ansTime       = 5;    % Time in seconds for subject to answer
p.feedbTime     = 2;    % Time in seconds for feedback
p.thankYou      = 1;    % Time in seconds for thank you

p.ansKey(1)     = 37;   % Key code for left arrow
p.ansKey(2)     = 39;   % Key code for right arrow

p.numHuman      = 6;    % Number of human stimuli
p.numNonHuman   = 12;   % Number of non human stimuli
p.numTotal      = p.numHuman+p.numNonHuman;

% Get some technical detials
p.savePath    	= [pwd '\Result\'];
p.stimuli       = [pwd '\Stimuli\'];
p.info          = getBasicInformation;
p.info2         = 'This is a simple example program of a choice experiment where subjects have to distinguish between humans and objects.';

try
    %% Init screen
    initWindow(2);

    %% Load and randomize images
    % Get all human stimuli
    f1 = dir([p.stimuli 'Human\*.jpg']);

    % Load all stimuli we found
    for i=1:length(f1)
        im.human(i) = loadImage([f1(i).folder filesep f1(i).name]);
    end

    % Get all non human stimuli
    f2 = dir([p.stimuli 'NonHuman\*.jpg']);

    % Load all stimuli we found
    for i=1:length(f2)
        im.nonHuman(i) = loadImage([f2(i).folder filesep f2(i).name]);
    end

    % Load question image
    im.quest = loadImage([p.stimuli 'question.jpg']);

    %% Computer buffers
    % This buffers will help to present the stimuli randomized

    % Make sure that the same stimulus type is not shown more than twice in a row
    % 1 --> non human stimuli
    % 2 --> human stimuli
    p.buffer = randomOrder(2, p.numTotal, 'ratio', [p.numNonHuman/p.numTotal p.numHuman/p.numTotal], 'maxRepeat', 2);

    % Now get the randomized indices for the stimuli
    p.buffer(2, p.buffer(1,:) == 1) = randperm(p.numNonHuman);
    p.buffer(2, p.buffer(1,:) == 2) = randperm(p.numHuman);

    %% Show instructions and first Fixation cross
    showInstructions('instructions.txt');
    fixationCross(p.fixCross1);

    %% Finally start
    for i=1:p.numTotal
        %% Init variable for trial
        mode = p.buffer(1,i);       % 1->nonHuman | 2->human

        if(mode == 1)
            stim    = im.nonHuman(p.buffer(2,i));
            goodKey = p.ansKey(1);
            badKey  = p.ansKey(2);
        else
            stim = im.human(p.buffer(2,i));
            goodKey = p.ansKey(2);
            badKey  = p.ansKey(1);
        end

        %% Present stimulus (either human or non human)
        showStimuli(stim, 'dontFlip');
        showStimuli(im.quest, 'y', 80);

        %% Wait for subject to answer
        res = getAnswer(p.ansTime, 'goodKey', goodKey, 'badKey', badKey);

        %% Show feedback
        if(res.answer == 1)
            showText('Correct', 'color', [0 255 0]);
        else
            showText('Wrong', 'color', [255 0 0]);
        end

        WaitSecs(p.feedbTime);

        %% Iti and save result variables
        fixationCross(p.fixCross2);

        r(i).res    = res;
        r(i).stim   = stim.filename;
        r(i).buffer = p.buffer(:,i);
    end

    %% Compute user feedback
    res         = [r.res];
    resHum      = res(p.buffer(1,:) == 1);
    resNonHum   = res(p.buffer(1,:) == 2);

    % Compute answer in %
    com.corrHum     = sum([resHum.answer] == 1)/length(resHum)*100;
    com.corrNonHum  = sum([resNonHum.answer] == 1)/length(resNonHum)*100;

    % Compute mean RT in s
    com.rtHum       = mean([resHum.rt]);
    com.rtNonHum    = mean([resNonHum.rt]);

    % User feedback
    disp(['Human stimuli:     correct = ' num2str(com.corrHum) '% | RT = ' num2str(com.rtHum)])
    disp(['Non human stimuli: correct = ' num2str(com.corrNonHum) '% | RT = ' num2str(com.rtNonHum)])

    %% Save data
    save2File(r, p, com, 'subject', p.subject{1}, 'path', p.savePath);

    %% Clean up
    showText('Thank you!');
    WaitSecs(p.thankYou);

    stopExperiment;
catch
    closeWindow;
    errorSave;
end

