function myHardwareSetup
% This function initializes the OTBR Toolbox and sets all basic parameters
% of the hardware in your setup. This function needs to be stored on each
% experimental computer. All required hardware definitions are stored in
% the global variable SETUP created by this function. Edit this function when
% building a new setup. Description of specific field names and values can
% be found in the function..
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x
%
% See also:  initOTBR, myParadigmSetup

% VERSION HISTORY:
% Author:         Tobias Otto, Jonas Rose
% Version:        1.2
% Last Change:    24.03.2022
%
% 05.06.2018, Tobias: first draft
% 05.12.2021, Tobias: typo
% 24.03.2022, Tobias: cleaning and new start values

%% Init global struct
global SETUP

%% (1a) Please enter path to the OTBR Toolbox folder
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SETUP.toolboxPath = 'D:\Matlab\OTBR-Toolbox-Gitlab\master';

%% (1b) Please enter path to the Psychophysics Toolbox folder
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SETUP.psychToolboxPath  = 'D:\Matlab\PsychToolboxOrig\PsychToolbox3Orig_GitHub_Branches\Master\Psychtoolbox';

%% (1c) Please specify platform of your experimental computer
% This version of the OTBR Toolbox supports these platforms:
% 'Windows64Bit'    --> Computer with a 64 bit Windows operating system
%                       installed  and a 64 bit MATLAB
% 'RaspberryPi3'    --> Raspberry Pi with Octave
SETUP.platform = 'Windows64Bit';

%% (2) Please choose the IO device
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The OTBR Toolbox provides nativ access to two different IO devices: the
% IO-Warrior 40 and a self made parallel port Interface. Please choose the
% interface you want to use.
% For training purposes the keyboard can be used as well.
% 'Parallel'     -> Grants access to the self made parallel port interface (WINDOWS ONLY)
% 'NetworkBox'   -> Grants access to the Odroid network IO device (Biopsychology)
% 'NetworkBoxACN'-> Grants access to the Odroid network IO device (ACN)
% 'Keyboard'     -> For testing your program in your office using the keyboard
% 'RaspberryPi3' -> Grants access to the IO pins of the Raspberry 3B+
SETUP.io.interface = 'Keyboard';

% Add IP address of NetworkBox as string here. E.g. '192.168.0.1'
% Otherwise leave this field empty!
SETUP.networkDIO.odroidIP   = [];

%% (3a) Please specify the OUTPUT devices
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Each device is connected to a separate pin/output at the output device.
% Please specify the pin numbers for the devices used in the setup. Unused
% outputs should contain an empty matrix.
SETUP.io.houseLight  = [];
SETUP.io.feeder      = 4;
SETUP.io.feederLight = 5;
SETUP.io.feederPower = [];
SETUP.io.punishment  = [];

%% (3b) Please specify the feeder
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This section defines if the amount of food given to a subject is defined
% in time (seconds) or in amount of food (e.g. number of pellets)
% Define time in seconds          --> 'time'
% Define amount in exact numbers  --> 'amount'
SETUP.io.feederType     = 'time';

% Manual shaping (WINDOWS ONLY)
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set this value to one, if you want to manually reward by pressing the 'f'
SETUP.manualShaping.status = 0;

%% (3c) Please specify the TRIGGER pins of you IO hardware
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This section defines if the amount of hardware pins that are used
% to send triggers to external hardware.
% Leave empty if you don't use this feature. Otherwise define the
% hardware pin numbers that are available for the OTBR toolbox.
% At least 3 pins are needed in SETUP.events.pins to use a minimal
% standard trigger functionality! 4 pins are recommended!
%
% Make sure the first pin defined in this field is connected
% to the first pin on the external hardware! The first pin
% defined should result in event number 1, the second should
% result in event number 2, ....
SETUP.events.pins = [];

%% (4) Please specify the number(s) of the INPUT pins
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Each input device (e.g. real switch or pecking key)
% must be specified here. Physicsal switches are connected to a single
% pin/input to the IO device. Please be aware that the first pin number
% corresponds to the first input key (doesn't matter on which input pin the
% key is connected). That means that the first response key could be
% connected to pin 7 and the second response key to pin 3. That would
% result in [7 3].
% If you are using touch screens only and no external input device you can
% leave this entry empty: SETUP.io.inputKeys = [];
SETUP.io.inputKeys     = [1 2 3 4 5 6 7 8 9];

%% (5) Please specify monitor dimensions
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Screen width and height in centimeters
SETUP.screen.ScreenWidth      = 16.5;
SETUP.screen.ScreenHeight     = 12.8;

%% (6) TOUCH SCREEN SETTINGS (WINDOWS ONLY)
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set this value to one, if you want to use a touch screen
SETUP.touchScreen.on = 0;

% Set to one to plot a red dot to the recognised touch location
% Not recommended during experiments!
SETUP.touchScreen.display = 0;

% Enter minimum time between two touches (in seconds)
SETUP.touchScreen.deltaT = 0.01;

% Enter minimum distance between two touches (in centimeters)
% After call of initWindow the relative value is computed and replaced
SETUP.touchScreen.deltaP = 0.5;

%% (7) Set Psych toolbox verbosity
% Set the verbosity of the psychophysics toolbox
% 0 - Disable all output - Same as using the SuppressAllWarnings flag.
% 1 - Only output critical errors.
% 2 - Output warnings as well.
% 3 - Output startup information and a bit of additional information.
%     This is the default.
% 4 - Be pretty verbose about information and hints to optimize your code
%     and system.
% 5 - Levels 5 and higher enable very verbose debugging output, mostly
%     useful for debugging PTB itself, not generally useful for end-users.
%
% For normal use 1 is fine!
SETUP.psychToolbox.verbosity = 1;

%% (8) UDP communication for remote status display
% contact details of the remote listener
SETUP.udp.ip        = [];               % remote ip
SETUP.udp.port      = 5006;             % remote port
SETUP.udp.lclPort   = 5005;             % local port

%% (9) EPHYS SETTINGS
% %%%%%%%%%%%%%%%%%%%%
% Enter values for the white bar at the top of the screen.  This is useful
% to get the exact timing when the stimulus was presented on the monitor.
% The coordinates are entered with reference to the left lower corner of
% the screen.
% The values are entered in centimeters and have the following meaning:
% 1. left lower corner, X coordinate
% 2. left lower corner, Y coordinate
% 3. width of bar
% 4. height of bar
%
% Defaults should be okay!

% White bar at the top of the screen, stretched over the whole screen
% [left, bottom, width, height]
SETUP.ephys.photodiode = [0, SETUP.screen.ScreenHeight-2, SETUP.screen.ScreenWidth, SETUP.screen.ScreenHeight];

%% (9a) Please specify the ephys hardware
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Enter recording system used ('intan', 'deuteron' or '' for all other systems).
% Necessary for sendEvent function.
SETUP.ephysSetup = '';

%% (10) Raspberry 3B+ SETTINGS
% %%%%%%%%%%%%%%%%%%%%%%%%%%%
% Enter the IP addresses of your Raspberry 3B+ as string systems here.
% The order of the IP addresses is important, because the system that is
% entered first is known as system one, the second system entered is known
% as system two, ...
% e.g. SETUP.raspberry.ip{1} = '192.168.0.20';
% e.g. SETUP.raspberry.ip{2} = '192.168.0.21';
SETUP.raspberry.ip{1} = [];

%% (11) Inputs for tracking
% Memory mapped file(s) for gaze tracker
SETUP.tracker.bufFleNme = {};
SETUP.tracker.cfgFleNme = {};
SETUP.tracker.bufFle    = {};    	% generate it using tracker.preview
SETUP.tracker.frameRate = 75;

% Select the training setup used ('arena', 'box'); leave empty if no tracking is used
% specifies in videoTracker which frameAcquired function will be used
SETUP.trainingSetup = '';

% Conversion matrix: transforms tracking coordinates into pixel of the
% screen; if not existing create via conversionMatrixCreator_Bird /
% conversionMatrixCreator_Human, set path to conversion matrix
SETUP.tracker.conversionMatNme = '';

%% (12) Select trigger counter for MRI experiments
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% With this entry the program for recording the triggers is selected.
% The program is part of the OTBR Toolbox and will be started automatically
% if desired.
% Two options are available:
% * 'Parallel'      --> to detect triggers at the parallel port
% * 'Network'    	--> to detect triggers with a network IO device
% * 'Keyboard'      --> to detect keyboards with a (virtual) keyboard (keys t and ´)
% * []          	--> Leave empty if you want to start the program manually!
SETUP.MRI.triggerCounter = [];

% Name of the application to be started.
% Please make sure that the application is located in the
% "Current Directory". If necessary, it must be copied from the folder
% <OTBR-Toolbox>\Tools\MRI-Counter into the "Current Directory"
% Leave empty if you want to start the program manually!
% e.g. SETUP.MRI.triggerCounterName = 'triggerCounter2_MCR9-2.exe';
SETUP.MRI.triggerCounterName = [];
