function bIO(port, val)
% Use this function to read or write data to the 8 way relay card
% Only one card is supported
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x
%
% See also:  init8PortRelay, close8PortRelay

% VERSION HISTORY:
% Author:         Tobias Otto
% Version:        0.4
% Last Change:    14.07.2011

% 28.02.2011, Tobias: first draft
% 04.03.2011, Tobias: changed logic for init
% 04.04.2011, Tobias: added all on and all off
% 14.07.2011, Tobias: cleaning available bytes fom ralais card

%% Init variables
global SERIAL8PORT
binVal  = [1 2 4 8 16 32 64 128];
setOn   = port(val==1);
setOff  = port(val==0);

%% Check user input
if(nargin ~= 2)
    disp(' **********************************************************');
    disp(' The 8 way relais card from Conrad Electronics is an');
    disp(' output only device. Reading any data is not implemented.');
    disp(' **********************************************************');
    error('This is a permanent error. Please solve problem and try again');
end

if(length(port) ~= length(val))
    disp(' **********************************************************');
    disp(' The number of pins and the corrosponding status must have');
    disp(' the same length');
    disp(' **********************************************************');
    error('This is a permanent error. Please solve problem and try again');
end

%% Check initialization
if(~exist('SERIAL8PORT','var') || ~isfield(SERIAL8PORT,'initDone') || SERIAL8PORT.initDone == 0)
    init8PortRelay;
end

%% Turn on
if(~isempty(setOn))
    %% Check special input: e.g. all on
    if(setOn == 0)
        setOn = 1:8;
    end

    %% Start communication
    byte(1) = 6;                    % Command       --> Set single
    byte(2) = SERIAL8PORT.cardAdr;	% Card addess   --> Set to 0
    byte(3) = sum(binVal(setOn));   % Data
    byte(4) = computeCRC(byte);     % Checksum
    byte    = uint8(byte);
    fwrite(SERIAL8PORT.s, byte, 'uint8');
end

%% Turn off
if(~isempty(setOff))
    %% Check special input: e.g. all off
    if(setOff == 0)
        setOff = 1:8;
    end

    %% Wait, if something was turned on
    % Necessary due to slow communication
    if(~isempty(setOn))
        pause(0.001);
    end

    %% Start communication
    byte(1) = 7;                    % Command       --> Set single
    byte(2) = SERIAL8PORT.cardAdr;	% Card addess   --> Set to 0
    byte(3) = sum(binVal(setOff));	% Data
    byte(4) = computeCRC(byte);     % Checksum
    byte    = uint8(byte);
    fwrite(SERIAL8PORT.s, byte, 'uint8');
end

%% Clean bytes available buffer
while(SERIAL8PORT.s.BytesAvailable ~= 0)
    fread(SERIAL8PORT.s, SERIAL8PORT.s.BytesAvailable,'uint8');
end