function out = bIO(varargin)
% Use this function to read or write data to the network io device
%
% bIO(inputNumber) returns the status of the selected input
% bIO(outputNumber, status) sets the selected output to either 1 or 2
% bIO([], [], event) sets a 9 bit event to the IO device (only for CED 1401 hardware)
% bIO returns the software version from IO device
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x
%
% See also: initNetworkIO, closeNetworkIO

% VERSION HISTORY:
% Author:         Tobias Otto
% Version:        1.11
% Last Change:    09.06.2022

% 03.06.2015, Tobias: first draft
% 05.06.2015, Tobias: bugfix in output section
% 18.04.2016, Tobias: added 1ms waitig time after request for reading data
% 26.04.2016, Tobias: removed waiting and changed code for Odroid, cosmetics
% 27.04.2016, Tobias: bugfix, num of output pins was wrong
%                     added warning that we don't have events yet!
%                     added version of server software when calling bIO directly
% 06.07.2016, Tobias: Bugfix for special input: e.g. all on
% 22.07.2016, Tobias: cosmetics in output
% 28.10.2016, Tobias: adjusted time option (network box returns time in ms now)
%                     cosmetics
% 07.12.2016, Tobias: added events for CED 1401
% 03.03.2018, Tobias: added automatic init to be compatible with other code
% 31.03.2020, Tobias: cosmetics and replaced pause with WaitSecs
% 09.06.2022, Tobias: updated number of possible outputs to 18 in bIO(0,x) case

%% Init variables
global SETUP
out = 0;

%% Check initialization
if(SETUP.networkDIO.initDone == 0)
    openNetworkIO;
end

%% Check input pins
if(nargin == 1)
    % Init variables for reading
    outInt  = uint32([0 0 0]);

    % 2 as first number means read data from pins
    % Send data
    pnet(SETUP.networkDIO.obj, 'write', uint32([2 varargin{1}]), 'intel');
    pnet(SETUP.networkDIO.obj, 'writepacket');

    siz2Read	= pnet(SETUP.networkDIO.obj, 'readpacket','uint32', 'noblock');
    if(siz2Read ~= 0)
        outInt  = pnet(SETUP.networkDIO.obj, 'read', siz2Read, 'uint32', 'intel', 'view', 'noblock');
        if(outInt(3) == 2)
            out = double(outInt(2));
        else
            out = 0;
        end
    end

    %% Use output pins
elseif(nargin == 2)
    % Copy varargin to variable
    pins    = varargin{1};
    status  = varargin{2};

    % Check user input
    if(length(pins) ~= length(status))
        disp(' **********************************************************');
        disp(' The number of pins and the corrosponding status must have');
        disp(' the same length');
        disp(' **********************************************************');
        error('This is a permanent error. Please solve problem and try again');
    end

    % Check special input: e.g. all on
    if(pins == 0)
        pins = 1:18;
        if(status == 1)
            status = ones(1,length(pins));
        else
            status = zeros(1,length(pins));
        end
    end

    % four means set pins for events/digital outputs
    pnet(SETUP.networkDIO.obj, 'write', uint32([4 pins status]), 'intel');
    pnet(SETUP.networkDIO.obj, 'writepacket');

    %% Use event pins (for CED 1401 only)
elseif(nargin == 3)
    % Copy varargin to variable
    eventData = varargin{3};

    % six means set pins for events/digital outputs
    pnet(SETUP.networkDIO.obj, 'write', uint32([6 eventData]), 'intel');
    pnet(SETUP.networkDIO.obj, 'writepacket');
elseif(nargin == 0)
    % 5 as first number means get software version
    % Send data
    pnet(SETUP.networkDIO.obj, 'write', uint32(5), 'intel');
    pnet(SETUP.networkDIO.obj, 'writepacket');

    % Waiting and look for some data
    WaitSecs(0.1);
    siz2Read = pnet(SETUP.networkDIO.obj, 'readpacket','uint32', 'noblock');

    if(siz2Read ~= 0)
        out = pnet(SETUP.networkDIO.obj, 'read', siz2Read, 'uint32', 'intel', 'view', 'noblock');
        out = out(1:2); % Return only time and version
        % Plot result
        fprintf('Server version %.2f | time since connection:%ds\n', double(out(2))/1000, out(1)/1000);
    else
        out = [-1 -1];
    end
end