function openNetworkIO
% Init network communication to Network-IO device
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x
%
% See also:  bIO, closeNetworkIO

% VERSION HISTORY:
% Tobias Otto
% 1.6
% 31.03.2020

% 03.06.2015, Tobias: first draft
% 05.06.2015, Tobias: added error handling
% 02.03.2016, Tobias: IP address now set in mysetup
% 25.10.2016, Tobias: removed init of variable initDone
% 28.10.2016, Tobias: added check for correct firmware version
% 31.10.2016, Tobias: replaced check for old firmware
% 31.03.2020, Tobias: added more error handling

%% Init variables
global SETUP
SETUP.networkDIO.odroidPort = 5005;
SETUP.networkDIO.localPort  = 5005;
SETUP.networkDIO.obj        = [];
pins        = 0;
reply       = 0;
multiSwitch = 0;
done        = 0;

%% Connect to odroid
if(SETUP.networkDIO.initDone == 0)
    SETUP.networkDIO.obj = pnet('udpsocket',SETUP.networkDIO.localPort);
    if(SETUP.networkDIO.obj < 0)
        error('Could not create an UDP socket. I give up!');
    end
    pnet(SETUP.networkDIO.obj,'udpconnect',SETUP.networkDIO.odroidIP, SETUP.networkDIO.odroidPort);

    status = pnet(SETUP.networkDIO.obj,'status');
    if(status > 0)
        disp('UDP socket created and connection established!');
    end
    % Set timeout
    pnet(SETUP.networkDIO.obj,'setwritetimeout',0.1)
    pnet(SETUP.networkDIO.obj,'setreadtimeout',0.1)

    %% Init time
    word = uint32(1);    % one means set time to zero
    % Send word to odroid
    pnet(SETUP.networkDIO.obj, 'write', word, 'intel');
    pnet(SETUP.networkDIO.obj, 'writepacket');

    %% Check communication
    % Send a "get time" packet and wait for a response
    word = uint32(5);    % three means get version and time
    % Send data
    pnet(SETUP.networkDIO.obj, 'write', word, 'intel');
    pnet(SETUP.networkDIO.obj, 'writepacket');

    % Wait for odroid to answer
    t = toc+5;
    while(t>toc)
        drawnow
        % Check if there is something to read
        siz2Read = pnet(SETUP.networkDIO.obj, 'readpacket','uint32', 'noblock');
        if(siz2Read ~= 0)
            % Get data
            dat = pnet(SETUP.networkDIO.obj, 'read', siz2Read, 'uint32', 'intel', 'view', 'noblock');

            % Check for correct response
            if(dat(end) == 5)
                % Set variable that init is done
                SETUP.networkDIO.initDone = 1;
                % User info
                disp([' Connected to IP ' SETUP.networkDIO.odroidIP ':' num2str(SETUP.networkDIO.odroidPort)]);
                disp([' Network Box Version: ' num2str(double(dat(2))/1000)]);
                done = 1;
                break;
            end
        end
    end

    if(done ~= 1)
        disp(' --------------------------------------------------------------------');
        disp(' There was a timeout during communication with the network IO device!');
        disp(' This is a permanent error and I give up now :(');
        error('Please solve problem and try again!');
    else
        % Add check of toolbox version of the time bug
        % Otherwise empty all buffers and continue
        if(double(dat(2))/1000 < 2.9)
            disp(' --------------------------------------------------------------------');
            disp(' You are using a network IO box with a firmware version before 2.9!');
            disp(' It is not recommended to use this IO box. Please update the Firmware');
            disp(' --------------------------------------------------------------------');
            % Close network connection
            closeNetworkIO;

            error('This is a permanent error! I give up now!');
        else
            % Empty any buffers that could be filles during setup
            % Waiting and look for some data
            WaitSecs(0.1);
            siz2Read = pnet(SETUP.networkDIO.obj, 'readpacket','uint32', 'noblock');
            if(siz2Read ~= 0)
                out = pnet(SETUP.networkDIO.obj, 'read', siz2Read, 'uint32', 'intel', 'view', 'noblock');
            end
        end
    end
else
    disp(' ---------------------------------------------');
    disp(' The network IO device is already initialized!');
    disp([' Connected to IP ' SETUP.networkDIO.odroidIP ':' num2str(SETUP.networkDIO.odroidPort)]);
    disp(' ---------------------------------------------');
end

