function out = bIO(varargin)
% Use this function to read or write data to the network io device
%
% bIO(inputNumber) returns the status of the selected input
% bIO(outputNumber, status) sets the selected output to either 1 or 2
% bIO([], [], event) sets a 9 bit event to the IO device
% bIO returns the software version from IO device
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x
%
% See also: initNetworkIO, closeNetworkIO

% VERSION HISTORY:
% Author:         Tobias Otto
% Version:        1.1
% Last Change:    28.10.2016

% 24.10.2016, Tobias: first draft
% 28.10.2016, Tobias: adjusted returned time from IO network box

%% Init variables
global SETUP
out = 0;

%% Check initialization
% if(~isfield(SETUP,'networkDIO') || SETUP.networkDIO.initDone == 0)
%     initNetworkIO;
% end

%% Check input pins
if(nargin == 1)
    % Two as first number means: read data from pins - form data for Odroid
    % Send data
    fwrite(SETUP.networkDIO.obj, uint32([2 varargin{1}]), 'uint32')

    % We get data if data are available from a callback function. All we
    % have to do here is read from the variables that holds the data
    out                     = SETUP.networkDIO.data(2);
    % Reset data
    SETUP.networkDIO.data  	= [0 0 0];

    %% Use output pins
elseif(nargin == 2)
    % Copy varargin to variable
    pins    = varargin{1};
    status  = varargin{2};

    % Check user input
    if(length(pins) ~= length(status))
        disp(' **********************************************************');
        disp(' The number of pins and the corrosponding status must have');
        disp(' the same length');
        disp(' **********************************************************');
        error('This is a permanent error. Please solve problem and try again');
    end

    % Check special input: e.g. all on
    if(pins == 0)
        pins = 1:5;
        if(status == 1)
            status = ones(1,length(pins));
        else
            status = zeros(1,length(pins));
        end
    end

    % Four as first value means: set pins for events/digital outputs
    % Send data
    fwrite(SETUP.networkDIO.obj, uint32([4 pins status]), 'uint32');

    %% Use event pins
elseif(nargin == 3)
    warning('Events are not yet implemented in the network io box');
elseif(nargin == 0)
    % Five as first number means get software version
    % Send data
    fwrite(SETUP.networkDIO.obj, uint32(5), 'uint32');

    % Waiting and look for data
    t = toc+1;
    while(~all(SETUP.networkDIO.data) && t > toc), end

    % Get data (we get the data from a callback function)
    if(all(SETUP.networkDIO.data))
        out                     = SETUP.networkDIO.data;
        SETUP.networkDIO.data   = [0 0 0];
        % Plot result
        fprintf('Server version %.2f | time since connection:%.3fs\n', double(out(2))/1000, out(1)/1000);
    else
        disp('There was an error somewhere ...');
    end
end
