function out = bIOStrobe(pin, num, freq, duty)
% Use this function to send a strobe signalvia a selected output pin or multiple pins.
% During the presentation of the strobe signal the IO device itself is blocked due to the
% presentation of the strobe signal, but MATLAB is still able to execute other commands!
%
% To estimate when the presentation of the square signal is finished, the time to the end of the
% presentation is returned in seconds
%
% Usage: out = bIOStrobe(pin, numStrobes, freq, dutyCycle);
%
% pin   : output pin where the signal is presented
% num   : number of square signal cycles that are presented.
% freq  : frequency of the signal in Hz
% duty  : (optional) duty cycle in percent (defines the percentage the signal is high)
% out   : the estimated time when the signal is presented (MATLAB is not blocked, but the IO device)
%
% Example: bIOStrobe(2, 400, 20); % Presents 400 cycles at 20 Hz on pin 2
%
% Example: bIOStrobe(1:4, 400, 60); % Presents 400 cycles at 60 Hz on pins 1 to 4
%
% % Example: bIOStrobe(1:2, 400, 120, 70); % Presents 400 cycles at 120 Hz on pins 1 to 2 with a
%                                          % duty cycle of 70% (70% signal high and 30% signal low)
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x
%
% See also: initNetworkIO, closeNetworkIO

% VERSION HISTORY:
% Author:         Tobias Otto
% Version:        1.3
% Last Change:    08.11.2021

% 24.08.2021, Tobias: first draft
% 25.08.2021, Tobias: added option for multiple pins
% 10.09.2021, Tobias: introduced duty cycle
% 08.11.2021, Tobias: changed version to work with NetworkBoxMatlab hardware (instrument control toolbox)

%% Init variables
global SETUP

%% Check initialization
if(SETUP.networkDIO.initDone == 0)
    openNetworkIO;
end

if(SETUP.io.interfaceNum ~= 8)
    error('This function only works with interface "NetworkBox" version 3.1 or higher');
end

if(nargin < 4)
    duty = 50;
end

%% Send data
fwrite(SETUP.networkDIO.obj, uint32([7 num freq duty pin]), 'uint32')

%% Compute time
out = 1/freq * num;

