function openNetworkIO
% Init network communication to Network-IO device
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x
%
% See also:  bIO, closeNetworkIO

% VERSION HISTORY:
% Tobias Otto
% 1.0
% 24.10.2016

% 24.10.2015, Tobias: first draft

%% Init variables
global SETUP
SETUP.networkDIO.odroidPort = 5005;
SETUP.networkDIO.localPort  = 5006;
SETUP.networkDIO.obj        = [];
SETUP.networkDIO.data     	= [0 0 0];
done                        = 0;

%% Init network
if(exist('instrfind','file') ~= 2)
    disp(' ------------------------------------------------------');
    disp(' This function uses the "Instrument Control Toolbox" !');
    disp('         Please solve problem and try again !');
    disp(' ------------------------------------------------------');
    error('This is a permanent error! I give up.');
end

%% Connect to odroid
if(SETUP.networkDIO.initDone == 0)
    % Create an udp object
    u = udp(SETUP.networkDIO.odroidIP ,SETUP.networkDIO.odroidPort);%, 'LocalPort', SETUP.networkDIO.localPort);
    SETUP.networkDIO.obj = u;   % This step is necessary for whatever reason :(

    % Set byte order correct and a time out
    set(SETUP.networkDIO.obj, 'ByteOrder', 'littleEndian');

    % Set input buffer size to 12 bytes! This is the max we get
    %set(SETUP.networkDIO.obj, 'InputBufferSize', 12);
    set(SETUP.networkDIO.obj, 'InputDatagramPacketSize', 12);

    % Set callback function
    %     set(SETUP.networkDIO.obj, 'BytesAvailableFcn', 'dataAvailable', ...
    %         'BytesAvailableFcnMode', 'byte', ...
    %         'BytesAvailableFcnCount', 12);
    set(SETUP.networkDIO.obj, 'DatagramReceivedFcn', 'dataAvailable');


    % Open connection
    fopen(SETUP.networkDIO.obj);

    % Check status
    if(strcmpi(SETUP.networkDIO.obj.Status, 'open'))
        disp('UDP socket created and connection established!');
    end

    %% Init time
    word = uint32(1);    % one means set time to zero
    % Send word to odroid
    fwrite(SETUP.networkDIO.obj, word, 'uint32')

    %% Check communication
    % Send a "get time" packet and wait for a response
    word = uint32(5);    % three means get version and time
    % Send data
    fwrite(SETUP.networkDIO.obj, word, 'uint32')

    % Wait for odroid to answer
    t = toc+5;
    while(t>toc)
        % Check if there is something to read
        if(any(SETUP.networkDIO.data))
            % Get data
            dat                     = SETUP.networkDIO.data;
            % Reset data
            SETUP.networkDIO.data  	= [0 0 0];

            % Check for correct response
            if(dat(end) == 5)
                % Set variable that init is done
                SETUP.networkDIO.initDone = 1;
                % User info
                disp([' Connected to IP ' SETUP.networkDIO.odroidIP ':' num2str(SETUP.networkDIO.odroidPort)]);
                disp([' Network Box Version: ' num2str(double(dat(2))/1000)]);
                %disp([' This took: ' num2str(dat(1)/1000000) ' seconds']);
                done = 1;
                break;
            end
        end
    end

    if(done ~= 1)
        disp(' --------------------------------------------------------------------');
        disp(' There was a timeout during communication with the network IO device!');
        disp(' This is a permanent error and I give up now :(');
        disp(' ');
        disp(' Closing network connection!');
        closeNetworkIO;
        error('Please solve problem and try again!');
    end

else
    disp(' ---------------------------------------------');
    disp(' The network IO device is already initialized!');
    disp([' Connected to IP ' SETUP.networkDIO.odroidIP ':' num2str(SETUP.networkDIO.odroidPort)]);
    disp(' ---------------------------------------------');
end

