function out = bIO(pinNr, status)
% This function handles the IO communication of the IO pins on the
% Raspberry Pi 2.
% In this release only outputs are supported: output 1 to 4
%
% INPUTS
%   * pinNr          : number of a pin to read from or write to , see mySetup
%
% INPUTS-OPTIONAL
%   * status         : 0 or 1, on or off
%
% EXAMPLES
% 1.
%   bIO(1,1) % Enables pin 1
% 2.
%   bIO(2,0) % Disables pin 2
% 3.
% 	bIO(0,1) % Turns all pins on
% 4.
%   bIO(0,0) % Turns all pins off
% 5.
%   bIO([1 2 3],[1 1 0]) % Turns on pin 1 and 2 and turns off pin 3
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x
%
% See also:  feeding, punishment, startExperiment, mySetup

% VERSION HISTORY:
% Author:         Tobias Otto
% Version:        1.3
% Last Change:    18.06.2018
%
% 21.06.2017, Tobias: First draft
% 16.02.2018, Tobias: updated IO init
% 15.06.2018, Tobias: added inputs to function
% 18.06.2018, Tobias: bugfix in output case

%% Init variables
global SETUP
pinsIn  = [26, 16, 13, 12, 18];         % Pin numbers for inputs
pinsOut	= [17, 27, 22, 23, 24, 25];     % Pin numbers for outputs
mode  = 0;
out     = 0;

%% Check input arguments
if(nargin == 1)
    % Read pins
    mode  = 1;

    % Check for all on or all off feature
    if(pinNr == 0)
        % Get all pin numbers
        pinNr 	= 1:length(pinsIn);
    end
elseif(nargin == 2)
    % Write pins
    mode  = 2;

    % Check for all on or all off feature
    if(pinNr == 0)
        % Get all pin numbers
        pinNr 	= 1:length(pinsOut);
        % Define mode vector
        status = repmat(status, 1, length(pinsOut));
    end

    % Check, if length of vectors is equal
    if(length(pinNr) ~= length(status))
        disp(' ----------------------------------------------------------');
        disp(' The amount of pin numbers have to match with the entries');
        disp(' in the status vector! Please check your code and try again');
        disp(' ----------------------------------------------------------');
        error('Vector of pin number and status don''t match!');
    end
end

%% Check, if outputs are initialized
if(SETUP.raspberryPi2.initDone == 0)
    % Error handling: Get Pi revision. PTB only handles > 1 at the moment
    if(RPiGPIOMex < 2)
        disp(' -----------------------------------------------');
        disp(' Sorry, RaspberryPi models before revision 2');
        disp(' are not yet supported');
        disp(' -----------------------------------------------');
        error('Bad RaspberryPi model');
    end

    % We are not root! Need some help from our powerful command line
    % friend: Call gpio command line utility to export pins for use by us
    % without need to be root. Set them to output mode.
    for i=pinsOut
        system(sprintf('gpio export %i out', i));
        % Add some time for the system to come up
        WaitSecs(0.1);
    end

    for i=pinsIn
        system(sprintf('gpio export %i in', i));
        % Add some time for the system to come up
        WaitSecs(0.1);
    end

    % List all pins which are exported for use by non-root applications,
    % and their configuration:
    system('gpio exports');

    % Init done
    SETUP.raspberryPi2.initDone = 1;
end

%% Set or get data
if(mode == 1)
    % Read data
    for i=1:length(pinNr)
        out(i) = RPiGPIOMex(0, pinsIn(pinNr(i)));
    end
elseif(mode == 2)
    % Write data
    for i=1:length(pinNr)
        RPiGPIOMex(1, pinsOut(pinNr(i)), status(i));
    end
end