function out = getMousePos(waitTime)
% This function returns the Mouse position (x, y) after a left click on the
% stimulus presentation window. Calling getMousePos without arguments
% blocks execution until a click is done. A call with the optional argument
% waitTime exits after a left click or after the given time.
%
% Usage: out = getMousePos(duration)
%
%
% INPUTS
%   * waitTime      : The time to wait unless defined number of responses
%                     has been reached before
%
% INPUTS-OPTIONAL
%  * no optional arguments
%
%
% OUTPUTS
%  * out          :  A struct containing all possible information
%   .x              -> The X position of the point where the user clicked.
%                      Zero if the user did not click.
%   .y              -> The Y position of the point where the user clicked.
%                      Zero if the user did not click.
%   .time           -> Response time of the user in seconds
%   .escape         -> This entry is set to one, if ESC was pressed
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x

% VERSION HISTORY:
%
% Tobias Otto
% 1.7
% 21.04.2017

% 02.10.2009, Tobias: first draft
% 05.10.2009, Tobias: bugfix: release loop crashed the x,y coordinates
% 05.10.2009, Tobias: Added escape option with keyboard
% 06.10.2009, Tobias: More variables init
% 10.05.2010, Tobias: added more user info and created output struct
% 11.02.2011, Tobias: Set x,y coordinates to NaN if user does not click
% 09.01.2017, Tobias: using ~ for unused output arguments
% 21.04.2017, Tobias: changed NaN to 0 in case user didn't respond

global WINDOW

%% Init variables
buttons     = [0 0 0];
x           = 0;
y           = 0;
escapeKey   = 27;
escape      = 0;
keyCode     = uint8(zeros(1,256));
stopTime    = 0;
startTime   = 0;

if(nargin == 0)
    waitTime = inf;
end

%% Waiting for a button or until the time is over
startTime   = GetSecs;
stopTime    = startTime + waitTime;

while(all(~buttons) && stopTime > GetSecs)
    % Check mouse
    [x,y,buttons] = GetMouse(WINDOW.handle);

    % Check for escape
    [~, ~, keyCode] = KbCheck;

    if(keyCode(escapeKey) ~= 0)
        escape = 1;
        break;
    end
end

%% Compute output
% Check if user clicked, if not set x,y to zero!
% Otherwise the last click position is stored in x any y
if(all(~buttons))
    x = 0;
    y = 0;
end
out.x       = x;
out.y       = y;
out.time    = GetSecs - startTime;
out.escape  = escape;

%% Waiting for release
while(any(buttons) && escape == 0)
    [~,~,buttons] = GetMouse;
end
