function result = punishment(time, varargin)

% function [result] = punishment(time, varargin)
%
% Use this function for punishment if a task wasn't successfully finished.
% During punishment the answers of the animal are recorded and available
% in the output variable.
% This function uses three options for punishment implicitly (all are
% defined in myHardwareSetup/ myParadigmSetup)
%   * if specified in section (3) in myHardwareSetup a device is used
%   * if specified in section (6) in myParadigmSetup a stimulus is presented
%   * if nothing of the above is specified the houselight will be turned
%     off as punishment
%
% The screen is cleared, but can be kept by using an optional input
% argument (not if a punishment stimulus is shown!).
%
% INPUTS
%   * time          : Duration of punishment
%
% INPUTS-OPTIONAL
%  * dontclear      : Keeps the content on the screen.
%
% OUTPUTS
%  * result         : struct containing responses recorded during punishment
%
% EXAMPLES
% 1.
%   result = punishment(2);
%   Punishment for two seconds
%
% 2.
%  result = punishment(1.5, 'dontclear');
%  Punishment for 1.5 seconds and keeps the screen content
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x
%
% See also:  feeding, bIO

% VERSION HISTORY:
% Author:         Tobias Otto, Jonas Rose
% Version:        1.3
% Last Change:    08.01.2019
%
% 20.06.2017, Tobias: First draft for biopsy toolbox 2
% 06.11.2017, Tobias: added punishment image as option
% 14.03.2018, Tobias: removed wrong example in user info
% 08.01.2019, Lukas: Bugfix when presenting stimuli (due to changes in showStimuli)

%% We need access to the skinner box
global SETUP;
global WINDOW;

%% Init variables
clearScreen = 1;

%% Get variable input arguments to change the default values
if nargin > 1
    i=1;
    while(i<=length(varargin))
        switch lower(varargin{i})
            case 'dontclear'
                clearScreen = 0;
                i           = i+1;
            otherwise
                disp(['Unknown argument: ' varargin(i)]);
                i = i+1;
        end
    end
end

%% Clear screen
if(~isempty(WINDOW.handle) && clearScreen)
    showStimuli;
end

%% Does an extra punishment device exist?
if(~isempty(SETUP.io.punishment))
    % Use punishment device
    bIO(SETUP.io.punishment , ones(1,length(SETUP.io.punishment)));
elseif(~isempty(SETUP.punishment.img))
    % Show punishment stimulus
    showStimuli(WINDOW.punishment.img, 'rect', WINDOW.punishment.rect);
else
    % Turn off house light
    bIO(SETUP.io.houseLight , zeros(1,length(SETUP.io.houseLight)));
end

%% Call buffer function
result = keyBuffer(time);

%% Turn off punishment or turn on house light
if(~isempty(SETUP.io.punishment))
    % Use punishment device
    bIO(SETUP.io.punishment , zeros(1,length(SETUP.io.punishment)));
elseif(~isempty(SETUP.punishment.img))
    % Remove punishment stimulus
    showStimuli;
else
    % Turn on house light
    bIO(SETUP.io.houseLight , ones(1,length(SETUP.io.houseLight)));
end
