function total = switchBuffer(duration,varargin)

% function total = switchBuffer(duration, ...)
%
% Never call this function directly, internally called by keyBuffer!
%
% INPUTS
%
% INPUTS-OPTIONAL
%
% OUTPUTS
%
% EXAMPLES
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x
%
% See also: keyBuffer, touchBuffer, bIO

% VERSION HISTORY:
% Author:         Tobias Otto, Jonas Rose
% Version:        1.6
% Last Change:    21.04.2020
%
% 20.06.2017: tobias: first draft for Biopsy Toolbox 2
% 08.11.2017, Tobias: added optional input 'timeoutBad' from Jonas
% 10.11.2017, Tobias: added optional input 'feedbackBad' from Jonas
% 08.12.2017, Tobias: added option to stop switchBuffer via network
% 11.12.2017, Tobias: added option to send network ID to control computer
% 26.04.2018, Tobias: added input keys ... to output struct as additional information
% 21.04.2020, Tobias: bugfix in sound feedback

%% BUGS/ TODO
% - no known bugs

%% Init variables
global SETUP
global WINDOW
global NETWORK
time                = GetSecs;
goodKey             = 0;
badKey              = 0;
neutralKey          = 0;
numOfGoodPecks      = inf;
numOfBadPecks       = inf;
numOfNeutralPecks 	= inf;
goodPecked          = 0;
badPecked           = 0;
neutralPecked       = 0;
hSound              = [];
isSound             = 0;
addTimeBadPeck      = 0;
timeBadPeck         = 0;
remoteStop          = 0;
sendNetID           = 0;
total = struct('raw', [], 'goodKey', 0, 'badKey', 0, 'neutralKey', 0,...
    'rt', 0);

%% Empty buffer
bIO(SETUP.io.inputKeys);

%% Get variable input arguments
i=0;
while(i~=length(varargin))
    i=i+1;
    switch lower(varargin{i})
        case 'goodkey'
            % Get the good keys
            i=i+1;
            goodKey = SETUP.io.inputKeys(varargin{i});

            % Is there an optional number of good pecks specified?
            try
                if(isa(varargin{i+1},'double')) % Work with i+1, because it's only a try
                    i=i+1;
                    numOfGoodPecks = varargin{i};
                else
                    disp(' ================================================');
                    disp(' Please define a number of good pecks ');
                    disp(' as separate argument');
                    disp(' ================================================');
                    error('Please solve problem and try again');
                end
            catch
                disp(' ================================================');
                disp(' Please define a number of good pecks ');
                disp(' as separate argument');
                disp(' ================================================');
                error('Please solve problem and try again');
            end
        case 'badkey'
            % Get the bad keys
            i=i+1;
            badKey = SETUP.io.inputKeys(varargin{i});

            % Is there an optional number of bad pecks specified?
            try
                if(isa(varargin{i+1},'double'))
                    i=i+1;
                    numOfBadPecks   = varargin{i};
                else
                    numOfBadPecks	= 1;
                end
            catch
                disp(' ================================================');
                disp(' Please define a number of bad pecks ');
                disp(' as separate argument');
                disp(' ================================================');
                error('Please solve problem and try again');
            end
        case 'neutralkey'
            % Get the neutral keys
            i=i+1;
            neutralKey = SETUP.io.inputKeys(varargin{i});

            % Is there an optional number of neutral pecks specified?
            try
                if(isa(varargin{i+1},'double')) % Work with i+1, because it's only a try
                    i=i+1;
                    numOfNeutralPecks = varargin{i};
                else
                    disp(' ================================================');
                    disp(' Please define a number of neutral pecks ');
                    disp(' as separate argument');
                    disp(' ================================================');
                    error('Please solve problem and try again');
                end
            catch
                disp(' ================================================');
                disp(' Please define a number of neutral pecks ');
                disp(' as separate argument');
                disp(' ================================================');
                error('Please solve problem and try again');
            end
        case 'sound'
            % Get the sound file handle
            i       = i + 1;
            hSound  = varargin{i};
            isSound = 1;
        case 'timeoutbad'
            % increase the waiting time for each response to a badKey
            i               = i + 1;
            addTimeBadPeck  = varargin{i};
        case 'feedbackbad'
            % give a visual feedback for each response to a badKey
            i         	= i + 1;
            stimBadPeck	= varargin{i};
            try
                if(isa(varargin{i+1},'double'))
                    i         	= i+1;
                    timeBadPeck	= varargin{i};
                else
                    disp(' ================================================');
                    disp(' Please define the time in seconds to display the');
                    disp(' stimulus as separate argument');
                    disp(' ================================================');
                    error('Please solve problem and try again');
                end
            catch
                disp(' ================================================');
                disp(' Please define the time in seconds to display the');
                disp(' stimulus as separate argument');
                disp(' ================================================');
                error('Please solve problem and try again');
            end
        case 'remotestop'
            remoteStop  = 1;
        case 'sendnetid'
            sendNetID = 1;
        otherwise
            disp(' ================================================');
            disp([' Unknown argument: ' varargin(i)]);
            disp(' ================================================');
            i = i+1;
    end
end

%% Init variables again, after we know the number of keys
total.goodKey       = zeros(1,length(goodKey));
total.badKey        = zeros(1,length(badKey));
total.neutralKey 	= zeros(1,length(neutralKey));
startTime           = GetSecs;
duration            = duration+startTime;
time                = 0;

%% Start loop
while((time < duration) && goodPecked ~= numOfGoodPecks && ...
        badPecked ~= numOfBadPecks && neutralPecked ~= numOfNeutralPecks)
    %% Read from pecking keys
    peck   = bIO(SETUP.io.inputKeys);
    time    = GetSecs;

    %% Check network to stop loop
    if(remoteStop == 1)
        % Check for valid data
        data = pnet(NETWORK.con, 'read', 1, 'int8');
        % If value is valid, stop loop and return number
        if(data == 31)
            pnet(NETWORK.con, 'write', int8(32));
            break;
        end
    end

    %% Is peck equal to any of the elements in SETUP.iow.inputKeys?
    if(any(SETUP.io.inputKeys == peck))
        total.raw           = cat(1,total.raw,[GetSecs-startTime peck]);
        total.goodKey       = total.goodKey + (goodKey==peck);
        total.badKey        = total.badKey + (badKey==peck);
        total.neutralKey    = total.neutralKey + (neutralKey==peck);
        % Helper variables for the while loop to exit
        goodPecked          = sum(total.goodKey);
        badPecked           = sum(total.badKey);
        neutralPecked       = sum(total.neutralKey);

        %% Check for sound feedback
        if(isSound == 1)
            controlSound(hSound);
        end

        %% Increment the duration on badKey responses
        if(any(badKey == peck) && addTimeBadPeck)
            duration = duration + addTimeBadPeck;
        end

        %% Show a negative feedback stimulus
        if(timeBadPeck ~= 0 && any(badKey == peck.index))
            % Show the negative feedback image
            Screen('DrawTexture', WINDOW.handle, stimBadPeck.texture, [], WINDOW.rect);
            Screen('Flip', WINDOW.handle);

            % Wait given time
            WaitSecs(timeBadPeck);

            % Recover screen content
            % Finally plot image
            for i=1:size(WINDOW.stimulus.onScreen,1)
                Screen('DrawTexture', WINDOW.handle, WINDOW.stimulus.onScreen(i, 1), [], WINDOW.stimulus.onScreen(i, 2:end));
            end
            Screen('Flip', WINDOW.handle);
        end

        %% Do we have to send the network ID
        if(sendNetID == 1)
            sendNetID = 2;
        end
    end
end

%% Get stop time and check output struct
total.rt  = GetSecs-startTime;

% Add the input key numbers to output struct
total.definedGoodKey             = goodKey;
total.definedBadKey              = badKey;
total.definedNeutralKey          = neutralKey;

% Add zeros for no answer
if(isempty(total.raw))
    total.raw   = [0 0];
end

%% Send network ID
if(sendNetID == 2)
    pnet(NETWORK.con, 'write', int8(NETWORK.netID));
end

