function num = connect2Host(singleHosts)
% This is the main function that handles the communication to the
% experimental computer. This function needs a proper edited myHardwareSetup.m
% file! Make sure that the IP address of all Raspberrys are entered in section
% 10 without exception. If the IP address of a Raspberry is not entered in
% myHardwareSetup.m the Raspberry can't be used.
%
% If you plan to start experiments on a Raspberry remotely, make sure that all
% experimental files (including initOTBR and myHardwareSetup and myparadigmSetup)
% are copied to the Raspberry using sendExperiment2Host.
%
% This function returns a hanlde for each successfully initialized Raspberry. After initialization
% only use this handle to identify each singe Raspberry
%
% Usage: h = connect2Host(num);     % connect to all or selected hosts defined in myHardwareSetup
%
% INPUTS-OPTIONAL
%   * num	: Number of Raspberrys you want to connect to. Has to be defined in myHardwareSetup.m
%
% OUTPUTS
%  * h      :  handle for each Raspberry. After initialization only use the handles to identify the
%              Raspberrys
%
% EXAMPLE
%   h = connect2Host;           % Init all Raspberrys defined in myHardwareSetup.m
%
%   h = connect2Host(3);        % Init the third Raspberry defined in myHardwareSetup.m
%
%   h = connect2Host([1 3]);	% Init the first and third Raspberry defined in myHardwareSetup.m
%
% Expert mode: This function is able to connect to single hosts that are
% not mentioned in myHardwareSetup.m The optional argument is a cell that contains
% the IP addresses e.g. {'192.168.0.50', '192.168.0.51'}
% Experimental and should only be used during development!
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x

% Tobias Otto
% 1.10
% 26.05.2021

% 03.11.2017, Tobias: first draft
% 06.11.2017, Tobias: changed name
% 04.12.2017, Tobias: changed code to work with more than one client
% 06.12.2017, Tobias: changed handshakes during network init
% 07.12.2017, Tobias: more error handling
% 08.12.2017, Tobias: added exchange of networkID for client computer
% 03.03.2018, Tobias: IP addresses and handles are now in SETUP
% 16.03.2018, Tobias: Opening socket now 5 times in a row to wait for client
% 20.03.2018, Tobias: waiting max 90 seconds for a host added optional argument to connect to single clients
% 07.05.2021, Tobias: added more user info; added option to connect to specific Raspberry
% 26.05.2021, Tobias: setting computers time on Raspberry after init automatically

%% Init variables
global SETUP
remotePort      = 5000;
netTimeout      = 0.1;
maxTime2Wait    = 60;  % Timeout in s for a raspberry to resond

%% Check input arguments
if(nargin == 0)
    hosts = SETUP.raspberry.ip;
else
    if(iscell(singleHosts))
        % Hosts are defined in cell with IP address
        hosts = singleHosts;
    elseif(isnumeric(singleHosts))
        hosts = SETUP.raspberry.ip(singleHosts);
    else
        error('Enter number of Raspberrys you want to connect to. Has to be defined in myHardwareSetup.m');
    end
end

%% Start main loop to init all raspberries
for i=1:length(hosts)
    %% Get variables for each client
    host    = hosts{i};
    num(i)	= SETUP.raspberry.num + 1;

    %% Check input arguments
    if(~ischar(host))
        error('Argument host has to be a string containing the IP address: e.g. 192.168.0.20');
    end

    if(sum(host == '.') ~= 3)
        error('Argument host has to be a string containing the IP address: e.g. 192.168.0.20');
    end

    %% Open socket and wait for host to connect
    % Default time out for pnet is approx. 20 seconds
    con     = -1;
    maxTime = GetSecs + maxTime2Wait;
    while(con == -1 && GetSecs <= maxTime)
        disp(['Opening socket for host ' host ' ... still ' num2str((maxTime-GetSecs)) 's for host to respond']);
        drawnow
        con = pnet('tcpconnect', host, remotePort);
    end

    %% Configure host
    if(con >= 0)
        disp(['Connecting to host ' host ' ...']);

        % Set time outs
        pnet(con,'setreadtimeout', netTimeout);
        pnet(con,'setwritetimeout', netTimeout);

        % Wait for handshake information
        data = [];
        time = GetSecs+2;
        while(isempty(data) && GetSecs < time)
            data = pnet(con, 'read', 1, 'int8');
        end

        % Check if we got the "magic number"
        if(isempty(data) || data ~= 111)
            error('First handshake to remote host incorrect');
        else
            % Send data in return --> a known number and the ID of the remote
            % client (used in other functions on the client)
            pnet(con, 'write', [int8(112) int8(num(i))]);

            % Wait for second handshake information
            data = [];
            time = GetSecs+2;
            while(isempty(data) && GetSecs < time)
                data = pnet(con, 'read', 2, 'int8');
            end

            if(data(1) ~= 112 || data(2) ~= num(i))
                error(['Second handshake to remote host incorrect! data = ' num2str(data)]);
            else
                % User info
                disp(['Ready to go with host ' host ' and client ID ' num2str(num(i))]);
            end
        end
    else
        error(['Couldn''t establish a connection to host: ' host ' ! Please start connect2ControlComputer on network client']);
    end

    %% Save connection handle to global struct
    SETUP.raspberry.num                         = num(i);
    SETUP.raspberry.host(num(i)).remotePort 	= remotePort;
    SETUP.raspberry.host(num(i)).con          	= con;
    SETUP.raspberry.host(num(i)).host           = host;
    SETUP.raspberry.host(num(i)).active         = 1;
    SETUP.raspberry.host(num(i)).netID          = num(i);
    SETUP.raspberry.host(num(i)).netTimeout     = netTimeout;

    %% Set time on raspberrys
    % Set time on remote host (valid for Raspberry only)
    setTime = sprintf('system(''sudo date -s ''''%s'''''')', datestr(now,'YYYY-mm-DD HH:MM:SS'));
    sendCommand2Host(num(i), setTime);

    %% Wait
    WaitSecs(0.2);
end
