function disconnectFromHost(h, varargin)
% This function closes the connection from the experimental computer to control computer(s).
% A call without arguments closes the connection to all Raspberrys defined in myHardwareSetup.m
% disconnectFromHost allows multiple optional arguments to disconnect from soecified hosts and to
% reboot or shutdown each Raspberry individually.
%
% Usage: disconnectFromHost(h, varargin)
%
% Example: disconnectFromHost(h);   % Disconnects from Raspberry which handle is stored in h
%
% Example: disconnectFromHost(h, 'reboot');     % Disconnects from Raspberry and reboots properly
%
% Example: disconnectFromHost(h, 'shutdown'); 	% Disconnects from Raspberry and shuts down properly
%
% INPUTS-OPTIONAL
%  * h              :  A handle that specifies the Raspberrys
%                      The handle is generated by the function connect2Host!
%  * 'reboot'       :  Reboots the Raspberry client
%  * 'forceReboot'	:  Reboots the Raspberry client even if the application on the Raspberry hangs
%                      or a new connection isn't possible
%  * 'shutdown'   	:  Shuts the Raspberry client with handle down!
%  * 'forceShutdown':  Shuts down the Raspberry client even if the application on the Raspberry
%                      hangs or a new connection isn't possible
%
% This function calls plink.exe an external program from the PuTTY MSI installer distribution.
% See http://www.chiark.greenend.org.uk/~sgtatham/putty/ for more details!
%
% Usage: con = disconnectFromHost(varargin)
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x

% Tobias Otto
% 1.10
% 20.05.2021

% 06.11.2017, Tobias: first draft
% 06.12.2017, Tobias: bugfix in looking for valid connections; error handling
% 03.03.2018, Tobias: ip addresses and handles now in SETUP struct
% 05.03.3018, Tobias: added shutdown option
% 19.03.2018, Tobias: added forceShutdown option
% 20.03.2018, Tobias: added option to shut down clients without a valid connection changed internal handling of optional arguments
% 21.03.2018, Tobias: bugfix for hard reboot
% 07.05.2021, Tobias: more user info; added option to reboot selected Raspberrys
% 11.05.2021, Tobias: more userinfo; added forceShutdown
% 12.05.2021, Tobias: bugfix in data type for singleHosts
% 20.05.2021, Tobias: bugfix in forceshutdown

%% Init variables
global SETUP
go              = 0;
reboot          = 0;
shutdown        = 0;
forceReboot     = 0;
forceShutdown   = 0;

% Check for defined hosts
% Check if we habe an ID or an IP address
if(exist('h', 'var') == 1)
    singleHosts   	= SETUP.raspberry.host(h).host;
    singleHosts   	= {singleHosts};  % Copy to cell to be compatible with old code :(
else
    singleHosts   	= {};
end

%% Check optional input argumemts
i=0;
while(i~=length(varargin))
    i=i+1;
    switch lower(varargin{i})
        case 'reboot'
            reboot = 1;
        case 'shutdown'
            shutdown = 1;
        case 'forcereboot'
            forceReboot = 1;
        case 'host'
            i=i+1;
            singleHosts = varargin{i};
        case 'forceshutdown'
            forceShutdown = 1;
    end
end

%% Check input arguments again
if(isempty(singleHosts))
    hosts = SETUP.raspberry.ip;
else
    if(iscell(singleHosts))
        hosts = singleHosts;
    else
        error('The optional argument has to be a cell array: e.g. {''192.168.0.50'', ''192.168.0.51''}');
    end
end

%% Start main loop to close connections
for j=1:length(hosts)
    %% Set variables for loop
    host = hosts{j};

    %% Check input arguments
    if(~ischar(host))
        error('Argument host has to be a string containing the IP address: e.g. 192.168.0.20');
    end

    if(sum(host == '.') ~= 3)
        error('Argument host has to be a string containing the IP address: e.g. 192.168.0.20');
    end

    %% Find connection
    for i=1:SETUP.raspberry.num
        if(SETUP.raspberry.host(i).active == 1 && strcmpi(SETUP.raspberry.host(i).host, host))
            go = 1;
            break;
        end
    end

    %% Check, if we found a valid connection
    if(go == 1)
        disp(['Closing connection to ' host ' ...']);
    else
        disp(['Couldn''t find a valid connection to host: ' host ' ! :(']);
        disp('Trying to reboot host anyway ... check client!');

        % Set variables to force a reboot via SSH
        forceReboot = 1;
        reboot      = 0;
        shutdown    = 0;
    end

    %% Send message to client to quit the connection
    if(reboot == 1)
        % Clean reboot
        pnet(SETUP.raspberry.host(i).con, 'write', int8(100));

    elseif(shutdown == 1)
        % Shut down client
        pnet(SETUP.raspberry.host(i).con, 'write', int8(101));

    elseif(forceReboot == 1)
        % Force reboot by connection via SSH
        rebootRaspberry(host);
    elseif(forceShutdown == 1)
        % Force shutdown by connection via SSH
        shutdownRaspberry(host);
    end

    %% Close network connection if we have any
    if(go == 1)
        pnet(SETUP.raspberry.host(i).con, 'write', int8(99));

        % Close connection
        pnet(SETUP.raspberry.host(i).con, 'close');
    end

    %% Delete connection handle from global struct if we have any
    if(go == 1)
        SETUP.raspberry.host(i).con     = [];
        SETUP.raspberry.host(i).active  = 0;
    end
end
