function sendCommand2Host(hostHandle, command, wait4Feedback)
% This function sends commands to the remote computer that will be
% executed. Make sure that the paradigm files exist on the remote computer
% before calling the files.
%
% By default the function does not wait for feedback from host.
% Set wait4Feedback to 1, wait for feedback but not display on local host.
% Set wait4Feedback to 2, wait for feedback and display on local host.
%
% Usage:    sendCommand2Host(hostHandle, command, wait4Feedback)
%
% Example:  sendCommand2Host(h, 'disp(''Hello World'');');
%           % Sends a command to a Raspberry whose handle is stored in h
%           % The handle is generated by the function connect2Host!
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.

% Tobias Otto
% 1.5
% 11.05.2021

% 06.11.2017, Tobias: first draft
% 04.12.2017, Tobias: added optional argument not to wait for feedback from remote host
% 01.03.2018, Jonas : can now send command to multiple hosts
% 03.03.2018, Tobias: ip addresses and handles now in SETUP struct
% 05.03.2018, Tobias: added semicolon to the end of every command (otherwise octave might block execution on raspberry!)
% 11.05.2021, Tobias: more user info

%% Init variables
global SETUP

go      = [];
con     = zeros(size(hostHandle));
data    = [];
num     = 11;

% Check inputs
if(nargin == 3)
    num = 10;
else
    wait4Feedback = 0;
end

%% Send command to experimental computer
% Indicate a command by sending a 10 or 11 first and wait for a reply
% 10 indicates that we want a network reply and 11 that no feedback is fine
for i=1:length(hostHandle)
    % the handle to each host
    con(i) = SETUP.raspberry.host(hostHandle(i)).con;
    pnet(con(i), 'write', int8(num));
end

% all hosts that have not yet replied
busyHosts = 1:length(hostHandle);
i=1;
while ~isempty(busyHosts)
    % read data from each host
    data = pnet(con(busyHosts(i)), 'read', 1, 'int8');
    if ~isempty(data)
        % Send data, if reply was successful
        if(data == 11)
            pnet(con(busyHosts(i)), 'printf', [command ';\n']);
        else
            error(['Host doesn''t answer handshake request! data is ' num2str(data)]);
        end
        % remove current host from list
        busyHosts(i) = [];
    end
    i = i+1;

    % start list from beginning
    if i>length(busyHosts)
        i = 1;
    end
end


%% DEBUG user info
if(num == 10)
    % all hosts that have not yet replied
    busyHosts = 1:length(hostHandle);
    i=1;
    while ~isempty(busyHosts)
        % read data from each host
        go = pnet(con(busyHosts(i)), 'read', 1, 'int8');
        if ~isempty(go)
            if(go == 13)
                warning(['Command "' command '" wasn''t executed successfully :( with feedback on machine ' ...
                    num2str(SETUP.raspberry.host(hostHandle(busyHosts(i))).host)]);
            elseif(go == 12)
                if wait4Feedback==2
                    disp(['Host: ' SETUP.raspberry.host(hostHandle(busyHosts(i))).host ' - Command "' command '" was executed successfully with feedback:)']);
                end
                busyHosts(i) = [];
            else
                warning(['Something went horribly wrong! Go is ' num2str(go)]);
            end
        end
        i = i+1;
        % start list from beginning
        if i>length(busyHosts)
            i = 1;
        end
    end
end