function sendExperiment2Host(host, localFolder, keepRemoteFiles)
% This function copies a complete experiment to a remote host on the
% network! The remote host (e.g. a Raspberry Pi3) has to be accessible
% via a windows share that is named "OTBRToolboxShare". The experimental
% files are copied to a folder "Experiment" within the share. Please make
% sure the folder exists on the remote computer.
%
% Before the local files are copied to the remote host all files on the
% share on the remote host will be deleted! Folders however will be kept
% since they may contain result files.
% Make sure to delete folders manually if not needed
%
% This function needs three arguments:
% * host        --> a string containing the IP address of the remote host
% * localFolder --> the local folder with *all* necessary files to run the
%                   experiment as string. All files and folders are copied
%                   recursively to the remote host!
%                   The IKN Toolbox itself is not needed
% * keepRemoteFiles --> set to 1 to keep the files on the remote system.
%                   By default all files are deleted before new files are
%                   copied
%
% Usage:    sendExperiment2Host(host, localFolder, keepRemoteFiles)
%
% Example:  sendExperiment2Host(h, 'c:\localExperimentFolder');
%           % Sends data from c:\localExperimentFolder to the Raspberry whose handle is stored in h
%           % and deletes the previously copied files on the Raspberry.
%           % The handle is generated by the function connect2Host!
%
% Example:  sendExperiment2Host(h, 'c:\localExperimentFolder', 1);
%           % Sends data from c:\localExperimentFolder to the Raspberry whose handle is stored in h
%             and keeps the previously copied files on the Raspberry
%           % The handle is generated by the function connect2Host!
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x

% Tobias Otto
% 1.7
% 11.05.2021

% 11.10.2017, Tobias: first draft
% 30.11.2017, Tobias: changed name of remote folder
% 06.12.2017, Tobias: using folder Experiment on remote machine
% 13.09.2018, Tobias: changed name of share
% 31.01.2019, Tobias: corrected documentation
% 05.02.2019, Tobias: keeping Experiment folder
% 10.05.2021, Tobias: changed host to Raspberry ID
% 11.05.2021, Tobias: bugfixing and more user info

%% Init variables
global SETUP
go = 1;

% Check if we habe an ID or an IP address
if(~ischar(host))
    host = SETUP.raspberry.host(host).host;
end

remoteFolder = ['\\' host '\OTBRToolboxShare\Experiment'];

%% Check input arguments
if(nargin == 2)
    keepRemoteFiles = 0;
end

if(nargin < 2)
    disp(' -------------------------------------------------------------------------------------------');
    disp(' This function needs at least two input arguments: host ip and local folder containing the experiment');
    go = 0;
end

if(~ischar(host))
    disp(' -------------------------------------------------------');
    disp(' Please enter IP address as string e.g. ''192.168.0.20''');
    go = 0;
end

if(~ischar(localFolder))
    disp(' -------------------------------------------------------');
    disp(' Please enter local folder as string e.g. ''d:\myExperiment''');
    go = 0;
end

if(exist(localFolder, 'dir') ~= 7)
    disp(' -------------------------------------------------------');
    disp([' The folder: ' localFolder]);
    disp(' doesn''t exist on this system!');
    go = 0;
end

if(go == 0)
    error(' Please solve problems and try again!');
end

%% Delete files
if(keepRemoteFiles ~= 1)
    if(exist(remoteFolder, 'dir') == 7)
        fprintf(' Deleting files on remote system %s ... ', host);

        % Get files and folders in "Experiment" share
        a = dir([remoteFolder filesep '*']);

        % Delete files and folders
        for i=3:length(a)
            if(a(i).isdir == 0)
                delete([remoteFolder filesep a(i).name]);
            else
                [status, message] = rmdir([remoteFolder filesep a(i).name], 's');
                if(status ~= 1)
                    fprintf('\n\n');
                    disp(message);
                end
            end
        end

        fprintf('done!\n');
    end
    if(exist(remoteFolder, 'dir') ~= 7)
        fprintf(' Creating new experiment folder ... ');
        [status, message] = mkdir(remoteFolder);
        if(status ~= 1)
            fprintf('\n\n');
            disp(message);
        end
        fprintf('done!\n');
    end
end

%% Copy files and folders
fprintf(' Copying files to remote system %s. This might take some time ... ', host);
[status, message] = copyfile(localFolder, remoteFolder);
if(status ~= 1)
    fprintf('\n\n');
    disp(message);
end
fprintf('done!\n');
