classdef clientPanel < handle

    % Class clientPanel
    %
    % cPnl = clientPanel(client,pos)
    %
    % Use objects of this class to monitor (or control) remote (or local)
    % paradigms. Each clientPanel is a ui-panel that can be freely
    % placed within a figure. Within each clientPanel are several subPanles
    % with different properties, subPanels can be chosen and placed freely.
    %
    % Typically don't call this directly but use: openCPanelFig.m
    %   Otherwise first open clientPanel, then add subPanels, options are:
    %     infoSPnl    : (required) status info and control of clientPanel
    %     perfSPnl    : Performance text and plot
    %     camSPnl     : IP-cam or webcam
    %     parViewSPnl : The paradgim form the perspective of the subject
    %                 : Requires paradigm on clientPanel-machine, including
    %                 : a folder '.\Stimuli' in the same directory, must
    %                 : contain all stimuli and only stimuli.
    %     lclCtrlSPnl : Control remote paradigm over LAN
    %     lanCtrlSPnl : Control local paradigm on this machine
    %   All subpanles accept the optional input position [l b w h]
    %
    %   ClientPanels require a network connection to the client that is to
    %   be monitored. Information must be send actively form the clients
    %   and if remote paradigms are to be controlled, listening for
    %   commands from the clientPanel must be actively implemented.
    %
    %   See the following functions for client-machines:
    %     send2ClientPanel      : Sends data to the clientPanel
    %                           : perfSPnl reads parameter-value pairs
    %               trlnr       : current trial number
    %               maxTrls     : max number of trials in session
    %               duration    : current duration of session
    %               trlCorr     : logical vec, all trials in session, corr?
    %               trlInCorr   : logical vec, all trials in session, incorr?
    %               trlNoRsp    : logical vec, all trials in session, response?
    %                           : parViewSPnl reads parameter-value pairs
    %               showStimuli : images (loaded with loadImage): vector of keys
    %                           : this input is identical to showstimuli(im,key)
    %               infotext1   : custom text in line 1
    %               infotext2   : custom text in line 2
    %               infotext3   : custom text in line 3
    %     connect2ClientPanel   : Establish the connection to clientPanel
    %     listen2ClientPanel    : Read commands from clientPanel
    %
    % INPUTS:
    % client: Struct    : The client that is to be monitored (controlled)
    %        .name       : nameString
    %        .remoteIP   : IP address of remote client
    %        .camera     : OPTIONAL if IP-cam (webcam) is to be displayed
    %        .camera.device  : full URL (or webcam name)
    %        .camera.userName: IP-cam only
    %        .camera.password: IP-cam only
    % pos   : [l b w h] : location of the uiPanel within the figure
    %
    %
    % Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
    % Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
    % Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
    %
    % Please cite the OTBR Toolbox where this function is used.
    % Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023).
    %               https://doi.org/10.3758/s13428-023-02199-x

    % Author    : Jonas Rose/ Tobias Otto
    % Version   : 0.6
    % Date      : 25.03.2019

    % 31.01.2018, Tobias: cBck_prepBtn: added check for valid user input
    % 13.02.2019, Tobias: added functionality to lanControl including necessary changes here and there
    % 14.02.2019, Tobias: changed lan separator to avoid erros for win stimuli
    % 22.02.2019, Jonas : allow rtsp as ip cam url
    %                   : added clearing of screen when 'showStimuli',[],[]
    %                   : added handle to plotting axes for lines
    % 23.02.2019, Jonas : fixed type(?) in regular expression '$'
    %                   : fixed bug in showStimuli,[],[] - isnan
    %                   : regexp doesn't work with $ used ~ now
    % 21.03.2019, Tobias: allowing multiple stimuli to be presented
    % 25.03.2019, Tobias: added file check for loading image files

    %   TODO:
    % * restart button does not work properly with lan (should maybe not
    %   trigger start or the start-listeners need to check if panel is running)
    %   (think about what can be private)
    % * option to relay the network info to another host? bytesAvailableFcn
    % * checks for: connection/ user input/ fcn calls
    % * Lan ctrl Local ctrl
    % * make sure that restarter is not simply duplicating ....
    % * buffer size
    % * full DOCUMENTATION...examples
    % * catch statement in case the webcam doesn't work


    properties
        pos         % position in the clientPanel in the figure
        client      % properties of client: name, ip & port; ip-object
        uiPnl       % this is the main client-panel object
        subPnl      % all subpanels: ctrl/ rmtCtrl/ lclCtrl/ cam/ perf/ par
        button      % all buttons (start, abort, prep, custom)
        userData    % user input
        lanData     % data that came from setup
        listeners   % all event-listeners
    end

    events
        userDone    % user has selected the data
        newData     % new data came over lan
        pause       % pause the panel
        start       % (re-)start the panel
        kill        % delete all panel components (also used for reboot)
        reset       % resets all subpanels to initialization-values
    end

    methods
        %% CONSTRUCTOR
        function [pnl] = clientPanel(client, pos)
            if nargin==1
                pnl.pos     = [0 0 1 1];
            else
                pnl.pos     = pos;
            end
            pnl.uiPnl   = uipanel('position',pnl.pos,'title',client.name,...
                'backgroundcolor','w');
            pnl.client  = client;

            %% network connection for monitoring
            % generate local port from remote ip - unique and predictable for setup
            ind = strfind(pnl.client.remoteIP,'.');
            pnl.client.localPort = ...
                str2double(pnl.client.remoteIP(ind(end)+1:end)) + 5500;
            % generate the udp object
            pnl.client.lanHand = udp(pnl.client.remoteIP, 5500, ...
                'localport', pnl.client.localPort,'inputbuffersize',8192);
            % execute bytes available function when terminator is encoutered
            set(pnl.client.lanHand,'BytesAvailableFcnMode','terminator');
            % define the terminator
            set(pnl.client.lanHand,'terminator', char(10));
            % set the bytes available function for udp-object
            set(pnl.client.lanHand, 'BytesavailableFcn',@udpBytesAvail);
            % listeners to pause/ (re-)start/ delete LanInfo-listener
            pnl.listeners.pauseLan      = addlistener(pnl,'pause', @pauseLan);
            pnl.listeners.startLan      = addlistener(pnl,'start', @startLan);
            pnl.listeners.deleteLan     = addlistener(pnl,'kill', @pauseLan);
            pnl.listeners.resetLan      = addlistener(pnl,'reset', @resetLan);

            % functions to pause (re-)start the udp lan info
            function [] = pauseLan(~,~)
                % close the udp client connection
                fclose(pnl.client.lanHand);
            end
            function [] = startLan(~,~)
                % open the connection
                fopen(pnl.client.lanHand);
            end
            function [] = resetLan(~,~)
                % close the udp client connection
                fclose(pnl.client.lanHand);
                % open the connection
                fopen(pnl.client.lanHand);
            end

            %% user-selected data
            % Listener object is notified after user selected data
            pnl.listeners.pnlUser = addlistener(pnl,'userDone',@hasUserData);
            % this will execute after the user provided data
            function [] = hasUserData(~,~)
                % make all prep buttons invisible
                for i=1:length(pnl.button.prep)
                    set(pnl.button.prep(i),'visible','off');
                end
                % start running the panel
                notify(pnl,'start');
            end

            %% lan/udp bytes availabe function
            function [] = udpBytesAvail(~,~)
                % do not show stimuli unless set
                pnl.lanData.parView.showStim = false;
                % read the data fom the udp buffer
                pnl.lanData.rawPacket = char(fread(pnl.client.lanHand)');
                % get rid of terminator
                dat = pnl.lanData.rawPacket(1:end-1); % get rid of terminator
                % split into parameter-Value pairs
                dat     = regexp(dat,'@','split');
                dat(1)  = []; % get rid of '@'
                % assign the data to appropriate variables
                i=1;
                while i<=length(dat)
                    paramVal = regexp(dat{i}, '~', 'split');
                    switch lower(paramVal{1})
                        case 'trlnr'
                            pnl.lanData.perf.trlNr = str2double(paramVal{2});
                            i = i+1;
                        case 'maxtrls'
                            pnl.lanData.perf.maxTrls = str2double(paramVal{2});
                            i = i+1;
                        case 'duration'
                            pnl.lanData.perf.duration = str2double(paramVal{2});
                            i = i+1;
                        case 'trlcorr'
                            % second value is the length of the vector
                            len = str2double(paramVal{2});
                            snipLen = 52;
                            pnl.lanData.perf.trlCorr = [];
                            for j=3:length(paramVal)
                                if j==length(paramVal)
                                    % length of last block depends on number of trials
                                    snipLen = mod(len,snipLen);
                                end
                                tmp = dec2bin(str2double(strrep(paramVal{j},' ','')))=='1';
                                pnl.lanData.perf.trlCorr = [...
                                    pnl.lanData.perf.trlCorr ...
                                    false(1,snipLen-length(tmp)) tmp];
                            end
                            i = i+1;
                        case 'trlincorr'
                            % second value is the length of the vector
                            len = str2double(paramVal{2});
                            snipLen = 52;
                            pnl.lanData.perf.trlInCorr = [];
                            for j=3:length(paramVal)
                                if j==length(paramVal)
                                    % length of last block depends on number of trials
                                    snipLen = mod(len,snipLen);
                                end
                                tmp = dec2bin(str2double(strrep(paramVal{j},' ','')))=='1';
                                pnl.lanData.perf.trlInCorr = [...
                                    pnl.lanData.perf.trlInCorr ...
                                    false(1,snipLen-length(tmp)) tmp];
                            end
                            i = i+1;
                        case 'trlnorsp'
                            % second value is the length of the vector
                            len = str2double(paramVal{2});
                            snipLen = 52;
                            pnl.lanData.perf.trlNoRsp = [];
                            for j=3:length(paramVal)
                                if j==length(paramVal)
                                    % length of last block depends on number of trials
                                    snipLen = mod(len,snipLen);
                                end
                                tmp = dec2bin(str2double(strrep(paramVal{j},' ','')))=='1';
                                pnl.lanData.perf.trlNoRsp = [...
                                    pnl.lanData.perf.trlNoRsp ...
                                    false(1,snipLen-length(tmp)) tmp];
                            end
                            i = i+1;
                        case 'subject'
                            pnl.lanData.subj = paramVal{2};
                            i = i+1;
                        case 'paradigm'
                            pnl.lanData.paradigm = paramVal{2};
                            i = i+1;
                        case 'showstimuli'
                            % show stimuli unless set
                            pnl.lanData.parView.showStim = true;
                            pnl.lanData.parView.imgs = ...
                                regexp(paramVal{2},';','split');
                            pnl.lanData.parView.imgs{end} = [];
                            pnl.lanData.parView.keys = ...
                                sscanf(paramVal{3}, '%f')';
                            i = i+2;
                        case 'infotext1'
                            pnl.subPnl.info.line1Text = paramVal{2};
                            i = i+1;
                        case 'infotext2'
                            pnl.subPnl.info.line2Text = paramVal{2};
                            i = i+1;
                        case 'infotext3'
                            pnl.subPnl.info.line3Text = paramVal{2};
                            i = i+1;
                        otherwise
                            warning(['Unrecognized parameter :' paramVal{1}])
                            break
                    end
                end
                % Network callback function updates this data
                notify(pnl,'newData');     % this will update the figures
            end
        end

        %% DESTRUCTOR
        % destroy one particular client panel for good
        function [] = destroy(pnl)
            % event to delete all panel components
            notify(pnl,'kill');
            % find and delete all listeners
            lstnrs = fieldnames(pnl.listeners);
            for i=1:length(lstnrs)
                delete(pnl.listeners.(lstnrs{i}));
            end
            delete(pnl.uiPnl);
        end

        %% Restarter
        % restart one particular client panel and re-open it clean
        function [] = restart(pnl)
            % tell everybody to re-start
            notify(pnl,'reset');
        end

        %% method for data-forwarding
        function [] = forwardLan(pnl, lclIp, rmtIp)
            % allows to define a remote client for forwarding incoming data
            % this is implemented in analogy to the panles, even though it
            % does not have a physical representation
            pnl.client.localIP             = lclIp;
            pnl.client.forward.remoteIP    = rmtIp;
            % find remote port (based on local IP) and open connection
            [pnl.client.forward.lanHand, pnl.client.forward.remotePort] = ...
                connectUdp2Monitor(rmtIp, lclIp);
            % add a listener that forwards new data on arrival
            pnl.listeners.pauseFwdLan   = addlistener(pnl,'pause',@pauseFwdLan);
            pnl.listeners.startFwdLan   = addlistener(pnl,'start',@startFwdLan);
            pnl.listeners.deleteFwdLan  = addlistener(pnl,'kill',@pauseFwdLan);
            pnl.listeners.newDataFwdLan = addlistener(pnl,'newData',@sendFwdLan);
            % inline functions
            function [] = sendFwdLan(~,~)
                % write the new raw data packet
                pnet(pnl.client.forward.lanHand, 'write',...
                    pnl.lanData.rawPacket);
                pnet(pnl.client.forward.lanHand, 'writepacket');
            end
            % functions to pause (re-)start the forward udp lan info
            function [] = pauseFwdLan(~,~)
                % close the udp client connection
                fclose(pnl.client.forward.lanHand);
            end
            function [] = startFwdLan(~,~)
                % open the connection
                fopen(pnl.client.forward.lanHand);
            end
        end

        %% SUBPANELS ------------------------------------------------------
        %% Subpanel Panel Info
        function [] = infoSPnl(pnl,pos)
            if nargin==1
                pos = [0 .35 .35 .3];
            end
            pnl.subPnl.info.line1Text = '';
            pnl.subPnl.info.line2Text = '';
            pnl.subPnl.info.line3Text = '';

            pnl.subPnl.info.pos = pos;
            pnl.subPnl.info.pnl = uipanel('parent', pnl.uiPnl, ...
                'position',pos, 'backgroundcolor','w', 'title', 'Info');% 'bordertype','none'

            % Text
            axes('parent',pnl.subPnl.info.pnl, ...
                'position',[0 0 1 .5],'visible','off');
            pnl.subPnl.info.line1 = text(.02,1,'Welcome to the OTBR Toolbox',...
                'horizontalalignment','left','verticalalignment','top',...
                'fontunits','normalized','fontsize',.15);
            pnl.subPnl.info.line2 = text(.02,.65,' ',...
                'horizontalalignment','left','verticalalignment','top',...
                'fontunits','normalized','fontsize',.15);
            pnl.subPnl.info.line3 = text(.02,.3,' ',...
                'horizontalalignment','left','verticalalignment','top',...
                'fontunits','normalized','fontsize',.15);

            % include the Buttons
            pausePnlButton(pnl, pnl.subPnl.info.pnl);
            removePnlButton(pnl, pnl.subPnl.info.pnl);
            restartPnlButton(pnl, pnl.subPnl.info.pnl);

            % listener to reset info panel
            pnl.listeners.resetInfo = addlistener(pnl,'reset',@resetInfo);
            % function to reset panel through listener
            function [] = resetInfo(~,~)
                infoSPnl(pnl,pnl.subPnl.info.pos);
            end

            % pick up from LAN, callback/ listener
            % listen for lanData
            pnl.listeners.perfLan  = addlistener(pnl,'newData',@infoNewData);

            % listen-function tied to lanData, updates performance text and figure
            function [] = infoNewData(~,~)
                % Update text
                pnl.subPnl.info.line1.String = pnl.subPnl.info.line1Text;
                pnl.subPnl.info.line2.String = pnl.subPnl.info.line2Text;
                pnl.subPnl.info.line3.String = pnl.subPnl.info.line3Text;
            end
        end

        %% SubPanel Local Setup Control
        function [] = lclCtrlSPnl(pnl,pos)
            if nargin==1
                pos = [0 0 .35 .35];
            end
            pnl.subPnl.lclCtrl.pos = pos;
            % open the lan remote-control subpanel
            pnl.subPnl.lclCtrl.pnl = uipanel('parent', pnl.uiPnl, ...
                'position',pos, 'backgroundcolor','w', ...
                'title', 'Local Control'); %'bordertype','none'
            % include the Buttons
            prepButton(pnl, pnl.subPnl.lclCtrl.pnl);
            startButton(pnl, pnl.subPnl.lclCtrl.pnl);
            abortButton(pnl, pnl.subPnl.lclCtrl.pnl);

            %restartButton(pnl, pnl.subPnl.lclCtrl.pnl); % maybe just add a feeding-button?
            % listener to reset LocalControl
            pnl.listeners.resetLclCtrl  = addlistener(pnl,'reset',@resetLclCtrl);
            function [] = resetLclCtrl(~,~)
                % restart lan control panel
                lanCtrlSPnl(pnl,pnl.subPnl.lclCtrl.pos);
            end
        end

        %% SubPanel Remote Setup Control
        function [] = lanCtrlSPnl(pnl,pos)
            if nargin==1
                pos = [0 0 .35 .35];
            end
            pnl.subPnl.lanCtrl.pos = pos;
            % open the lan remote-control subpanel
            pnl.subPnl.lanCtrl.pnl = uipanel('parent', pnl.uiPnl, ...
                'position',pos, 'backgroundcolor','w',...
                'title', 'Lan Control');%'bordertype','none'

            % include the Buttons
            prepButton(pnl, pnl.subPnl.lanCtrl.pnl);
            startButton(pnl, pnl.subPnl.lanCtrl.pnl);
            abortButton(pnl, pnl.subPnl.lanCtrl.pnl);
            restartButton(pnl, pnl.subPnl.lanCtrl.pnl); % maybe just add a feeding-button?
            % listener to reset LanControl
            pnl.listeners.resetLanCtrl  = addlistener(pnl,'reset',@resetLanCtrl);
            function [] = resetLanCtrl(~,~)
                % restart lan control panel
                lanCtrlSPnl(pnl,pnl.subPnl.lanCtrl.pos);
            end
        end

        %% SubPanel Camera
        function [] = camSPnl(pnl, cam, pos)
            if nargin==2
                pos = [.35 .35 .65 .65];
            end
            if ~isfield(cam,'device')
                error('Camera must tell me the name');
            end
            pnl.subPnl.cam.pos  = pos;
            pnl.subPnl.cam.info = cam;
            % test webcam or IP-cam (URL of IP-cam starts with http or rtsp)
            if strncmpi(cam.device(1:4),'http',4) || strncmpi(cam.device(1:4),'rtsp',4)
                % create IP-camera object
                pnl.subPnl.cam.camHand  = ...
                    ipcam(cam.device, cam.userName, cam.password);
            else
                % create webcam object
                pnl.subPnl.cam.camHand = webcam(cam.device);
            end
            % create a panel for camera
            pnl.subPnl.cam.pnl = uipanel('parent', pnl.uiPnl, ...
                'position',pos,'backgroundcolor','w'); %,'bordertype','none'
            % axis for the camera-stream
            ax = axes('Parent',pnl.subPnl.cam.pnl,'position',[0 0 1 1]);
            % the camear will update this image
            pnl.subPnl.cam.prevImg = image(ax, snapshot(pnl.subPnl.cam.camHand));
            axis('image');
            axis('off');
            % slisteners to start/ stop/ delete the camera
            pnl.listeners.pauseCam  = addlistener(pnl,'pause',@pauseCam);
            pnl.listeners.startCam  = addlistener(pnl,'start',@startCam);
            pnl.listeners.deleteCam = addlistener(pnl,'kill',@deleteCam);
            pnl.listeners.resetCam  = addlistener(pnl,'reset',@resetCam);
            % functions to start/ stop / delete camera
            function [] = pauseCam(~,~)
                % close camera stream
                closePreview(pnl.subPnl.cam.camHand);
                % delete camera to prevent matlab from streaming invisibly
                delete(pnl.subPnl.cam.camHand);
            end
            function [] = startCam(~,~)
                % start camera stream
                pnl.subPnl.cam.camHand  = ...
                    ipcam(pnl.subPnl.cam.info.device, ...
                    pnl.subPnl.cam.info.userName, ...
                    pnl.subPnl.cam.info.password);
                preview(pnl.subPnl.cam.camHand, pnl.subPnl.cam.prevImg);
            end
            function [] = deleteCam(~,~)
                % stop camera
                closePreview(pnl.subPnl.cam.camHand);
                delete(pnl.subPnl.cam.camHand);
            end
            function [] = resetCam(~,~)
                % stop camera
                closePreview(pnl.subPnl.cam.camHand);
                delete(pnl.subPnl.cam.camHand);
                % re-open the camera panel with the same parameters
                pnl.subPnl.cam.camHand  = ...
                    ipcam(pnl.subPnl.cam.info.device, ...
                    pnl.subPnl.cam.info.userName, ...
                    pnl.subPnl.cam.info.password);
                preview(pnl.subPnl.cam.camHand, pnl.subPnl.cam.prevImg);
            end
        end

        %% Subpanel Performance Info
        function [] = perfSPnl(pnl,pos)
            if nargin ==1
                pos = [.35 0 .65 .35];
            end
            pnl.subPnl.perf.pos = pos;
            pnl.subPnl.perf.pnl = uipanel('parent', pnl.uiPnl, ...
                'position',pos, 'backgroundcolor','w','bordertype','none', ...
                'title', 'Performance');

            % TEXT
            axes('parent',pnl.subPnl.perf.pnl, ...
                'position',[0 0 .3 1],'visible','off');
            % static text (labels)
            text(.3,.95,{'Subj.:','Task:'},...
                'horizontalalignment','right','verticalalignment','top',...
                'fontunits','normalized','fontsize',.075);
            text(.5,.65,{'maxTrls:','dur.:','trialNr:',...
                '%-corr.:', '%-incorr.:', '%-noRsp.:'},...
                'horizontalalignment','right','verticalalignment','top',...
                'fontunits','normalized','fontsize',.075);
            % dynamic text (userData and lanData)
            pnl.subPnl.perf.userText = text(.35,.95,{'Who?','What?'},...
                'horizontalalignment','left','verticalalignment','top',...
                'fontunits','normalized','fontsize',.075);
            pnl.subPnl.perf.lanText  = text(.55,.65,{'','','','','',''},...
                'horizontalalignment','left','verticalalignment','top',...
                'fontunits','normalized','fontsize',.075);

            % PLOT
            lnWdth = 1;     % plotting line width
            pnl.subPnl.perf.axHand = axes('parent',pnl.subPnl.perf.pnl, ...
                'position',[.35 .2 .625 .75],...
                'fontunits','normalized','fontsize',.075);
            ylabel('sum perf.');
            xlabel('trial number');
            % correct is blue
            pnl.subPnl.perf.lines(1) = line(pnl.subPnl.perf.axHand, ...
                1,0,'color','b','linewidth',lnWdth);
            hold on;
            % incorrect is red
            pnl.subPnl.perf.lines(2) = line(pnl.subPnl.perf.axHand, ...
                1,0,'color','r','linewidth',lnWdth);
            % no response in trial is black
            pnl.subPnl.perf.lines(3) = line(pnl.subPnl.perf.axHand, ...
                1,0,'color','k','linewidth',lnWdth);

            % pick up from LAN, callback/ listener
            % listen for lanData
            pnl.listeners.perfLan  = addlistener(pnl,'newData',@perfNewData);
            % listen for userData
            pnl.listeners.perfUser = addlistener(pnl,'userDone',@perfUserData);

            % initialize variables accessed by the listener-function
            pnl.lanData.parView.imgs    = [];
            pnl.lanData.parView.keys    = [];
            pnl.lanData.perf.trlNr      = 0;
            pnl.lanData.perf.maxTrls    = 0;
            pnl.lanData.perf.duration   = 0;
            % will be number of trial long
            pnl.lanData.perf.trlCorr  	= false;
            pnl.lanData.perf.trlInCorr  = false;
            pnl.lanData.perf.trlNoRsp   = false;
            % will obly be used if not set be local user
            pnl.lanData.subj            = 'Who?';
            pnl.lanData.paradigm        = 'What?';

            % listener to reset performance panel
            pnl.listeners.resetPerf = addlistener(pnl,'reset',@resetPerf);
            % function to reset panel through listener
            function [] = resetPerf(~,~)
                perfSPnl(pnl,pnl.subPnl.perf.pos);
            end

            % listen-function tied to lanData, updates performance text and figure
            function [] = perfNewData(~,~)
                % update the progress-lines
                set(pnl.subPnl.perf.lines(1),...
                    'YData', cumsum(pnl.lanData.perf.trlCorr),...
                    'XData', 1:length(pnl.lanData.perf.trlCorr));

                set(pnl.subPnl.perf.lines(2),...
                    'YData', cumsum(pnl.lanData.perf.trlInCorr),...
                    'XData', 1:length(pnl.lanData.perf.trlInCorr));

                set(pnl.subPnl.perf.lines(3),...
                    'YData', cumsum(pnl.lanData.perf.trlNoRsp),...
                    'XData', 1:length(pnl.lanData.perf.trlNoRsp));
                % Add check to avoid warning
                if(pnl.lanData.perf.maxTrls == 1)
                    pnl.lanData.perf.maxTrls = 2;
                end
                if pnl.lanData.perf.maxTrls ~= 0
                    set(pnl.subPnl.perf.axHand,'XLim',...
                        [1 pnl.lanData.perf.maxTrls]);
                end

                % update the text
                pnl.subPnl.perf.lanText.String = {...
                    num2str(pnl.lanData.perf.maxTrls), ...
                    num2str(pnl.lanData.perf.duration), ...
                    num2str(pnl.lanData.perf.trlNr), ...
                    num2str(sum(pnl.lanData.perf.trlCorr)/...
                    pnl.lanData.perf.trlNr*100), ...
                    num2str(sum(pnl.lanData.perf.trlInCorr)/...
                    pnl.lanData.perf.trlNr*100), ...
                    num2str(sum(pnl.lanData.perf.trlNoRsp)/...
                    pnl.lanData.perf.trlNr*100)}';
                % Update subject and paradigm if not set buy local user
                if strcmpi(pnl.subPnl.perf.userText.String{1},'Who?')
                    pnl.subPnl.perf.userText.String{1} = ...
                        pnl.lanData.subj;
                end
                if strcmpi(pnl.subPnl.perf.userText.String{2},'What?')
                    pnl.subPnl.perf.userText.String{2} = ...
                        pnl.lanData.paradigm;
                end
            end

            % tied to userDone, updates userData text
            function [] = perfUserData(~,~)
                pnl.subPnl.perf.userText.String{1} = pnl.userData.subj{1};
                pnl.subPnl.perf.userText.String{2} = ...
                    pnl.userData.paradigm(1:end-2);
            end
        end

        %% Subpanel Paradigm Viewer
        function [] = parViewSPnl(pnl,pos)
            if nargin ==1
                pos = [0 .65 .35 .35];
            end
            % create the subpanel
            pnl.subPnl.parView.pos = pos;
            pnl.subPnl.parView.pnl = uipanel('parent', pnl.uiPnl, ...
                'position',pos, 'bordertype','none','backgroundcolor','w', ...
                'title', 'Paradigm');
            % this button loads paradigm and subject name
            prepButton(pnl, pnl.subPnl.parView.pnl);
            % wait for user to select data
            pnl.listeners.parViewUser = addlistener(pnl,'userDone',@startParView);

            % tied to listener userDone, initializes the figure etc
            function [] = startParView(~,~)
                % pick up screen and key-properties
                global SETUP;

                % Save local SETUP variable, because it will be overwritten
                % by the remote SETUP data
                SETUPSAVE = SETUP;

                % Get the data we need
                run(fullfile(pnl.userData.folder, 'myHardwareSetup'));
                run(fullfile(pnl.userData.folder, 'myParadigmSetup'));
                % reformat and keep what you need
                pnl.subPnl.parView.screen.screenWidth   = SETUP.screen.ScreenWidth;
                pnl.subPnl.parView.screen.screenHeight  = SETUP.screen.ScreenHeight;
                pnl.subPnl.parView.screen.x         = SETUP.screen.X ./ pnl.subPnl.parView.screen.screenWidth;
                pnl.subPnl.parView.screen.y         = (SETUP.screen.ScreenHeight-SETUP.screen.Y-SETUP.screen.height) ./ pnl.subPnl.parView.screen.screenHeight;
                pnl.subPnl.parView.screen.width     = SETUP.screen.width ./ pnl.subPnl.parView.screen.screenWidth;
                pnl.subPnl.parView.screen.height    = SETUP.screen.height ./ pnl.subPnl.parView.screen.screenHeight;
                pnl.subPnl.parView.bgColor          = SETUP.screen.bgColor;

                % load all images
                if ~exist(fullfile(pnl.userData.folder ,'Stimuli'),'dir')
                    error(['Stimulus-folder: ' fullfile(pnl.userData.folder,'Stimuli') ' does not exist!']);
                end

                stimFldr    = fullfile(pnl.userData.folder,'Stimuli');
                fls         = dir(stimFldr);

                for i=3:length(fls)
                    % Check file and allow only image files
                    if(contains(fls(i).name, {'.png', '.jpg', '.jpeg', '.tif', '.bmp'}))
                        % use the file-names as field names
                        pnl.subPnl.parView.imgs.(['img' fls(i).name(1:end-4)]) ...
                            = imread(fullfile(stimFldr, fls(i).name));
                    end
                end

                % Set background and define a "background image"
                pnl.subPnl.parView.noImg(:,:,1) = ones(3,3)*pnl.subPnl.parView.bgColor(1);
                pnl.subPnl.parView.noImg(:,:,2) = ones(3,3)*pnl.subPnl.parView.bgColor(2);
                pnl.subPnl.parView.noImg(:,:,3) = ones(3,3)*pnl.subPnl.parView.bgColor(3);

                % set panel color to setup background-color
                set(pnl.subPnl.parView.pnl, ...
                    'backgroundcolor',pnl.subPnl.parView.bgColor);

                % create axis and image for each stimulus location
                for i=1:length(pnl.subPnl.parView.screen.x)
                    axes('parent', pnl.subPnl.parView.pnl,'pos', ...
                        [pnl.subPnl.parView.screen.x(i), ...
                        pnl.subPnl.parView.screen.y(i), ...
                        pnl.subPnl.parView.screen.width(i), ...
                        pnl.subPnl.parView.screen.height(i)]);
                    % a dummy-image
                    pnl.subPnl.parView.keys(i) = ...
                        image(pnl.subPnl.parView.noImg);
                    set(gca,'box','off','xtick',[],'ytick',[],...
                        'xticklabel','','yticklabel','');
                    axis('image');
                end
                % listen for new paradigm data
                pnl.listeners.parViewDat = addlistener(pnl,'newData',@updateParView);

                % listener to reset paradigm-viewer panel
                pnl.listeners.resetParView = addlistener(pnl,'reset',@resetParView);

                % Remove SETUP which contains remote client data and reset
                % to local data again
                SETUP = [];
                SETUP = SETUPSAVE;

                % function to reset panel through listener
                function [] = resetParView(~,~)
                    parViewSPnl(pnl,pnl.subPnl.parView.pos);
                end
            end

            %% update function tied to listener newData
            function [] = updateParView(~,~)
                if pnl.lanData.parView.showStim && ...
                        any(isnan(pnl.lanData.parView.keys))
                    % clear all stimuli if images and keys are passed empty
                    for i=1:length(pnl.subPnl.parView.keys)
                        pnl.subPnl.parView.keys(i).CData = ...
                            pnl.subPnl.parView.noImg;
                    end
                elseif pnl.lanData.parView.showStim
                    % otherwise update defined images on defined keys
                    for i=1:length(pnl.lanData.parView.keys)
                        % lanData contains names of images and of key
                        [~, fname] = fileparts(pnl.lanData.parView.imgs{i});
                        % Check function name and correct
                        pnl.subPnl.parView.keys(pnl.lanData.parView.keys(i)).CData = ...
                            pnl.subPnl.parView.imgs.(['img' fname]);
                    end

                    % Delete old images in any case --> get all pposition
                    % that are not defined with new images
                    allKeys = 1:length(pnl.subPnl.parView.keys);
                    allKeys(pnl.lanData.parView.keys) = [];

                    for i=1:length(allKeys)
                        pnl.subPnl.parView.keys(allKeys(i)).CData = ...
                            pnl.subPnl.parView.noImg;
                    end
                end
            end
        end

        %% BUTTONS --------------------------------------------------------

        %% Button Start/ Pause Panel
        function [] = pausePnlButton(pnl, parent)
            pnl.button.psePnl = uicontrol('parent',parent, ...
                'style','pushbutton',...
                'string','Start panel', ...
                'fontunits','normalized', 'fontsize',.6,...
                'units','normalized', 'position',[.1 .75 .8 .15],...
                'callback', @cBck_strtPnlBtn);
            % change button state if monitor is started
            pnl.listeners.btnPsePnl = addlistener(pnl,'start',@startPseBtn);
            pnl.listeners.btnPsePnlReset = ...
                addlistener(pnl,'reset',@startPseBtn);
            function [] = startPseBtn(~,~)
                % adjust the button
                set(pnl.button.psePnl,'callback', @cBck_psePnlBtn, ...
                    'string', 'Pause panel');
                % some info
                pnl.subPnl.info.line1.String = 'Panel is Running';
            end
            % callback prepare client for paradigm
            function [] = cBck_psePnlBtn(src,~)
                % adjust the button
                set(src,'callback', @cBck_strtPnlBtn, ...
                    'string', 'Start panel');
                % create a pause-event
                notify(pnl,'pause');
                % some info
                pnl.subPnl.info.line1.String = 'Panel is Paused';
            end
            function [] = cBck_strtPnlBtn(src,~)
                % adjust the button
                set(src,'callback', @cBck_psePnlBtn, ...
                    'string', 'Pause panel');
                % create a (re-)start event
                notify(pnl,'start')
                % some info
                pnl.subPnl.info.line1.String = 'Panel is Running';
            end
        end

        %% Button Remove SubPanel
        function [] = removePnlButton(pnl, parent)
            pnl.button.killPnl = uicontrol('parent',parent, ...
                'style','pushbutton',...
                'string','Remove panel', ...
                'fontunits','normalized', 'fontsize',.6,...
                'units','normalized', 'position',[.1 .6 .375 .1],...
                'callback', @pnl_destroy);
            function [] = pnl_destroy(~,~)
                if strncmpi(questdlg('Close this panel?', ...
                        'Are you sure?','Yes','No','Yes'), 'Ye',2)
                    destroy(pnl);
                end
            end
        end

        %% Button Restart SubPanel
        function [] = restartPnlButton(pnl, parent)
            pnl.button.resetPnl = uicontrol('parent',parent, ...
                'style','pushbutton',...
                'string','Reset panel', ...
                'fontunits','normalized', 'fontsize',.6,...
                'units','normalized', 'position',[.525 .6 .375 .1],...
                'callback', @pnl_restart);
            function [] = pnl_restart(~,~)
                if strncmpi(questdlg('Restart this panel?', ...
                        'Are you sure?','Yes','No','Yes'), 'Ye',2)
                    restart(pnl);
                end
            end
        end

        %% Button Prepare Setup
        function [] = prepButton(pnl, parent)
            % allow several of these buttons
            if isfield(pnl.button,'prep')
                i = length(pnl.button.prep)+1;
            else
                i = 1;
            end
            % set up the paradigm
            pnl.button.prep(i) = uicontrol('parent',parent, ...
                'style','pushbutton',...
                'string','Select paradigm', ...
                'fontunits','normalized', 'fontsize',.25,...
                'units','normalized', 'position',[.1 .5 .8 .35],...
                'callback', @cBck_prepBtn);

            % callback prepare client for paradigm
            function [] = cBck_prepBtn(~,~)
                % Init variables
                done = 0;

                % pause panel
                notify(pnl,'pause');
                % check if input is OK
                % get from the user
                [pnl.userData.paradigm, pnl.userData.folder] = ...
                    uigetfile('','Locate the paradigm');

                % Check for valid data
                if(ischar(pnl.userData.paradigm))
                    pnl.userData.subj = inputdlg('Enter subject name');

                    % send event-notification that selection is complete
                    if(~isempty(pnl.userData.subj))
                        pause(.5);
                        notify(pnl,'userDone');
                        done = 1;
                    end
                end

                % Delete entries in struct if not done
                if(done == 0)
                    pnl.userData.subj       = [];
                    pnl.userData.paradigm   = [];
                    pnl.userData.folder     = [];
                end
            end
        end

        %% Button Start Paradigm
        function [] = startButton(pnl, parent)
            % start/ pause button is not active until subject name and
            %   paradigm load are completed
            pnl.button.start = uicontrol('parent',parent, ...
                'style','pushbutton',...
                'string','START', 'enable', 'off',...
                'fontunits','normalized', 'fontsize',.5,...
                'units','normalized', 'position',[.1 .275 .8 .15],...
                'callback', @cBck_startBtn);
            % Listener object is notified after user selected data
            pnl.listeners.strtBtn = addlistener(pnl,'userDone',@hasUserData_start);

            % callback function
            function [] = cBck_startBtn(src,~)
                % It's all a try ....
                try
                    % Inform user
                    set(src,'string', 'Copying data to client');
                    % Copy files to experiment
                    sendExperiment2Host(pnl.client.remoteIP, pnl.userData.folder);
                    pause(0.25);

                    % Inform user
                    set(src,'string', 'Connecting to client');
                    % Connect to host (e.g. Raspberry)
                    pnl.client.OTBRHand = connect2Host({pnl.client.remoteIP});

                    % Establish communication for panel from remote client to
                    % this computer
                    sendCommand2Host(pnl.client.OTBRHand, ['connect2ClientPanel(''' pnl.client.localIP ''' , ''' pnl.client.remoteIP ''');']);
                    pause(0.25);

                    % Inform user
                    set(src,'string', 'Starting experiment');
                    % Start experiment on remote machine
                    sendCommand2Host(pnl.client.OTBRHand, [pnl.userData.paradigm(1:end-2) '(''' pnl.userData.subj{1}, ''');']);
                    % Done
                    set(src,'string', 'Experiment is running', ...
                        'callback',@cBck_pauseBtn, 'Enable', 'off');
                catch
                    set(src,'string', 'Error!');
                end
            end

            % callback pause button
            function [] = cBck_pauseBtn(src,~)
                set(src,'string','RUN',...
                    'callback',@cBck_startBtn);
            end

            % tied to listener enable start button after user selected data
            function [] = hasUserData_start(~,~)
                % enable the start button
                set(pnl.button.start,'enable','on');
            end
        end

        %% Button Abort Paradigm
        function [] = abortButton(pnl,parent)
            % aborts the paradigm and restarts remote client
            pnl.button.abort = uicontrol('parent',parent, ...
                'style','pushbutton',...
                'string','ABORT',...
                'fontunits','normalized', 'fontsize',.5,...
                'units','normalized', 'position',[.6 .15 .3 .1],...
                'callback', @cBck_abortBtn);
            % callback abort button
            function [] = cBck_abortBtn(~,~)
                sure = questdlg('Abort paradigm, restart remote box?',...
                    'Close Request Function','Yes','No','Yes');
                switch sure
                    case 'Yes'
                        % get rid of the camera
                        % get rid of the client
                        % get rid of the clent panel
                        destroy(pnl);
                    case 'No'
                        return
                end
            end
        end

        %% Button restart Raspberry
        function [] = restartButton(pnl,parent)
            % send custom command to remote client
            pnl.button.restart = uicontrol('parent',parent, ...
                'style','pushbutton',...
                'string','Reboot Client and Panel',...
                'fontunits','normalized', 'fontsize',.5,...
                'units','normalized', 'position',[.1 .15 .5 .1],...
                'callback', @cBck_restartBtn);

            % Callback function for custom
            function [] = cBck_restartBtn(~,~)
                % Init variables
                go = 0;

                quest = sprintf('Do you really want to reboot the client and this viewer?\nDid you copy the results from your client?');
                sure = questdlg(quest,...
                    'Close Request Function','Yes','No','Yes');
                switch sure
                    case 'Yes'
                        go = 1;
                    case 'No'
                        return
                end

                if (go == 1)
                    % Close all connections and reboot client
                    disconnectFromHost('host', {pnl.client.remoteIP}, 'forcereboot');

                    % Reboot panel
                    restart(pnl)
                end
            end
        end
    end
end