function [fig, cPnl] = openCPanelFig(clientList, subPanels)

% function [fig, cPnl] = openCPanelFig(clientList, subPanels)
%
% Opens a figure with clientPanels. Each clientPanel can monitor (control)
% remote (local) paradigms. Closing the figure also cleans up all the
% connections made by the clientPanels.
%
%   ClientPanels require a network connection to the client that is to
%   be monitored. Information must be send actively form the clients
%   and if remote paradigms are to be controlled, listening for
%   commands from the clientPanel must be actively implemented.
%
%   See the following functions for client-machines:
%     send2ClientPanel      : Sends data to the clientPanel
%                           : perfSPnl reads parameter-value pairs
%               trlnr       : current trial number
%               maxTrls     : max number of trials in session
%               duration    : current duration of session
%               trlCorr     : logical vec, all trials in session, corr?
%               trlInCorr   : logical vec, all trials in session, incorr?
%               trlNoRsp    : logical vec, all trials in session, response?
%                           : parViewSPnl reads parameter-value pairs
%               showStimuli : images (loaded with loadImage): vector of keys
%                           : this input is identical to showstimuli(im,key)
%     connect2ClientPanel   : Establish the connection to clientPanel
%     listen2ClientPanel    : Read commands from clientPanel
%
% INPUT:
% clientList: Struct, length is number of clientPanels, each panel needs
%           : clientList.client.name        : string name
%           : clientList.client.remoteIP    : ip address of remote host
%           : clientList.client.localIP     : ip address of local host
%       If a camera-panel is selected, each client also needs
%           : .camera.device    : full url of ipcam (or name of webcam)
%           : .camera.username  : ipcams only
%           : .camera.password  : ipcams only
%
% INPUT-OPTIONAL:
% subPanels : cell of strings to select the subpanels, same for all
%           : clientPanles. Possible strings are:
%           : performance   : progress, text and figure
%           : camera        : ipcamera/ webcam
%           : paradigm      : paradigm viewer (subjects perspective)
%                           : Requires paradigm on clientPanel-machine,
%                           : including a folder '.\Stimuli' in the same
%                           : directory, must contain all stimuli and only
%                           : stimuli.
%           : lanControl    : control a remote paradigm
%           : lclControl    : control a local paradigm
%
% OUTPUT:
% fig       : figure-handle
% cPnl      : cell of clientPanel handles
%
% EXAMPLE:
% 1. One clientPanel with camera
% clientList.client = struct('name','remoteBox','remoteIP','10.147.143.148');
% clientList.camera = struct('device','http://10.147.143.147/mjpg/video.mjpg', ...
%                           'userName','admin', 'password','1234');
% subPanels = {'performance', 'paradigm', 'camera'};
% [fig, cPnl] = openCPanelFig(clientList, subPanels);
%
% Possible string/URLs for IP Cams:
%   Edimax 3005w    --> http://192.168.0.100/mjpg/video.mjpg
%   Edimax 3115w    --> http://192.168.0.100/mjpg/video.mjpg
%   Edimax 3116w    --> http://192.168.0.100/mjpg/video.mjpg
%   Sygonix 43176Y  --> rtsp://192.168.0.100/v04
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x

% Author    : Jonas Rose/ Tobias Otto
% Version   : 0.3
% Date      : 20.02.2019

% 16.01.2019, Jonas: first draft
% 21.02.2019, Tobias: modified docu and added webcam links
% 27.02.2019, Tobias: added error handling and cosmetics

%% Which subPanels are requested by the user
showLanCtrl     = any(strcmpi(subPanels,'lancontrol'));
showLclCtrl     = any(strcmpi(subPanels,'lclcontrol'));
showPerf        = any(strcmpi(subPanels,'performance'));
showParadigm    = any(strcmpi(subPanels,'paradigm'));
showCam         = any(strcmpi(subPanels,'camera'));
stop            = 0;

%% Check if all necessary toolboxes are installed
if(~exist('udp'))
    disp(' -------------------------------------------------------------------------------');
    disp('  openCPanelFig needs the Instrument Control Toolbox installed on your system!');
    disp('  Please install the Instrument Control Toolbox and try again!');
    disp(' -------------------------------------------------------------------------------');
    stop = 1;
end

if(showCam == 1 && exist('webcam') == 0)
    disp(' -------------------------------------------------------------------------------');
    disp('  openCPanelFig needs either the Support Package for IP Cameras or the');
    disp('  Support Package for USB Webcams installed on your system.');
    disp('  Which package you need depends on your setup!');
    disp('  Please install either of the packages and try again!');
    disp(' -------------------------------------------------------------------------------');
    stop = 1;
end

if(stop == 1)
    error('We can''t continue here. I give up!');
end

%% Open the figure
fig = figure('menubar','none','color','w','CloseRequestFcn',@my_closeReq);

%% Init panels
% how many colums of panels will we need?
colNr = ceil(sqrt(length(clientList)));
% how many rows?
rowNr = ceil(length(clientList)/colNr);
% position of the panels
l = 0:1/colNr:1-1/colNr;
b = 1-1/rowNr:-1/rowNr:0;
w = 1/colNr;
h = 1/rowNr;

%% Open each panel
for i=1:length(clientList)
    % may seem odd but panels will be place row-by-row from top to bottom
    [c, r] = ind2sub([colNr rowNr],i);
    pos = [l(c) b(r) w h];
    % create each clientPanel
    cPnl{i} = clientPanel(clientList(i).client, pos);

    % always add subpanels Info
    infoSPnl(cPnl{i});
    % add requested subpanels
    if showLanCtrl
        % LAN control
        lanCtrlSPnl(cPnl{i});
    end
    if showLclCtrl
        % local control
        lclCtrlSPnl(cPnl{i});
    end
    if showPerf && ~(showLanCtrl || showLclCtrl)
        % performance wider if there is no control subpanel
        perfSPnl(cPnl{i}, [0 0 1 .35]);
    elseif showPerf
        % performance
        perfSPnl(cPnl{i});
    end
    if showParadigm
        % paradigm
        parViewSPnl(cPnl{i});
    end
    if showCam
        % camera
        camSPnl(cPnl{i}, clientList(i).camera);
    end
end

%% This takes care of the objects when the figure is being closed
    function [] = my_closeReq(~,~)
        % Close request function
        % do you really want to close?
        selection = questdlg('Close Figure and disconnet clients??',...
            'Really?',...
            'Yes','No','Yes');
        switch selection,
            case 'Yes'
                try
                    for j=1:length(cPnl)
                        destroy(cPnl{j});
                    end
                    delete(gcf)
                catch
                    warning('Deleting the clientPanels failed, better restart Matlab');
                    delete(gcf)
                end
            case 'No'
                return
        end
    end
end