function closeExtraWindow(screenNum)
% Use this function to close the extra stimulus window, normally at the end of
% your experiment. Requires global WINDOW, see initExtraWindow.m.
% This function can also be used to close the toolbox-windows if the
% handles have accidentally been deleted.
%
% closExtraWindow needs the monitor numer as argument!
%
% EXAMPLES
% 1.
%   closeWindow(3)
%   ans = []
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x
%
% See also:  initWindow, deleteSubplots, closeWindow

% VERSION HISTORY:
% Author:         Tobias Otto
% Version:        1.0
% Last Change:    22.03.2021
%
% 22.03.2021: Tobias: first draft based on closeWindow

%% Init variables
global WINDOW

%% Add checks
% Add argument check
if(nargin ~= 1)
    disp('***************************************************************')
    disp(' closeExtraWindow needs one argument: the number of the extra ');
    disp(' screen that should be closed');
    disp('***************************************************************')
    error('Please solve problem and try again');
end

% Check if screenNum is identical with an extra monitor
if(~any([WINDOW.extraStimulus.choosenMonitor] == screenNum))
    sysScreenNum = Screen('Screens');
    disp('*****************************************************************')
    disp(' The given screen number is NOT a screen that was initialized !!!');
    disp(' using initExtraWindow ');
    disp([' The screen numbers of your systems are: ' num2str(sysScreenNum)]);
    disp(' Where 0 is the number for the first two screens!');
    disp('*****************************************************************')
    error('Please solve problem and try again');
end

% Check if chosen screen is different from the main presentation screen
if(screenNum == WINDOW.choosenMonitor)
    disp('****************************************************************')
    disp(' The selected screen is already used as main presentation screen. ');
    disp(' Please select another monitor from your system.');
    disp([' The screen numbers of your systems are: ' num2str(sysScreenNum)]);
    disp(' Where 0 is the number for the first two screens!');
    disp('****************************************************************')
    error('Please solve problem and try again');
end

%% Close stimulus window only if it exists
% If closeWindow is called after a clear all, the Window handles are
% deleted. Check if WINDOW is a struct -> everything is fine
% Or check if field stimulus exists, if not get back handles
if(~isstruct(WINDOW) || ~isfield(WINDOW.extraStimulus(screenNum), 'handleFigure'))
    % Find handles and copy to necessary struct
    WINDOW.extraStimulus(screenNum).handleFigure = findobj('name', ['Extra Display ' num2str(screenNum)]);
end

if(~isempty(WINDOW.extraStimulus(screenNum).handleFigure) && ishandle(WINDOW.extraStimulus(screenNum).handleFigure))
    disp('--> Deleting MATLAB figure context')
    % Delete context
    delete(WINDOW.extraStimulus(screenNum).handleFigure)
end

%% Close window
try
    Screen('Close', WINDOW.extraStimulus(screenNum).handle)

    % Prepare variables for next call of open Window
    WINDOW.extraStimulus(screenNum).handle = [];
catch
    disp('--> No handle to psychtoolbox screen available')
end

