function closeWindow
%Use this function to close the stimulus window, normally at the end of
% your experiment. Requires global WINDOW, see initWindow.m.
% This function can also be used to close the toolbox-windows if the
% handles have accidentally been deleted. All functions of closeWindow are
% included in the stopExperiment function.
% Tipp: call closeExperiment since it also checks hardware connections and
% resets them. Still closeWindow can be used to ONLY close the presentation
% window, while the user needs to close everything else by himself.
%
%
% EXAMPLES
% 1.
%   closeWindow
%   ans = []
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x
%
% See also:  initWindow, deleteSubplots

% VERSION HISTORY:
% Author:         Tobias Otto
% Version:        1.4
% Last Change:    31.03.2020
%
% 16.06.2017: Tobias: first draft for Biopsy Toolbox 2
% 17.07.2017, Tobias: set WINDOW.handle to a defined state
% 20.03.2018, Tobias: closing figure handle before deleting (for octave)
% 24.03.2020, Tobias: Updated documentation
% 31.03.2020, Tobias: more error handling

%% BUGS/ TODO
% - no known bugs

%% Init variables
global WINDOW
global SETUP

%% Find timers, stop and delete them (Windows only)
if(SETUP.platformNum == 1)
    t = timerfind;
    if(~(isempty(t)))
        stop(t);
        delete(t);
        disp('--> Deleted all timer objects');
    end
end

%% Close stimulus window only if it exists
% If closeWindow is called after a clear all, the Window handles are
% deleted. Check if WINDOW is a struct -> everything is fine
% Or check if field stimulus exists, if not get back handles
if(~isstruct(WINDOW) || ~isfield(WINDOW, 'stimulus'))
    % Find handles and copy to necessary struct
    WINDOW.stimulus.handle = findobj('name','Display');
end

if(~isempty(WINDOW.stimulus.handle) && ishandle(WINDOW.stimulus.handle))
    disp('--> Deleting MATLAB figure context')
    % Delete context
    delete(WINDOW.stimulus.handle)
end

%% Close window
try
    Screen('Closeall')
    % Show cursor
    ShowCursor;
    % Prepare variables for next call of open Window
    WINDOW.handle = [];
catch
    disp('--> No handle to psychtoolbox screen available')
end

