function bHand = createButton(text, varargin)
% Creates a button on the stimulus window
% Position defines the left upper corner of the button.
%
% Usage: buttonPosition = createButton(text, varargin)
% varargin  -> x        -> x Position in percent (0 -> left, 100 -> right)
%           -> y        -> y Position in percent (0 -> top, 100 -> bottom)
%           -> color    -> Text color as RGB vector with values from 0:255
%           -> size     -> Text size in pixel
%           -> dontFlip -> Don't flip buffers (advanced users)
%
% Use the Psychtoolbox function GetMouse to check if the user clicked within the
% button. More for advanced users, sorry for that :( Feel free to contact us
% if you need help with this.
%
% Example:  buttonPos = createButton('Okay', 'x', 10, 'y', 10);
%           Displays a button with text "Okay" in the left upper corner;
%
% Example:  buttonPos = createButton('Start');
%           Displays a button with text "Start" at the center of the screen;
%
% Example:  buttonPosl = createButton('Start', 'y', 20, 'x', 30, 'dontflip');
%           buttonPosr = createButton('Stop', 'y', 20, 'x', 70);
%           Displays two button with text "Start" and "Stop" in the upper
%           part of the screen.
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x

% Tobias Otto
% 1.1
% 22.05.2019

% 19.11.2009, Tobias: first draft
% 22.05.2019, Tobias: updated user info

%% Init variables
global WINDOW;
x           = 50;
y           = 50;
color       = WINDOW.white;
dontFlip    = 0;
textSize    = WINDOW.textSize;
penWidth    = 5;

%% Parse input
i=1;
while(i<=length(varargin))
    switch lower(varargin{i})
        case 'x'
            i=i+1;
            x = varargin{i};
            i=i+1;
        case 'y'
            i=i+1;
            y = varargin{i};
            i=i+1;
        case 'color'
            i=i+1;
            color = varargin{i};
            i=i+1;
        case 'size'
            i=i+1;
            % Set text size
            textSize = varargin{i};
            Screen('TextSize', WINDOW.handle, textSize);
            i=i+1;
        case 'dontflip'
            i=i+1;
            dontFlip = 1;
        otherwise
            closeWindow;
            error('Invalid input arument, please check and try again !!!');
    end
end

%% Get size of text
textBounds = Screen('TextBounds', WINDOW.handle, text);

%% Compute pixel for position x and y
% Anyway correct the position for the text size.
% Correct x and set text to middle of given position in percent
x = WINDOW.rect(3)*x/100 - textBounds(3)/2;
% Correct y and make text a bit higher than PsychToolbox wants it.
y = WINDOW.rect(4)*y/100 - textBounds(4)/2 + textSize/2;

%% Finally draw Box and text
[~, ~, textbounds]      = DrawFormattedText(WINDOW.handle, text, x, y, color, [],[],[],WINDOW.textSpacing);
boxRect                 = round(textbounds-[textSize/3 textSize/3 -textSize/3 -textSize/3]);
Screen('FrameRect', WINDOW.handle,color ,boxRect, penWidth);

%% Flip if necessary
if(dontFlip==0)
    Screen('Flip', WINDOW.handle);
end

%% Copy textBounds to output variable
bHand = textbounds;
