function initExtraWindow(screenNum)
% Initializes an additional Window with limited funktionality (displaying stimuli only)!
% This function only works if your setup has more than one screen and if the main windows is already
% initialized using initWindow.
%
% The function initExtraWindow needs the display number as argument to place the stimulus window on
% the correct monitor.
% Screen 1: corresponds to the display area of the monitor with the Windows-internal name \\.\DISPLAY1
% Screen 2: corresponds to the display area of the monitor with the Windows-internal name \\.\DISPLAY2
% ...
% Screen n: corresponds to the display area of the monitor with the Windows-internal name \\.\DISPLAYn
%
% INPUTS-OPTIONAL
%   * displayNr     : the number of the monitor used for displaying stimuli
%
% EXAMPLES
%   * initExtraWindow(1);    % open stimulus window in screen 1 if the main window if e.g. on screen  2
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x
%
% See also:  closeWindow, loadImage, showStimuli, deleteSubplots, rotateStimuli, initWindow

% VERSION HISTORY:
% Author:         Tobias Otto
% Version:        1.0
% Last Change:    12.03.2021
%
% 12.03.2021, Tobias: first draft based on initWindow


%% Init variables
% Get access to global struct
global WINDOW
global SETUP

if(nargin == 0)
    error('This function needs at least one input argument: the monitor number!');
end

if(ischar(screenNum))
    screenNum = str2double(screenNum);
end

%% Start main checks
% Check call of initOTBR
if(~isstruct(SETUP))
    disp('*******************************************************************');
    disp(' The struct SETUP is empty ! This could have a couple of reasons:');
    disp('  * forgot to call ''initOTBR'' before initializing the window');
    disp('  * call of ''clear'' or ''clear all''');
    disp('*******************************************************************');
    error('Please solve problem and try again');
end

% Only start if a valid main screen is open
if(isempty(WINDOW.handle))
    disp('*******************************************************************');
    disp(' Call this funktion after initializing a main stimulus presentation');
    disp(' screen using initWindow');
    disp('*******************************************************************');
    error('Please solve problem and try again');
end

% Check operating system
if(SETUP.platformNum ~= 1)
    disp('*********************************************************************');
    disp(' This function is only valid on Windows systems with multiple screens');
    disp('*********************************************************************');
    error('Please solve problem and try again');
end

% Get WINDOW coordinates and check input arguments
% Finally check if the number of available screens match
sysScreenNum = Screen('Screens');

if(~any(sysScreenNum == screenNum))
    disp('***************************************************************')
    disp(' The number of screens is not available in your setup !!!');
    disp([' The screen numbers of your systems are: ' num2str(sysScreenNum)]);
    disp(' Where 0 is the number for the first two screens!');
    disp('***************************************************************')
    error('Please solve problem and try again');
end

% Check if chosen screen is different from the main presentation screen
if(screenNum == WINDOW.choosenMonitor)
    disp('****************************************************************')
    disp(' The selected screen is already used as main presentation screen. ');
    disp(' Please select another monitor from your system.');
    disp([' The screen numbers of your systems are: ' num2str(sysScreenNum)]);
    disp(' Where 0 is the number for the first two screens!');
    disp('****************************************************************')
    error('Please solve problem and try again');
end


%% Open screen
% Open MATLAB figure and configure it for the selected screen. This figure protects the programs,
% icons, etc. in case a touch screen is used. Otherwise programs may be started
fprintf('--> Initializing extra figure ...');

% Multiple screens are only valid for windows, but the windows check was done before
sm          = get(0);
position    = sm.MonitorPositions(screenNum,:);

WINDOW.extraStimulus(screenNum).handleFigure  = figure('Name',['Extra Display ' num2str(screenNum)], ...
    'NumberTitle','off', ...
    'MenuBar','none', ...
    'Position', position);

% Wait for windows to appear
WaitSecs(0.2);
drawnow
fprintf(' done\n');

% Get real position of stimulus window
WINDOW.extraStimulus(screenNum).realPositionFigure = get(WINDOW.extraStimulus(screenNum).handleFigure, 'Position');

% Continue opening the PsychToolbox Screen screen
fprintf('--> Initializing PsychToolbox screen (this can take a while) ...');
try
    [WINDOW.extraStimulus(screenNum).handle, WINDOW.extraStimulus(screenNum).rect] = Screen('OpenWindow', screenNum, SETUP.screen.bgColor);
    Screen('Flip', WINDOW.extraStimulus(screenNum).handle);
catch
    fprintf('--> failed in first try!\n');
    fprintf('--> Initializing PsychToolbox screen using different settings (this can take another while) ...');
    Screen('Preference', 'SkipSyncTests', 1);
    [WINDOW.extraStimulus(screenNum).handle, WINDOW.extraStimulus(screenNum).rect] = Screen('OpenWindow', screenNum, SETUP.screen.bgColor);
    Screen('Flip', WINDOW.extraStimulus(screenNum).handle);
end
fprintf(' done\n');

% Hide mouse in the left upper corner to avoid random occurance of wait bar
% (on old computers)
if(screenNum == 0)
    SetMouse(WINDOW.rect(3), WINDOW.rect(4), WINDOW.handle);
end

%% Enable transparency if necessary
if(SETUP.screen.transparency == 1)
    % User info
    fprintf('Enabling transparency ...');

    [sourceFactorOld, destinationFactorOld, colorMaskOld] = ...
        Screen('BlendFunction', WINDOW.extraStimulus(screenNum).handle, 'GL_SRC_ALPHA', 'GL_ONE_MINUS_SRC_ALPHA');
    WINDOW.extraStimulus(screenNum).transparency         = 1;
    WINDOW.extraStimulus(screenNum).sourceFactorOld      = sourceFactorOld;
    WINDOW.extraStimulus(screenNum).destinationFactorOld = destinationFactorOld;
    WINDOW.extraStimulus(screenNum).colorMaskOld         = colorMaskOld;

    % User info
    fprintf(' done\n');
else
    WINDOW.extraStimulus(screenNum).transparency         = 0;
end

%% Get some toolbox related variables
WINDOW.extraStimulus(screenNum).monitorFlipInterval  = Screen('GetFlipInterval',WINDOW.handle);
WINDOW.extraStimulus(screenNum).frameRate            = Screen('FrameRate', WINDOW.handle);
WINDOW.extraStimulus(screenNum).nomFrameRate         = Screen('NominalFrameRate', WINDOW.handle);
WINDOW.extraStimulus(screenNum).choosenMonitor       = screenNum;
% More settings for "human mode"
WINDOW.extraStimulus(screenNum).boundingBox          = 30;
WINDOW.extraStimulus(screenNum).textSize             = 35;
WINDOW.extraStimulus(screenNum).textSpacing          = 1.4;   %sets the spacing between the lines
WINDOW.extraStimulus(screenNum).black                = 0;
WINDOW.extraStimulus(screenNum).white                = 255;
WINDOW.extraStimulus(screenNum).gray                 = (WINDOW.black+WINDOW.white)/2;
WINDOW.extraStimulus(screenNum).font                 = {'Arial', 'Geneva', 'Helvetica', 'sans-serif'};

%% Init Text
Screen('TextSize', WINDOW.extraStimulus(screenNum).handle, WINDOW.textSize);
Screen('TextFont', WINDOW.extraStimulus(screenNum).handle, char(WINDOW.font{2}));

