function initWindow(screenNum, rotation)
% Initializes a Window for displaying stimuli using the OTBR Toolbox.
% The function initWindow optionally takes the display number as argument
% to place the stimulus window on the correct monitor.
% Tipp: When testing the toolbox it is helpful to use two monitors
% (one for coding one for display). In this case the Windows internal
% screen ID number (easiest case 1 or 2) needs to be used to initialize
% the presentation window on this specific monitor.
% If you use only one monitor the function can be called without argument
% or with a 0 instead of a screen number. When using two monitors and
% you want to have a presentation screen over both monitors you can also
% call the function without argument (if the extended monitor option within
% windows is being used).
%
% Screen 1: corresponds to the display area of the monitor with the Windows-internal name \\.\DISPLAY1
% Screen 2: corresponds to the display area of the monitor with the Windows-internal name \\.\DISPLAY2
% ...
% Screen n: corresponds to the display area of the monitor with the Windows-internal name \\.\DISPLAYn
%
% Screen 0: Full screen in one monitor setups or both screens in multi
%           monitor setups
%
% In some cases the presented objects on the screen have to be mirrored
% horizontally or vertically. Especially if a mirror is used in an MRI setting.
% To mirror the objects on the screen use a second, optional argument
% On Windows computers: make sure scaling on all screens is set to 100% !!!
% 1 --> horizontal mirroring
% 2 --> vertical mirroring
% 3 --> 180 rotation
%
% INPUTS-OPTIONAL
%   * displayNr     : the number of the monitor used for displaying stimuli
%   * rotation      : mirroring or rotation
%
% EXAMPLES
%   * initWindow(1);    % open stimulus window in screen 1
%
%   * initWindow(1, 2); % open stimulus window in screen 1 and mirror it veretically
%
%   * initWindow;       % open stimulus window on default screen (in one monitor setups)
%
%   * initWindow([], 3; % open stimulus window on default screen (in one monitor setups)
%                       % and rotate it by 180
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x
%
% See also:  closeWindow, loadImage, showStimuli, deleteSubplots, rotateStimuli

% VERSION HISTORY:
% Author:         Tobias Otto
% Version:        1.13
% Last Change:    23.06.2023
%
% 16.05.2017, Tobias: first draft for Biopsy Toolbox 2
% 17.07.2017, Tobias: set default screen to 0 if called without arguments
% 06.11.2017, Tobias: added support for punnishment stimulus
% 09.11.2017, Tobias: added onScreen entry in WINDOWS struct to save screen content
% 18.05.2018, Tobias: hiding mouse in fullscreen mode
% 11.06.2018, Tobias: allowing char as input to be compatoble to old code
% 06.02.2019, Tobias: typo punsiment
% 24.03.2020, Tobias: Updated documentation
% 05.02.2021, Tobias: removed limitiation for two screens
% 19.03.2021, Tobias: adding integration for multiple screens and their position
% 19.05.2021, Tobias: necessary updates for Psychtoolbox-3.0.16; minimizing info on Raspberry
% 25.06.2021, Tobias: enabled protective figure for case 0 (two screens)
% 30.06.2022, Tobias: this function only work with Psychtoolbox 3.0.16 or higher on Raspberry!
% 23.06.2023, Tobias: introduced mirroring and rotation. Thanks to Andreas Volgmann @ Ifado Dortmund

%% BUGS/ TODO
% - no konwn bugs

%% Init variables
% Get access to global struct
global WINDOW
global SETUP

if(nargin == 0 || isempty(screenNum))
    screenNum = 0;
end

if(ischar(screenNum))
    screenNum = str2double(screenNum);
end

psychVersion = Screen('Version');

%% Check call of initOTBR
if(~isstruct(SETUP))
    disp('*******************************************************************');
    disp(' The struct SETUP is empty ! This could have a couple of reasons:');
    disp('  * forgot to call ''initOTBR'' before initializing the window');
    disp('  * call of ''clear'' or ''clear all''');
    disp('*******************************************************************');
    error('Please solve problem and try again');
end

%% Get WINDOW coordinates and check input arguments
% Finally check if the number of available screens match
sysScreenNum = Screen('Screens');

if(~any(sysScreenNum == screenNum))
    disp('******************************************************************************************************')
    disp(' The number of screens is not available in your setup !!!');
    disp([' The screen numbers of your systems are: ' sysScreenNum]);
    disp(' Where 0 is the number for the first two screens!');
    disp('******************************************************************************************************')
    error('Pleae solve problem and try again');
end

[width, height]         = Screen('WindowSize', screenNum);  % Get width and height of selected screen
WINDOW.availScreens 	= Screen('Screens');
WINDOW.screen.x0        = 0;
WINDOW.screen.y0        = 0;
WINDOW.screen.xMax      = width;
WINDOW.screen.yMax      = height;
WINDOW.screen.scrsz     = [width height];

%% Open screen
% Open MATLAB figure and configure it for the selected screen. This figure protects the programs,
% icons, etc. in case a touch screen is used. Otherwise programs may be started
fprintf('--> Initializing figure ...');


% Multiple screens are only valid for windows
if(SETUP.platformNum == 1)
    % Get screen positions from MATLAB
    % (This only works if the position in the MonitorPositions field is similar to the windows
    % entries)
    sm = get(0);
    if(screenNum ~= 0)
        position    = sm.MonitorPositions(screenNum,:);
    else
        tmpMax      = max(sm.MonitorPositions(:,4));
        position    = [sm.MonitorPositions(end, 1:2) sm.MonitorPositions(1,3) + sm.MonitorPositions(end,3) tmpMax];
    end
else
    if(psychVersion.major >= 3 && psychVersion.minor >= 0 && psychVersion.point>=16)
        % After Psychtoolbox update to version 3.0.16 this code is needed
        [width, height] = Screen('WindowSize', 0);
        position        = [0 0 width height];
    else
        error('This function only works with the OpenGL Psychtoolbox version 3.0.16 or higher!')
    end
end

WINDOW.stimulus.handle  = figure('Name','Display', ...
    'NumberTitle','off', ...
    'MenuBar','none', ...
    'Position', position);

% Wait for windows to appear
WaitSecs(0.2);
drawnow
if(SETUP.platformNum == 1)
    fprintf(' done\n');
end

% Get real position of stimulus window
WINDOW.stimulus.realPosition = get(WINDOW.stimulus.handle, 'Position');

% Continue opening the PsychToolbox Screen screen
if(SETUP.platformNum == 1)
    fprintf('--> Initializing PsychToolbox screen (this can take a while) ...');
end

try
    [WINDOW.handle, WINDOW.rect] = Screen('OpenWindow', screenNum, SETUP.screen.bgColor);
    Screen('Flip', WINDOW.handle);
catch
    if(SETUP.platformNum == 1)
        fprintf('--> failed in first try!\n');
        fprintf('--> Initializing PsychToolbox screen using different settings (this can take another while) ...');
    end
    Screen('Preference', 'SkipSyncTests', 1);
    [WINDOW.handle, WINDOW.rect] = Screen('OpenWindow', screenNum, SETUP.screen.bgColor);
    Screen('Flip', WINDOW.handle);
end
fprintf(' done\n');

% Hide mouse in the left upper corner to avoid random occurance of wait bar
% (on old computers)
if(screenNum == 0)
    SetMouse(WINDOW.rect(3), WINDOW.rect(4), WINDOW.handle);
end

% Init entry in struct to save the actual stimuli in the screen (needed for
% keyBuffer to indicate a bad/good response with a stimulus)
WINDOW.stimulus.onScreen = [];

%% Mirror or rotate screen (Thanks to Andreas Volgmann @ Ifado Dortmund)
if(nargin ==2)
    % Translate origin into the geometric center of text
    [xc , yc] = RectCenter(WINDOW.rect);
    Screen('glTranslate' , WINDOW.handle , xc , yc , 0);
    
    if(rotation == 1 || rotation == 3)
        % Flip left/right = Mirrored horizontally
        Screen('glScale' , WINDOW.handle , -1 , 1 , 1);
    end
    
    if(rotation == 2 || rotation == 3)
        % Flip top/bottom = Mirrored Vertically
        Screen('glScale', WINDOW.handle, 1, -1, 1);
    end

    % We need to undo the translations
    Screen('glTranslate' , WINDOW.handle , -xc , -yc , 0);
end

%% Compute rects for given stimulus positions
if(SETUP.platformNum == 1)
    fprintf('Computing position for stimuli ...');
end

for i=1:length(SETUP.screen.X)
    rect = [SETUP.screen.X(i), SETUP.screen.Y(i), ...
        SETUP.screen.X(i) + SETUP.screen.width(i), ...
        SETUP.screen.Y(i) + SETUP.screen.height(i)];
    % Add real coordinates in screen pixel
    rect = round(rect.*[WINDOW.rect(3) WINDOW.rect(4) WINDOW.rect(3) WINDOW.rect(4)]);
    WINDOW.stimulus.rect(i,:) = round(rect);
end

if(SETUP.platformNum == 1)
    fprintf(' done\n');
end

%% Compute rect for ephys/white bar on top
% Init ephys: position for rect
% User info
if(SETUP.platformNum == 1)
    fprintf('Computing position for ephys setting ...');
end

% Values of pos in percent:
% [left lower corner (X), left lower corner (Y), width of bar, height of bar]
pos	= SETUP.ephys.photodiode;
xy  = WINDOW.rect;
pos = round([pos(1)*xy(3) ((1-pos(2))-pos(4))*xy(4) ((pos(1)+pos(3))*xy(3)) (1-pos(2))*xy(4)]);
WINDOW.ephys.photodiode  = round(pos);

% User info
if(SETUP.platformNum == 1)
    fprintf(' done\n');
end

%% Compute rect for punishment stimulus
if(~isempty(SETUP.punishment.img))
    % User info
    if(SETUP.platformNum == 1)
        fprintf('Computing position for punishment stimulus ...');
    end

    % Values of pos in percent:
    % [left lower corner (X), left lower corner (Y), width of bar, height of bar]
    pos	= SETUP.punishment.imgPos;
    xy  = WINDOW.rect;
    pos = round([pos(1)*xy(3) ((1-pos(2))-pos(4))*xy(4) ((pos(1)+pos(3))*xy(3)) (1-pos(2))*xy(4)]);
    WINDOW.punishment.rect = round(pos);

    % Load image
    WINDOW.punishment.img = loadImage(SETUP.punishment.img);

    % User info
    if(SETUP.platformNum == 1)
        fprintf(' done\n');
    end
end

%% Enable transparency if necessary
if(SETUP.screen.transparency == 1)
    % User info
    if(SETUP.platformNum == 1)
        fprintf('Enabling transparency ...');
    end

    [sourceFactorOld, destinationFactorOld, colorMaskOld] = ...
        Screen('BlendFunction', WINDOW.handle, 'GL_SRC_ALPHA', 'GL_ONE_MINUS_SRC_ALPHA');
    WINDOW.transparency         = 1;
    WINDOW.sourceFactorOld      = sourceFactorOld;
    WINDOW.destinationFactorOld = destinationFactorOld;
    WINDOW.colorMaskOld         = colorMaskOld;

    % User info
    if(SETUP.platformNum == 1)
        fprintf(' done\n');
    end
else
    WINDOW.transparency         = 0;
end

%% Get some toolbox related variables
WINDOW.monitorFlipInterval  = Screen('GetFlipInterval',WINDOW.handle);
WINDOW.frameRate            = Screen('FrameRate', WINDOW.handle);
WINDOW.nomFrameRate         = Screen('NominalFrameRate', WINDOW.handle);
WINDOW.choosenMonitor       = screenNum;
% More settings for "human mode"
WINDOW.boundingBox          = 30;
WINDOW.textSize             = 35;
WINDOW.textSpacing          = 1.4;   %sets the spacing between the lines
WINDOW.black                = 0;
WINDOW.white                = 255;
WINDOW.gray                 = (WINDOW.black+WINDOW.white)/2;
WINDOW.font                 = {'Arial', 'Geneva', 'Helvetica', 'sans-serif'};

%% Init Text
Screen('TextSize', WINDOW.handle, WINDOW.textSize);
Screen('TextFont', WINDOW.handle, char(WINDOW.font{2}));

%% Init timer for manual shaping (for windows only)
if(SETUP.manualShaping.status == 1 && SETUP.platformNum == 1)
    disp(' ================================================================');
    disp(' ');
    disp('              !!! Manual shaping mode is used !!!');
    disp(' ');
    disp('         Please call closeWindow to stop manual shaping.');
    disp(' ================================================================');

    SETUP.manualShaping.reward = 0;
    SETUP.manualShaping.timer = timer('TimerFcn','manShap', ...
        'BusyMode','Queue', ...
        'Period',0.05 , ...
        'ExecutionMode','fixedSpacing');
    start(SETUP.manualShaping.timer)
end


%% Order fields
WINDOW = orderfields(WINDOW);

