function [text2display, escape] = inputTextBox(varargin)
% This function creates a box that allows the user to type a free defined
% word or text. The input is terminated by pressing enter.
% The text is available in MATLAB as a string.
% Use ESC key to exit function
%
% Usage: [output, escape] = inputTextBox(varargin)
%
% INTPUT
%           -> x          -> x Position in percent
%           -> y          -> y Position in percent (top is 0, bottom is 100)
%           -> color      -> text color as triplet (RGB)
%           -> time       -> maximum time to wait in seconds (default is 30s)
%           -> maxSymbols -> maximum Number of symbols (determines only the size of the box)
%
% OUTPUT:
%           output     -> string of text entered
%           escape     -> set to 1 if ESC was pressed, otherwise 0
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x

% Tobias Otto
% 1.4
% 13.03.2017

% 11.09.2008, Tobias: Release version
% 18.09.2008, Tobias: Added '' to character
% 09.10.2008, Tobias: bugfix, ignoring unusual keys or answers from kbcheck
% 22.04.2010, Tobias: Added escape with ESC key
% 13.03.2017, Tobias: computing position in percent,
%                     added size and question option

%% Init variables
global WINDOW
x               = 0.5;
y               = 0.5;
textSize        = WINDOW.textSize;
color           = [255 255 255];
penWidth        = 1;
ENTER           = 13;   % Code for [enter] is 13
ESC             = 27;   % Escape character
escape          = 0;
time            = 30;
keyIsDown       = 0;
keyCodeNum      = 1;
keyCode         = uint8(zeros(1,256));
seconds         = 1;
maxSymbols      = 20;
text2display    = '';
question        = ' ';
x2              = x;
specialChar     = [186 192 222 219 160]; %     ?(in PTB indicies)
validCharacters = [8 32:90  97:122 specialChar];

%% Parse inputarguments
i=1;
while(i<=length(varargin))
    switch lower(varargin{i})
        case 'x'
            i=i+1;
            x = varargin{i}/100;
            i=i+1;
        case 'y'
            i=i+1;
            y = varargin{i}/100;
            i=i+1;
        case 'color'
            i=i+1;
            color = varargin{i};
            i=i+1;
        case 'time'
            i=i+1;
            time = varargin{i};
            i=i+1;
        case 'maxsymbols'
            i=i+1;
            maxSymbols = varargin{i};
            i=i+1;
        case 'textsize'
            i=i+1;
            textSize = varargin{i};
            i=i+1;
        case 'question'
            i=i+1;
            question = varargin{i};
            i=i+1;
        otherwise
            closeWindow;
            error('Invalid input arument, please check and try again !!!');
    end
end

%% Compute position of text box
% Input box
oldTextSize = Screen('TextSize',WINDOW.handle ,textSize);
textBounds  = Screen('TextBounds', WINDOW.handle, char(ones(1,maxSymbols)*87));
textBounds 	= round(textBounds.*[0 0 1.8 1.9]);

x = x * (WINDOW.rect(3)-textBounds(3)/2);
y = y * (WINDOW.rect(4)-textBounds(4)/2);

% Compute frame position
frame   = round([x y-textBounds(4)/4 x+textBounds(3)/2 y+3*textBounds(4)/4]);


%% Compute position of question
textBounds  = Screen('TextBounds', WINDOW.handle, question);
textBounds 	= round(textBounds.*[0 0 1.8 1.9]);
x2 = x2 * (WINDOW.rect(3)-textBounds(3)/2);
y2 = 0.9*frame(2) - textBounds(4);

%% Start main loop
% Start with frame
Screen('FrameRect', WINDOW.handle, color, frame, penWidth);
Screen('DrawText', WINDOW.handle, question, x2, y2, color);
Screen('Flip', WINDOW.handle);
% Clear buffer
WaitSecs(0.01);
KbCheck;

%% Finally the loop
t1 = GetSecs+time;
while(t1>=GetSecs && keyCodeNum ~= ENTER && escape == 0 && length(text2display) < maxSymbols)
    % Check keys
    [keyIsDown,~, keyCode] = KbCheck;

    % If the user presses funny keys or KBCheck returns unusual values, simply ignore them!
    if(keyIsDown && sum(keyCode)== 1)
        % Get the valid key code
        keyCodeNum = find(keyCode);

        if(keyCodeNum == ESC)
            disp('ESCAPE');
            escape = 1;
        end

        if(any(keyCodeNum == validCharacters))
            if(keyCodeNum ~= 8)
                if(any(specialChar))
                    switch keyCodeNum
                        case 222
                            keyCodeNum = 196;
                        case 192
                            keyCodeNum = 214;
                        case 186
                            keyCodeNum = 220;
                        case 219
                            keyCodeNum = 223;
                        case 160
                            keyCodeNum = 63;
                    end
                end

                text2display = [text2display char(keyCodeNum)];
            else
                text2display = text2display(1:end-1);
            end

            %% Draw text and frame
            Screen('FrameRect', WINDOW.handle, color , frame, penWidth);
            Screen('DrawText', WINDOW.handle, text2display, x, y, color);
            Screen('DrawText', WINDOW.handle, question, x2, y2, color);
            Screen('Flip', WINDOW.handle);
            while KbCheck; end
        end
    end
end

%% Clean uo
% Set text size to old text size
Screen('TextSize',WINDOW.handle ,oldTextSize);
