function image = loadImage(filename)
% Loads an image for use with the OTBR Toolbox. Call imformats to see a
% list of supported formats and their file extensions.
% Make sure initWindow is already called. Otherwise this function will
% fail.
%
%
% INPUTS
%   * filename      : Path and name of the image-file
%
% INPUTS-OPTIONAL
%  none
%
% OUTPUTS
%  * image          : the image-object
%
% EXAMPLES
% 1.
%   im = loadImage('d:\Exp\pic1.jpg');    % Load the specified file
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x
%
% See also:  initWindow, showStimuli, rotateStimuli

% VERSION HISTORY:
% Author:         Tobias Otto
% Version:        1.5
% Last Change:    30.03.2020
%
% 04.07.2013, Tobias: first draft
% 10.07.2013, Tobias: removed support for animated gifs
% 24.10.2013, Tobias: removed unnecessary functions
% 10.07.2018, Tobias: image data not saved in struct anymore
% 23.01.2020, Tobias: enabling transparency for png files
% 30.03.2020, Tobias: user info

%% BUGS/ TODO
% - ko known bugs

%% Init variables
global WINDOW

%% Get image and file information
tmp = imfinfo(filename);

%% Copy some important information to the image object
image.filename      = tmp.Filename;
image.fileModDate   = tmp.FileModDate;
image.fileSize      = tmp.FileSize;
image.format        = tmp.Format;
image.width         = tmp.Width;
image.height        = tmp.Height;
image.bitDepth      = tmp.BitDepth;
image.colorType     = tmp.ColorType;

%% Load image
[data, ~, transparency] = imread(filename);

% Combine to a 4D matrix
if(~isempty(transparency))
    data(:,:,4) = transparency;
end

%% Create texture for PsychToolbox needs
image.texture = Screen('MakeTexture', WINDOW.handle, data);

%% Order fields
image = orderfields(image);