function out = plotScale(duration, text, varargin)
% Use this function to plot a scale on the stimulus presentation window
% that is highly adjustable.
%
% Usage: out = plotScale(duration, text, varargin)
%
% INPUTS
%   * duration	: The time to wait unless a response is made
%   * text      : A cell containing a string or character for each position
%                 of the scale.
%                 Simple formatting is allowed: \n is a line break
%
% INPUTS-OPTIONAL
%  * x      	: Position of scale in X, 0% -> left, 100% right
%  * y          : Position of scale in Y ,0% -> top, 100% bottom
%  * width      : Width of scale in percent (with respect to screen)
%  * textSize   : Text size in pixel
%  * penWidth   : Pen width in pixel (for scales)
%  * color      : Color as RGB triplet
%  * noDivision : Plot of the sub division lines is disabled. The text of
%                 each element is still plotted!
%  * active     : Set by default! Sets the scale as active (responses are
%                 saved). Only one active scale per screen! Otherwise
%                 results are random!!
%  * inactive   : Plots scale, but ignores clicks on the screen. Scale is
%                 grey to show that it's not active.
%                 This option includes the noFlip option! That means the
%                 scale isn't plotted on the screen when calling alone.
%  * markText   : Marks the text of the given index in a different color.
%                 The index defines the position of the text and has to be
%                 a scalar. A second argument is needed to define the
%                 color. Color is represented as RGB triplet.
%
% OUTPUTS
%  * out          :  A struct containing all possible information
%   .x              -> The X position of the point where the user clicked.
%                      Zero if the user did not click.
%   .y              -> The Y position of the point where the user clicked.
%                      Zero if the user did not click.
%   .time           -> Response time of the user in seconds
%   .escape         -> This entry is set to one if ESC was pressed.
%   .click          -> Answer in range from 0-> left to 1-> right
%                      Negative values means: click left from scale
%                      Values bigger than one means: click right from scale
%   .index          -> Index ranging from 1 to <number of text elements>
%                      for an easier analysis
%
%
% Example:  out = plotScale(5, {'1', '2', '3', '4', '5'}, 'y', 70, 'color', [255 0 255]);
%           Plots a scale in the lower middle of the screen with the values
%           ranging from 1 to 5. The scale is displayed 5 seconds or until
%           the user responds.
%
% Example:  out = plotScale(inf, {'a','b','c','d','e','f','g'});
%           Plots a scale ranging from a to g in the middle of the screen (default values).
%           The display time is set to infinity, that means the user has to respond.
% Example:  out = plotScale(10, {'Very\nlong\ntext', 'short1', 'short2', 'very long\n again'})
%
% Example:  plotScale(inf, {'a','b','c','d','e','f','g'}, 'y', 70, 'inactive', 'dontFlip', 'color', [128 128 128]);
%           out = plotScale(inf, {'a','b','c','d','e','f','g'});
%           Plots two scales on the screen, but only the first is active.
%           Plotting the inactive Scale has to be done first (noflip)!!!
%
% Example:  out = plotScale(10, {'a','b','c','d','e','f','g'}, 'markText', [2 5], [255 0 0]);
%           Plots a scale in the middle of the screen and the characters b
%           and e are marked in red.
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x

% Tobias Otto
% 1.19
% 08.11.2019

% 10.05.2010, Tobias: first draft
% 11.05.2010, Tobias: more user info
% 21.05.2010, Tobias: bugfix, added showCursor and again more user info
% 25.05.2010, Tobias: added index entry in struct to index scale words
% 01.06.2010, Tobias: replaced screen(drawtext) with drawFormattedText to format scale text
% 30.06.2010, Tobias: bug fix: added code for only two entries in scale
% 13.09.2010, Tobias: added a linitation in y. the user has to click in the text
% 29.09.2010, Tobias: removed plot of pixel in command window (cosmetics)
% 26.09.2011, Tobias: case sensitive typ due to MATLAB API
% 23.09.2012, Tobias: default color is white now
% 29.10.2014, Tobias: cosmetics added ;
% 05.01.2015, Tobias: added 'inactive' option to scale
% 25.02.2015, Tobias: maximized the range for the user to click
% 05.11.2015, Tobias: added option 'markText' and dontflip
% 13.03.2017, Tobias: changing text size to old size anyway
% 14.03.2017, Tobias: using text boxes for a nicer look
% 16.03.2017, Tobias: fixed output in case user didn't respond color in inactive mode is darker automatically
% 12.05.2017, Tobias: bug fix when no click occured: field index was still 1
% 19.06.2018, Tobias: forced click output to range from 0 to 1; added 'noDivision' as optional input
% 08.11.2019, Tobias: Bugfix in case duration is zero

%% Init variables
global WINDOW

posX        = 50;   % Position of scale in X with respect to center of scale, 0% -> left, 100% right
posY        = 50;   % Position of scale in Y with respect to center of scale, 0% -> top, 100% bottom
width       = 90;   % Width of scale in percent with respect to screen
textSize    = 15;   % Text size in pixel
color       = [255 255 255];
penWidth    = [];   % Pen width in pixel
active      = 1;
noDivision  = 0;

% Don't change anything here
entries     = length(text);
barHeight   = 12;   % Size (in pixel) of bars in scale, all bars have the same size
subLines    = [];
textDist    = 5;    % Size (in pixel) between scale and text
out         = [];
markText    = 0;
dontflip    = 0;
newLine     = 1;

%% Parse input
i=1;
while(i<=length(varargin))
    switch lower(varargin{i})
        case 'x'
            i=i+1;
            posX = varargin{i};
            i=i+1;
        case 'y'
            i=i+1;
            posY = varargin{i};
            i=i+1;
        case 'width'
            i=i+1;
            width = varargin{i};
            i=i+1;
        case 'textsize'
            i=i+1;
            textSize = varargin{i};
            i=i+1;
        case 'penwidth'
            i=i+1;
            penWidth = varargin{i};
            i=i+1;
        case 'color'
            i=i+1;
            color = varargin{i};
            i=i+1;
        case 'inactive'
            i=i+1;
            active  = 0;
            color   = round(color/2);
        case 'marktext'
            markText = 1;
            i=i+1;
            markIndex = varargin{i};
            i = i+1;
            colorText = varargin{i};
            i = i+1;
        case 'dontflip'
            i = i+1;
            dontflip = 1;
        case 'nodivision'
            i = i+1;
            noDivision = 1;
        otherwise
            closeWindow;
            disp('======================================================');
            disp('plotScale: Invalid input argument: ');
            disp(varargin{i});
            disp('======================================================');
            error('Invalid input arument, please check and try again !!!');
    end
end

%% Check input arguments
% Compute x positions of entries in scale (including start and stop)
if(entries < 2)
    closeWindow;
    error('The scale needs at least two entries !!!');
end

%% Compute position for lines
% Convert values to pixel
posX    = posX/100*WINDOW.rect(3);
posY    = posY/100*WINDOW.rect(4);
width   = width/100*WINDOW.rect(3);

% Compute start and stop points for lines (vector with start x,y and stop x,y)
% Main scale line
mainLine = [posX-width/2 posY posX+width/2 posY];

subLines(1,:)       = round([mainLine(1) posY-barHeight/2 mainLine(1) posY+barHeight/2]);
for i=1:entries-2
    dist            = subLines(i,1) + width/(entries-1);   % Distance in X between sub lines
    subLines(i+1,:) = round([dist posY-barHeight/2 dist posY+barHeight/2]);
end
% In case we have only two entries, adjust i to make the computation work,
% because we have no sublines.
if(entries == 2)
    i = 0;
end
subLines(i+2, :) = round([mainLine(3) posY-barHeight/2 mainLine(3) posY+barHeight/2]);

%% Compute position of text
% Change text size for scale, conserve the old size
oldTextSize = Screen('TextSize',WINDOW.handle ,textSize);

for i=1:entries
    % Get size of text
    textBounds = computeRealTextBounds(text{i});

    % Compute position
    textPos(i,:) = round([subLines(i,1)-textBounds(3)/2 subLines(i,2)-textDist-textBounds(4) subLines(i,1)+textBounds(3)/2 subLines(i,2)-textDist]);
end

%% Draw scale and text
% Main scale line
Screen('DrawLine', WINDOW.handle, color, mainLine(1), mainLine(2), mainLine(3), mainLine(4), penWidth);
% Sub lines
for i=1:entries
    % Draw lines
    if(noDivision == 0 || i == 1 || i == entries)
        Screen('DrawLine', WINDOW.handle, color, subLines(i,1), subLines(i,2), subLines(i,3), subLines(i,4), penWidth);
    end

    % Draw text
    if(markText == 1 && ismember(i, markIndex))
        DrawFormattedText(WINDOW.handle, text{i}, 'center', 'center', colorText, [], [], [], [], [],  textPos(i,:));
    else
        DrawFormattedText(WINDOW.handle, text{i}, 'center', 'center', color, [], [], [], [], [],  textPos(i,:));
    end
end

%% Flip
if(dontflip == 0)
    Screen('Flip', WINDOW.handle);
end
% Switch to old text size again
Screen('TextSize',WINDOW.handle ,oldTextSize);

%% This is only needed when scale is active!
if(active == 1)
    %% Show Cursor
    ShowCursor;

    %% Get mouse click
    % Compute range for the user to click
    maxY        = max(subLines(:,2))*1.15;
    minY        = min(textPos(:,2))*0.95;
    % More variables to be initialized
    startT      = GetSecs;
    tmpT        = 0;
    out.y       = (minY+maxY)/2;
    out.escape  = 0;

    % Get user click
    % ---------------
    % Init variables in case duration is zero!
    out = getMousePos(0);
    while(tmpT < duration && out.escape == 0)
        out     = getMousePos(duration-tmpT);
        % Are we in the range? Fine
        if( out.y > minY && out.y < maxY)
            tmpT = inf;
        else
            tmpT = GetSecs-startT;
        end
    end

    % Compute user answer in range from 0-> left to 1-> right
    out.click = (out.x-subLines(1,1))/width;
    % Correct if user clicked next to scale
    if(out.click < 0)
        out.click = 0;
    elseif(out.click > 1)
        out.click = 1;
    end

    % Compute position index
    lowerEdge       = linspace(0,1,entries)-(1/(entries+entries-2));
    upperEdge       = linspace(0,1,entries)+(1/(entries+entries-2));
    lowerEdge(1)    = -Inf;
    upperEdge(end)  = Inf;
    out.index       = find(out.click < upperEdge & out.click > lowerEdge);
    if(isempty(out.index) || out.x == 0)
        out.index   = 0;
        out.text    = '';
        out.click   = 0;
    else
        out.text  	= text{out.index};
    end

    %% Clean up
    HideCursor;
end
