function rect = showStimuli(im, position, varargin)
% Displays previously loaded image(s) on the monitor.
% This functions can be used in two different modes:
%
% Mode 1: second argument is a scalar/number that is linked to the
% previously defined "response keys" in the file myParadigmSetup.m
% This mode allows to present a stimulus at a previously defined position
% by prividing a single number.
%
% Mode 2: the position of the stimulus can be positioned indivisually by
% either defining the position of the left upper corner in percent or by
% defining a rect (see exampled below).
%
% The call of showStimuli requires a call of initOTBR.
% The position of the image is defined by the position of the InputKey(s)
% or by the x and y position in percent to position the stimulus freely on
% the presentation monitor.
%
% INPUTS
%   * data      	: image object
%
% INPUTS-OPTIONAL
%   * position (as second argument, see examples): Position of the image
%                    (images), scalar or vector or x and y position
%                    (see examples)
%   * x             : x Position in percent (0 -> left, 100 -> right)
%   * y             : y Position in percent (0 -> top, 100 -> bottom)
%   * rotation  	: Specifies a rotation angle in degree for rotated
%                     drawing of the texture
%   * transparency  : Specifies the transparency of an image in percent.
%                     From 0 (fully opaque) to 100 (fully transparent)
%   * ephys         : displays a white bar (usually on top of the monitor)
%                     to trigger a photo diode. This is useful to get the
%                     exact timing when the stimulus was presented on the
%                     monitor.
%   * rect          : Specifies the position of the stimulus in pixel
%   * dontFlip      : Don't flip buffers (advanced users)
%
% OUTPUTS
%   * rect          : returns the rect vector of the image
%
% EXAMPLES
% 1. Displays image go at position 1 (previously defined in myParadigmSetup.m)
%    showStimuli(go , 1);
%
% 2. Displays two images go and nogo at position 2 and 4
%    showStimuli([go nogo] , [2 4]);
%
% 3. Display image independently of predefined key
%    positions! Stimulus 1 is presented at the left upper corner of the
%    screen. 10% in x direction and 15% in y directtion
%    showStimuli(stimulus1, 'x', 10, 'y', 15);
%
% 4. Display image independently of predefined key positions! Stimulus 2 is
%    presented at the center of the screen (no additional arguments required)
%    showStimuli(stimulus2);
%
% 5. Display two images at the same time independently of predefined keys
%    positions! The optional argument 'dontFlip' is needed to allow
%    multiple other stimuli.
%    showStimuli(stimulus1, 'x', 50, 'y', 75, 'dontFlip');
%    showStimuli(stimulus2, 'x', 70, 'y', 10);
%
% 6. Displays image go at position 2 and rotated by 75
%    showStimuli(go , 2, 'rotation', 75);
%
% 7. Displays image go at position 2 and with 75% transparency
%    showStimuli(go , 2, 'transparency', 75);
%
% 8. Displays image go at position 1 (normal oriented) and nogo at
%    position 2 (rotated by 215 degree)
%    showStimuli([go nogo] , [1 2] , 'rotation', [0 215]);
%
% 9. Displays image go at position 1 (with 80% transparency) and nogo at
%    position 2 (transparency 10%)
%    showStimuli([go nogo], [1 2], 'transparency', [80 10]);
%
% 10. Displays image go and a white bar on top of the screen
%     showStimuli(go , 2, 'ephys');
%
% 11. This will clear the display window (delete the stimuli)
%     showStimuli;
%
% 12. Display image at a given position in pixel (coordinates are in
%     psychtoolbox format! [left top right bottom]).
%     This is for advanced users mostly!
%     The second argument will be ignored by showStimuli
%     showStimuli(go, 'rect', [10 10 310 110]);
%
% 13. Display more images at given positions in pixel (coordinates are in
%     psychtoolbox format!) This is for advanced users mostly!
%     showStimuli([go nogo], 'rect', [10 10 100 100; 210 410 310 510]);
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x
%
% See also:  initWindow, loadImage

% VERSION HISTORY:
% Author:         Tobias Otto
% Version:        1.5
% Last Change:    30.03.2020
%
% 04.07.2017, Tobias: first draft for Biopsy Toolbox 2
% 06.11.2017, Tobias: added optional argument rect
% 05.06.2018, Tobias: implemented mode for human experiments
% 10.12.2018, Tobias: bugfix in rect mode; reconfigure human and animal mode
% 22.05.2019, Tobias: more user info for "human" mode
% 30.03.2020, Tobias: adjusted user info

%% BUGS/ TODO
% - no konwn bugs

%% Init variables
global WINDOW;
num         = nargin;
rot      	= [];
trans       = [];
i        	= 0;
dontflip	= 0;
ephys       = 0;
useRect     = 0;
humanMode   = 0;
x           = 50;
y           = 50;
rect        = [];

% Clear content of global WINDOW struct. Anyway what we do with this
% function we will modify the content of the screen!
WINDOW.stimulus.onScreen = [];

%% Check input arguments
% showStimuli is used in animal research and human research. For human
% experiments the given pattern for pecking keys is ignored.
% In "human mode" the second input is always a string
if(exist('position', 'var') && any(ischar(position)))
    % Add argument to varargin as first input
    varargin    = [{position}, varargin];
    humanMode   = 1;
end

% One more option for "human mode" is possible
if(~exist('position', 'var') && isempty(varargin))
    humanMode   = 1;
end

%% Get variable input arguments
while(i~=length(varargin))
    % Increment
    i  = i+1;

    switch lower(varargin{i})
        case 'x'
            i    	= i+1;
            x     	= varargin{i};
        case 'y'
            i     	= i+1;
            y    	= varargin{i};
        case 'rotation'
            % Get the angle to rotate images -> one angle per image
            i   = i+1;
            rot = varargin{i};
            rot = num2cell(rot);
        case 'transparency'
            % Get the angle to rotate images -> one angle per image
            i       = i+1;
            trans   = 1-(varargin{i}/100);
            trans   = num2cell(trans);
            if(WINDOW.transparency ~= 1)
                closeWindow;
                disp(' ===========================================================');
                disp('  Transparency is not enabled in myParadigmSetup.m');
                disp('  Please enable transparency in section 4 and restart');
                disp('  your experiment!');
                disp(' ===========================================================');
                error('Please solve problem and try again');
            end
        case 'ephys'
            ephys	= 1;
        case 'dontflip'
            % We don't want to present the image
            dontflip    = 1;
        case 'rect'
            % Get the rect where we present the given stimulus
            i       = i+1;
            rect    = varargin{i};
            useRect	= 1;
        otherwise
            disp(' ===========================================================');
            disp([' Unknown argument: ' varargin(i)]);
            disp(' ===========================================================');
            i = i+1;
    end
end

%% Plot stimuli on the screen
if(num ~= 0)
    %% Init more variables
    % Compute position of stimuli in "human mode"
    if(humanMode == 1)
        if(useRect == 0)
            % Value is given in percent
            x       = round((WINDOW.rect(3)-WINDOW.rect(1)) * x/100);
            rect    = [x-floor(im.width/2) 0 x+ceil(im.width/2) 0];

            % Value is given in percent
            y           = round((WINDOW.rect(4)-WINDOW.rect(2)) * y/100);
            rect([2 4])	= [y-floor(im.height/2) y+ceil(im.height/2)];
        end

        % Get length in any case
        len = size(rect, 1);
    else
        len     = length(im);
        rect    = WINDOW.stimulus.rect(position,:);
    end

    %% Convert inputs into format we need
    % If no optional argument is given, we have to add a vector of empty
    % matrices. Otherwise conversion is already done in while loop for parsing
    % the input arguments
    if(isempty(trans))
        trans = cell(1, len);
    end
    if(isempty(rot))
        rot = cell(1, len);
    end

    % Present stimulus in given rect
    for i=1:len
        % Finally plot image
        Screen('DrawTexture', WINDOW.handle, im(i).texture, [], rect(i,:), rot{i}, [], trans{i});

        % Save current stimuli in global WINDOWS struct to save screen content
        % First handle to image and then the rect vector
        WINDOW.stimulus.onScreen = [WINDOW.stimulus.onScreen; [im(i).texture rect(i,:)]];
    end

    % Present white bar to signal that the stimulus is presented, in case ephys is selected
    if(ephys == 1)
        Screen('FillRect', WINDOW.handle, [255 255 255], WINDOW.ephys.photodiode);
    end

    %% Draw now if possible
    if(dontflip == 0)
        Screen('Flip', WINDOW.handle);
    end
else
    %% Clean screen
    Screen('Flip', WINDOW.handle);
end