function buffer = shuffleVectorRestricted(in, constrains, maxRep)
% This function randomizes a given vector using several constrains.
% The number of contrains is not limited, but should not exceed a
% reasonable level, otherwise randomisation is not possible and an error
% occurs. Contrains have to be defined as rows in a matrix. The number of
% rows define the number of constrains.
%
% Each element of the input vector must be assigned to a group
% (at least 2). Afterwards it can be defined how often an element of the
% group may appear one after the other.
%
% Example:
% Input vector:  1  2  3  4  5  6  7  8  9 10 11 12
% Group A     :  1  1  1  1  2  2  2  2  3  3  3  3
% Group B     :  1  1  1  1  1  1  2  2  2  2  2  2
% As an example the 5th element in the vector is a member of 2 in group A
% and a member of 1 in group B. To make ture that the vector is randomized
% in a way that each member of a group does not appear more than two times
% the resulting vector would look like this:
% Output vector: 11  6  4  9  7  3  1 10  2  8  5 12
% Group A      :  3  2  1  3  2  1  1  3  1  2  2  3
% Group B      :  2  1  1  2  2  1  1  2  1  2  1  2
%
% Usage: buffer = shuffleVectorRestricted(in, constrains, maxRep)
%
% Example call: shuffleVectorRestricted(1:12, [1 1 1 1 2 2 2 2 3 3 3 3; 1 1 1 1 1 1 2 2 2 2 2 2] , 2)
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x
%
% See also: randomOrder, nBackBuffer

% Tobias Otto
% 1.1
% 16.06.2020

% 04.05.2020, Tobias: first draft
% 16.06.2020, Tobias: maxRep may now be a vector for different restrictions
% 07.04.2022, Tobias: removed number of loops as output

%% Check input variables
inLen   = length(in);
siz     = size(constrains);

if(inLen ~=  siz(2))
    error('The number of columns of the contrains matrix and the input vector have to be identical!');
end

%% First randomize input number and restriction vectors to have a good start
index	= randperm(inLen);
% Create buffer
buffer  = [in; constrains];
% Randomize buffer
buffer  = buffer(:, index);

%% Eliminate Repetitions
i       = 1;
z       = 1;
good    = 0;
contLen = size(buffer, 1)-1;
cont    = zeros(1, contLen);
% Check length of max repetitions and create vector if necessary
if(length(maxRep) == 1)
    maxRep = maxRep * ones(1, contLen);
end

if(length(maxRep) ~= contLen)
    error('The number of entries in maxRep has to match the number of restrictions');
end

while(i <= (inLen-1))
    % Check for similar entries in restriction j
    for j=1:contLen
        if(buffer(j+1,i) == buffer(j+1,(i+1)))
            cont(j) = cont(j)+1;
        else
            cont(j) = 0;
        end

        %% Check number of similar entries and replace if necessary
        % Do that for every constrain separately!
        if(cont(j) == maxRep(j))
            % Replace next entry to other random position in vector
            newPos      = randperm(inLen-2, 1)+1; % New position, but not first or last!
            tmpEntry    = buffer(:,i+1);
            % Replace finally
            buffer(:,i+1)    	= buffer(:,newPos);
            buffer(:,newPos)	= tmpEntry;
            % Set counter to 1 to restart checks from beginning
            i       = 1;
            cont    = zeros(1, contLen);
            good    = 0;
            %break;
        else
            good = 1;
        end
    end

    if(good == 1)
        % Increment counter to continue
        i       = i+1;
        good    = 0;
    end

    % Global counter
    z = z+1;
end

