function out = getTouch(definedKeys)
% function [out] = getTouch(definedKeys)
%
% Reads data from the touchscreen and returns a struct containing the
% coordinates.
% This functions is usually not used for experiments, but to get access to
% a touchscreen in a function like touchBuffer. In other words using this
% function needs programming knowledge to avoid funny effects! This function
% should only be used in experiments that require a touchscreen as
% input-device, if the paradigm could also be implemented with intput-keys
% use keyBuffer instead.
%
% INPUTS
%   * definedKeys   : Vector containing the indices of the defined input
%                     area. Needed to distinguish between 'valid' answer
%                     and bad answer.
%
% OUTPUTS
%  * out          :  A struct containing all possible information
%   .globalValue  -> global coordinates of the touch (coordinates on screen)
%   .localValue   -> local coordinates in the touched image
%   .index        -> the index of the image
%   .time         -> the time of the touch
%
% EXAMPLES
% 1.
%   result = getTouch(1);
%               % Reads coordinates from touch screen with focus on area 1.
% 2.
%   result = getTouch([1 3 6 7 8]);
%               % Reads coordinates from touch screen with focus on
%               area 1, 3, 6, 7 and 8.
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x
%
% See also: touchBuffer, bIO

% VERSION HISTORY:
% Author:         Tobias Otto, Jonas Rose
% Version:        1.3
% Last Change:    15.06.2022

% 19.06.2017, Tobais: release version for biopsy toolbox 2
% 20.06.2017, Tobias: added wait for button release (only for Raspberry Pi)
% 05.10.2021, Dorian: added getMouseTracking if SETUP.trainingSetup is 'box'
% 15.06.2022, Tobias: replaced strcmpi check (arena or box) with logical check (new setup ID introduced)

%% BUGS/ TODO
% - Check help

%% Get Buffers to read from
global SETUP
global WINDOW
out.globalValue     = [0 0];
out.localValue      = [0 0];
out.index           = 0;
out.time            = 0;

%% Check for a click event
if(SETUP.trainingSetupID == 2)  % SETUP.trainingSetupID = 2 --> box
    %imitates GetMouse via tracking coordinates instead of click coordinates
    [x,y,buttons] = getMouseTracking;
else
    [x,y,buttons] = GetMouse(WINDOW.handle);
end

if(any(buttons))
    % Copy data to SETUP struct
    SETUP.touchScreen.lastGlobalValue   = SETUP.touchScreen.globalValue;
    SETUP.touchScreen.globalValue    	= [x y]./WINDOW.rect(3:4);  % Normalize click position
    SETUP.touchScreen.lastTime          = SETUP.touchScreen.time;
    SETUP.touchScreen.time              = GetSecs;

    % Assumption: We only have a valid  touch if:
    % * the time between two touches is greater than SETUP.touchScreen.deltaT
    % * the distance between two touches is greater than SETUP.touchScreen.deltaP
    if(SETUP.touchScreen.lastTime+SETUP.touchScreen.deltaT < SETUP.touchScreen.time || ...
            all(abs(SETUP.touchScreen.globalValue - SETUP.touchScreen.lastGlobalValue) > SETUP.touchScreen.deltaP))

        % Copy values to result struct
        out.time        = SETUP.touchScreen.time;
        out.globalValue = SETUP.touchScreen.globalValue;

        %% Find index of image or screen
        for i=definedKeys
            if(SETUP.touchScreen.globalValue(1) > SETUP.screen.X(i) && ...
                    SETUP.touchScreen.globalValue(1) < SETUP.screen.X(i)+SETUP.screen.width(i) && ...
                    SETUP.touchScreen.globalValue(2) > SETUP.screen.Y(i) && ...
                    SETUP.touchScreen.globalValue(2) < SETUP.screen.Y(i)+SETUP.screen.height(i))
                % Get local coordinates
                out.localValue(1)   = (out.globalValue(1)-SETUP.screen.X(i))/SETUP.screen.width(i);
                out.localValue(2)   = (out.globalValue(2)-SETUP.screen.Y(i))/SETUP.screen.height(i);
                out.index           = i;
                break
            end
        end

        % Define values for pecks on off pecks
        if(out.index == 0)% || ~any(out.index == definedKeys))
            out.localValue                  = [-1 -1];
            out.index                       = SETUP.touchScreen.offIndex;
            SETUP.touchScreen.localValue    = out.localValue;
            SETUP.touchScreen.screenIndex   = out.index;
        end

        % Plot info on command window
        if(SETUP.touchScreen.display == 1)
            if(out.index == SETUP.touchScreen.offIndex)
                disp(['Image # -1 | Global: ' num2str(out.globalValue)])
            else
                disp(['Image # ' num2str(i) ' | Global: ' num2str(out.globalValue) ' | Local: ' num2str(out.localValue)])
            end
        end
    end
end

%% Wait for a button release on mouse
% Necessary for Raspberry PI with octave
while(any(buttons))
    [~,~,buttons] = GetMouse(WINDOW.handle);
end

