function total = touchBuffer(duration,varargin)

% !!! This function should only be used in experiments that require a
% touchscreen as input-device, if the paradigm could also be implemented
% with input-keys use keyBuffer instead. !!!
% This buffer terminates if a specified number of inputs on "good keys" or "bad
% keys" is reached. To use the function specify at least the pin numbers of
% the good or bad keys and the number of inputs. Requires a call of myHardwareSetup
% and initWindow.
%
% This version of the touchBuffer does not save all frames of the camera
% tracking data (too time consuming, affects online tracking). To save
% GazeData create interface with point grey video tracker and save data in
% memory-mapped file (requires second Matlab or compiled function).
%
%
% INPUTS
%  * duration      : The time to wait anyway.
%
% INPUTS-OPTIONAL
%  * 'goodKey'      : vector of screen key numbers and scalar of response amount
%  * 'badKey'       : vector of screen key numbers and scalar of response amount
%  * 'neutralKey'   : vector of screen key numbers and scalar of response amount
%  * 'timeoutBad'   : increase the waiting duration the given time in s for
%                   : each response to a bad key
%  * 'feedbackBad'  : give visual feedback for each response to a bad key
%                   : pass the duration of the feedback and the stimiulus.
%  * 'acquireGaze'  : two methods to define coordinates: coordinate-limits or logical matrix
%                   : 1. pass the coordinate-limits n*6
%                   : [posXMin posXMax posYMin posYMax angleMin angleMax; ...
%                   :  posXMin posXMax posYMin posYMax angleMin angleMax]
%                   : 2. pass matrix containing true/false (size corresponding
%                   :  to tracker image ROI)
%                   : Acquires gaze-tracking, check each row of inputs.
%                   : keyBuffer returns when within limits for #Frames.
%                   : requires:
%                   :  * SETUP.tracker.bufFle % one row for each input row
%                   :  * SETUP.tracker.frameRate
%  * 'holdGaze'     : two methods to define coordinates: coordinate-limits or logical matrix
%                   : 1. pass the coordinate-limits n*6
%                   : [posXMin posXMax posYMin posYMax angleMin angleMax; ...
%                   :  posXMin posXMax posYMin posYMax angleMin angleMax]
%                   : 2. pass matrix containing true/false (size corresponding
%                   :  to tracker image ROI)
%                   : Checks if gaze-tracking is held, each row of inputs.
%                   : keyBuffer returns when outside limits for #Frames.
%                   : requires:
%                   :  * SETUP.tracker.bufFle % one file for each input row
%                   :  * SETUP.tracker.frameRate
%  * 'nrFrames'     : the number of frames to integrate in gaze tracker,
%                   : default is 1
%  * 'remoteStop'   : a remote computer can interrupt the execution via network
%  * 'sendNetID'    : send the clients network ID after a *successful*
%                     response to a control computer (network init required)
%
% OUTPUTS
%   *total    -> A struct containing a lot of information
%    .raw           -> index number of key including the time
%    .goodKey       -> A vector containing the overall responses on good keys
%    .badKey        -> The same for bad keys
%    .neutralKey  	-> The same for neutral keys
%    .offkey        -> The same for not defined keys
%    .rt            -> Time of the function to finish
%    .localValue	-> local coordinate of response (image coord. system),
%                       including time
%    .globalValue 	-> global coordinate of response (screen coord. system),
%                       including time
% OUTPUTS-OPTIONAL
%   *total      -> additional fields depending on optional inputs
%    .gaze      	-> true if keyBuffer terminated because of tracker
%    .gazeTime      -> the time of gaze-tracker event
%                   -> if nrFrames>1, time of LAST frame violateing criterion
%
% EXAMPLES
% 1.
%   result = touchBuffer(10,'goodkey',[1 2],3);
%   touchBuffer terminates after 10 seconds OR after three responses on
%   keys 1 or 2. All other responses are buffered, but don't
%   terminate keybuffer
%
% 2.
%   result = touchBuffer(10,'badkey',[3 4],8,'goodkey',[1 2],3);
%   touchBuffer terminates after 10 seconds OR after three responses on
%   keys 1 or 2 OR after eight responses on keys 3 or 4
%
% 3.
%   result = touchBuffer(10);
%   touchBuffer terminates after 10 seconds, irrspective of resoponses
%
% 4.
%   result = touchBuffer(10, 'feedbackBad', badStimulus, 1.5);
%   touchBuffer terminates after 10 seconds, after a bad response the
%   stimulus badStimulus is shown for 1.5 seconds. The screen content will
%   be recovered afterwards
%
% 5.
%   result = touchBuffer(10, 'acquireGaze', posMatrix);
%   touchBuffer terminates after the subject was tracked within coordinates
%   specified in the logical array posMatrix or after 10 seconds. The
%   logical matrix needs to correspond to the video tracker size.
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x
%
% See also:  keyBuffer, getTouch, switchBuffer, freeGetTouch

% VERSION HISTORY:
% Author:         Tobias Otto, Jonas Rose
% Version:        1.10
% Last Change:    16.10.2023
%
% 20.06.2017: Tobias: first draft for Biopsy Toolbox 2
% 08.11.2017, Tobias: added optional input 'timeoutBad' from Jonas
% 09.11.2017, Tobias: added optional input 'feedbackBad' from Jonas and copied gaze code from Jonas
% 04.12.2017, Tobias: adjusted output for no response
% 08.12.2017, Tobias: added option to stop switchBuffer via network
% 11.12.2017, Tobias: added option to send network ID to control computer
% 26.04.2018, Tobias: added input keys ... to output struct as additional information
% 26.03.2019, Lukas:  Fix for camera tracking, copied from older version of touchBuffer
% 17.07.2019, Tobias: now just checking the given pecking keys, since we allow overlapping pecking keys
% 27.03.2020, Aylin:  added option to use matrix as trackLim coordinates, updated camera tracking
% 06.03.2022, Tobias: renamed intern function name to touchBuffer again and added OTBR license
% 16.10.2023, Dorian: Fix compability issues between arena tracking and hold/acquireGaze

%% BUGS/ TODO
% - Check help
% - Check tracking with multiple cameras


%% Get access to global struct
global SETUP
global WINDOW
global NETWORK

%% Init variables
goodKey             = 0;
badKey              = 0;
neutralKey          = 0;
numOfGoodPecks      = inf;
numOfBadPecks       = inf;
numOfNeutralPecks	= inf;
goodPecked          = 0;
badPecked           = 0;
neutralPecked       = 0;
hSound              = [];
isSound             = 0;
addTimeBadPeck      = 0;
timeBadPeck         = 0;
acqrGaze            = false;
holdGaze            = false;
matrix              = false;
nrFrames            = 1;
remoteStop          = 0;
sendNetID           = 0;
%numKeys             = 1:length(SETUP.screen.X);
gazeOk              = true;
total = struct('raw', [], 'goodKey', 0, 'badKey', 0, 'neutralKey', 0, ...
    'offKey', 0, 'rt', 0, 'globalValue', [], 'localValue', []);

%% Get variable input arguments
i=0;
while(i~=length(varargin))
    i=i+1;
    switch lower(varargin{i})
        case 'goodkey'
            % Get the good keys
            i       = i+1;
            goodKey = varargin{i};

            % Is there an optional number of good pecks specified?
            try
                if(isa(varargin{i+1},'double')) % Work with i+1, because it's only a try
                    i               = i+1;
                    numOfGoodPecks  = varargin{i};
                else
                    disp(' ================================================');
                    disp(' Please define a number of good inputs ');
                    disp(' as separate argument');
                    disp(' ================================================');
                    error('Please solve problem and try again');
                end
            catch
                disp(' ================================================');
                disp(' Please define a number of good inputs ');
                disp(' as separate argument');
                disp(' ================================================');
                error('Please solve problem and try again');
            end
        case 'badkey'
            % Get the bad keys
            i       = i+1;
            badKey  = varargin{i};

            % Is there an optional number of bad pecks specified?
            try
                if(isa(varargin{i+1},'double'))
                    i               = i+1;
                    numOfBadPecks   = varargin{i};
                else
                    numOfBadPecks   = 1;
                end
            catch
                disp(' ================================================');
                disp(' Please define a number of bad inputs ');
                disp(' as separate argument');
                disp(' ================================================');
                error('Please solve problem and try again');
            end
        case 'neutralkey'
            % Get the neutral keys
            i       = i+1;
            neutralKey = varargin{i};

            % Is there an optional number of good pecks specified?
            try
                if(isa(varargin{i+1},'double')) % Work with i+1, because it's only a try
                    i               = i+1;
                    numOfNeutralPecks  = varargin{i};
                else
                    disp(' ================================================');
                    disp(' Please define a number of neutral inputs ');
                    disp(' as separate argument');
                    disp(' ================================================');
                    error('Please solve problem and try again');
                end
            catch
                disp(' ================================================');
                disp(' Please define a number of neutral inputs ');
                disp(' as separate argument');
                disp(' ================================================');
                error('Please solve problem and try again');
            end
        case 'sound'
            % Get the sound file handle
            i       = i + 1;
            hSound  = varargin{i};
            isSound = 1;
        case 'timeoutbad'
            % increase the waiting time for each response to a badKey
            i               = i + 1;
            addTimeBadPeck  = varargin{i};
        case 'feedbackbad'
            % give a visual feedback for each response to a badKey
            i          	= i + 1;
            stimBadPeck	= varargin{i};
            try
                if(isa(varargin{i+1},'double'))
                    i         	= i+1;
                    timeBadPeck	= varargin{i};
                else
                    disp(' ================================================');
                    disp(' Please define the time in seconds to display the');
                    disp(' stimulus as separate argument');
                    disp(' ================================================');
                    error('Please solve problem and try again');
                end
            catch
                disp(' ================================================');
                disp(' Please define the time in seconds to display the');
                disp(' stimulus as separate argument');
                disp(' ================================================');
                error('Please solve problem and try again');
            end
        case 'acquiregaze'
            % acquire gaze in tracker
            i           = i + 1;
            acqrGaze    = true;
            trackLim    = varargin{i};
            if size(trackLim,2) > 6
                matrix = true;
            end

        case 'holdgaze'
            % hold gaze in tracker
            i           = i + 1;
            holdGaze    = true;
            trackLim    = varargin{i};
            if size(trackLim,2) > 6
                matrix = true;
            end

        case 'nrframes'
            % number of frames to integrate in gaze-tracker
            nrFrames    = varargin{i};
            i           = i + 1;
        case 'remotestop'
            remoteStop  = 1;
        case 'sendnetid'
            sendNetID = 1;
        otherwise
            disp(' ================================================');
            disp([' Unknown argument: ' varargin(i)]);
            disp(' ================================================');
            i = i+1;
    end
end

%% Init variables again, after we know the number of keys
total.goodKey       = zeros(1,length(goodKey));
total.badKey        = zeros(1,length(badKey));
total.neutralKey 	= zeros(1,length(neutralKey));
startTime           = GetSecs;
duration            = duration+startTime;
time                = 0;
numKeys             = [goodKey badKey neutralKey];  % Only use defined keys to look for
numKeys             = numKeys(numKeys~=0);

%% Setup the tracker
if acqrGaze || holdGaze
    % there is one memory mapped file for each camera
    %%%     trackDim        = 1:length(SETUP.tracker.bufFle); - OLD VERSION: INCORRECT? ######################
    trackDim        = length(SETUP.tracker.bufFle);     % change Aylin #####################
    % tracker in right area (initial value different for acquire and hold)
 %   gazeIn          = [~acqrGaze & holdGaze; ...
 %       ~acqrGaze & holdGaze; ...
 %        ~acqrGaze & holdGaze];
    % number of frames that have satisfied the current criterion
    gazeFrms        = 0;
    % get the index of the current frame from the first tracker
    firstFrm        = SETUP.tracker.bufFle{1}.Data(7);
    lastFrm         = firstFrm;
    % init output, touchbuffer terminated because of gaze event
    total.gazeEvent = false;
end

%% Start main loop for buffering pecks
%  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
while((time < duration) && goodPecked ~= numOfGoodPecks && ...
        badPecked ~= numOfBadPecks && neutralPecked ~= numOfNeutralPecks &&...
        gazeOk)
    %% Get actual time
    time = GetSecs;

    %% Check network to stop loop
    if(remoteStop == 1)
        % Check for valid data
        data = pnet(NETWORK.con, 'read', 1, 'int8', 'noblock');
        % If value is valid, stop loop and return number
        if(data == 31)
            pnet(NETWORK.con, 'write', int8(32));
            break;
        end
    end

    %% Read gaze tracker if there is a new frame on the first tracker
    if (acqrGaze || holdGaze) && lastFrm < SETUP.tracker.bufFle{1}.Data(7)
        % the index of the current frame (first tracker)
        lastFrm = SETUP.tracker.bufFle{1}.Data(7);

        % check each tracker dimension
        for i = 1 : trackDim

            % input is true/false matrix or not?
            if matrix	% input matrix 'true'
                % matrix with same size as image from videoTracker (> than n*6)
                % use coordinates of tracked object as indices for matrix,
                % output true/false (e.g. round center with true, rest false)
                if isnan(SETUP.tracker.bufFle{i}.Data(1)) || isnan(SETUP.tracker.bufFle{i}.Data(2))
                    gazeIn = false;
                else
                    %%% Pay attention to the assignment of X,Y coordinates and matrix indexing
                    % SETUP.tracker.bufFle{i}.Data(1): X coordinate, equals column in matrix
                    % SETUP.tracker.bufFle{i}.Data(2): Y coordinate, equals row in matrix
                    % indexing in matrix: row,column - matrix(Y,X)
                    gazeIn(i) = trackLim(round(SETUP.tracker.bufFle{i}.Data(2)),...	% Y = row of matrix
                        round(SETUP.tracker.bufFle{i}.Data(1)));                    % X = column of matrix
                end
            else        % input: corrdinate-limits n*6 (matrix = 'false')
                % matrix containing [posXMin posXMax posYMin posYMax angleMin angleMax]
                % all values larger min & smaller max
                gazeIn(i) = ...
                    SETUP.tracker.bufFle{i}.Data(1)>trackLim(i,1) && ... % x
                    SETUP.tracker.bufFle{i}.Data(1)<trackLim(i,2) && ...
                    SETUP.tracker.bufFle{i}.Data(2)>trackLim(i,3) && ... % y
                    SETUP.tracker.bufFle{i}.Data(2)<trackLim(i,4) && ...
                    abs(SETUP.tracker.bufFle{i}.Data(3))>trackLim(i,5) && ... % angle
                    abs(SETUP.tracker.bufFle{i}.Data(3))<trackLim(i,6);
            end
        end

        %% Acquire: count frames that satisfy the criterion in all tracker
        if acqrGaze && all(gazeIn)
            gazeFrms = gazeFrms+1;
        elseif acqrGaze
            % reset counter otherwise
            gazeFrms = 0;
        end
        % hold: count frames that violate the criterion in any tracker
        if holdGaze && any(~gazeIn)
            gazeFrms = gazeFrms+1;
        elseif holdGaze
            % reset counter otherwise
            gazeFrms = 0;
        end

        if gazeFrms > nrFrames
            % this will halt the loop
            gazeOk          = false;
            % the time of the LAST frame that satisfied the criterion
            total.gazeTime  = toc;
            total.gazeEvent = true;
        end
    end

    %% Read from pecking keys
    peck = getTouch(numKeys);

    % Is peck equal to any of the elements in inputKeys?
    if(peck.index ~= 0)
        %% Update variables
        total.raw           = cat(1, total.raw, [time peck.index]);
        total.localValue    = cat(1, total.localValue, [time peck.localValue]);
        total.globalValue   = cat(1, total.globalValue, [time peck.globalValue]);
        total.goodKey       = total.goodKey + (goodKey == peck.index);
        total.badKey        = total.badKey + (badKey == peck.index);
        total.neutralKey  	= total.neutralKey + (neutralKey == peck.index);
        total.offKey        = total.offKey + (SETUP.touchScreen.offIndex == peck.index);
        % Helper variables for the while loop to exit
        goodPecked          = sum(total.goodKey);
        badPecked           = sum(total.badKey);
        neutralPecked       = sum(total.neutralKey);

        %% Check for sound feedback
        if (isSound == 1)
            play(hSound);
        end

        %% Increment the duration on badKey responses
        if (any(badKey == peck.index) && addTimeBadPeck)
            duration = duration + addTimeBadPeck;
        end

        %% Show a negative feedback stimulus
        if (timeBadPeck ~= 0 && any(badKey == peck.index))
            % Show the negative feedback image
            Screen('DrawTexture', WINDOW.handle, stimBadPeck.texture, [], WINDOW.rect);
            Screen('Flip', WINDOW.handle);

            % Wait given time
            WaitSecs(timeBadPeck);

            % Recover screen content
            % Finally plot image
            for i = 1 : size(WINDOW.stimulus.onScreen,1)
                Screen('DrawTexture', WINDOW.handle, WINDOW.stimulus.onScreen(i, 1), [], WINDOW.stimulus.onScreen(i, 2:end));
            end
            Screen('Flip', WINDOW.handle);
        end

        %% Do we have to send the network ID
        if (sendNetID == 1)
            sendNetID = 2;
        end
    end
end

%% Get stop time and check output struct
total.rt = GetSecs - startTime;

% Add the input key numbers to output struct
total.definedGoodKey             = goodKey;
total.definedBadKey              = badKey;
total.definedNeutralKey          = neutralKey;

if(isempty(total.raw))
    total.raw           = [0 0];
    total.globalValue   = [0 0 0];
    total.localValue    = [0 0 0];
end

%% Send network ID
if(sendNetID == 2)
    pnet(NETWORK.con, 'write', int8(NETWORK.netID));
end
