%% Create conversion matrix

% This script creates a conversionMatrix which translates tracked coordinates of the cameras
% to touched positions on the screen. This script is necessary to use camera tracking in a Skinner Box, to register
% screen packs and touches on a setup without a touchscreen.
% CameraTracking_Birds accounts for more natural movements of birds/other lab animals(beak tilt, tendencies to peck edges of stimuli etc.).
% CameraTracking_Humans registers precise and perpendicular pointing on the screen and is therefore more accurate in ideal situations.
% For more info on how to use tracking check https://gitlab.ruhr-uni-bochum.de/ikn/OTBR/-/wikis/How-tos#tracking
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x

% VERSION HISTORY:
% Author:  		Dorian Roeders, Farina Lingstaedt
% Version: 		1.1
% Last Change:  22.3.2022

% 30.11.2021, Dorian, Farina: first version
% 22.03.2022, Dorian: Transform parts of the script to functions and removimg hard coded parts

%% Precalibration for the birds,
% you need to do this part yourself, the code section afterwards is for the birds
% hold a point black tool on the center of the white dots for as long as they
% are present, this gives a general range in which the birds pecks are
% registered in the next part.

initOTBR; % initialization of toolbox
initWindow(); % opens window for stimulus presentation, input: MS-Windows display number
whiteDot     = loadImage(''); % specify image of dot to be used

res1=dotPresentationHuman([800 1200 600 800],[200], [140], whiteDot);


%% create vectors used to create conversion matrix in next step, use the bird Matrix for this

% find rows and cols of positions the dot was presented on:
% since matrix is a length by height matrix of the screen in pixel, the row
% and col are the according positions of the dot on the screen
[row, col] = find(~cellfun(@isempty,res1));

% loop thorugh all presented dots:
for i = 1:length(row)

    if res1{row(i,1),col(i,1)} ~= 0 % the dot was tracked
        vals = res1{row(i,1),col(i,1)}; % vals contains values from both cameras (changes in each iteration)
        val(i,1)= vals(1); % first column of val contains value from camera 1
        val(i,2)= vals(2); % second column of val contains value from camera 2
    else % the dot was not tracked
        val(i,1) = 0;
        val(i,2) = 0;
    end

end

position = [row, col]; % combine row and col to position matrix

% val(val<23) = 0; % exclude artifacts

plot(val) % sanity check: one curve should relatively steadily increase (with plateaus) while the other should oszilate
%% Let a bird do this and it will callibrate your screen

initOTBR; % initialization of toolbox
initWindow(); % opens window for stimulus presentation, input: MS-Windows display number
whiteDot     = loadImage(''); % specify image of dot to be used

res2=dotPresentationBirds([800 1200 600 800],[200], [140], whiteDot, res1, 50);
%% create vectors used to create conversion matrix in next step, use the bird Matrix for this

% find rows and cols of positions the dot was presented on:
% since matrix is a length by height matrix of the screen in pixel, the row
% and col are the according positions of the dot on the screen
[row, col] = find(~cellfun(@isempty,res2));

% loop thorugh all presented dots:
for i = 1:length(row)

    if res2{row(i,1),col(i,1)} ~= 0 % the dot was tracked
        vals = res2{row(i,1),col(i,1)}; % vals contains values from both cameras (changes in each iteration)
        val(i,1)= vals(1); % first column of val contains value from camera 1
        val(i,2)= vals(2); % second column of val contains value from camera 2
    else % the dot was not tracked
        val(i,1) = 0;
        val(i,2) = 0;
    end

end

position = [row, col]; % combine row and col to position matrix

% val(val<23) = 0; % exclude artifacts

plot(val) % sanity check: one curve should relatively steadily increase (with plateaus) while the other should oszilate

%% create conversion matrix with the values at hand
matrixCalculation(res2,row,col,val);
