classdef videoTracker < handle

    % vT = videoTracker(vInNme, vInNr, vInFormat, savFName)
    %
    % Videotracker object. Detects a bright object on dark background.
    % Can be changed to detect dark object on bright background.
    % Writes to a memory mapped file (camera serial _dat.txt), writes
    % [x, y, angle, frame#, time]. Memory mapped files can be read by
    % touchBuffer etc. This works between different Matlab instances.
    % Currently this function expects bw-camera input. Object can be run
    % multiple times for multiple cameras, ideally from compiled exe or
    % different Matlab instance to run on different physical processors.
    % Currently tested only on Point Grey (Chameleon3 CM3-U3-13Y3M) camera.
    %
    % To control acquisition from a different Matlab instance use memory
    % mapped file (camera serial _ctrl.txt). It has four double entries:
    %   1. if set to -1 object will be destroyed and acquisition stopped
    %   2. if set to -1 (default) no 'raw' data will be saved on hdd but
    %      data is availabe for reading from memory mapped file.
    %   3 & 4. values will be saved on hdd. Use to send trial number etc.
    %
    % INPUTS
    % vInNme        : name of viedeo device (e.g. 'winvideo', 'pointgrey')
    % vInNr         : index of video device (e.g. 1 selects first pointgrey)
    % vInFormat     : video format, find using:
    %               : devInfo = imaqhwinfo('pointgrey',1);
    %               : devInfo.SupportedFormats
    %
    % INPUTS OPTIONAL
    % savFName      : if file name provided, raw data is saved
    %
    % METHODS
    % vt.preview    : shows video and tracking, use to configure the camera
    %               : writes _cfg.mat
    % vt.start      : starts tracking and writing to memory mapped file
    % vt.stop       : stop tracker, can still preview or start
    % vt.destroy    : destroy tracker
    %
    %
    % Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
    % Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
    % Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
    % Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
    %               https://doi.org/10.3758/s13428-023-02199-x

    % VERSION HISTORY:
    % Author:         Jonas Rose
    % Version:        1.13
    % Last Change:    15.06.2022
    %
    % Feb 5 2018 : Jonas    : config and mmapped files are now based on
    %                         camera serial
    % Feb 7 2018 : Jonas    : ctrl file to control the behavior of the
    %                         video-tracker
    % Mid 2018   : Jonas    : can now be sued with multiple cameras, mmfile
    %                         names are based on camera serial (must be
    %                         readable)
    % Sep 25 2018: Jonas    : added obj.trackWhite : logical to control if
    %                         white marker on black background (1) or black
    %                         object on light background (0) e.g in arena
    % sep 26 2018: Jonas    : added option to save 'raw' blob-data in file
    % Jul 21 2021: Lukas,   : adapted traking to the plus-shaped arena. Added logical masks that restrict the area
    %              Sara       available for tracking. All changes are written between %### bars
    % Aug 04 2021 : Aylin   : prepared option to track black and white simulatenously
    % Aug 05 2021 : Aylin   : modified mask for tracking. A logical matrix
    %                         called 'mask' needs to be saved as: 'noMask'
    %                         'maskW' 'maskE' 'maskN' 'maskS' 'maskHexagon'
    %                         and copied within the relevant folder.
    % Aug 09 2021 : Aylin   : added code for simultaneous black and white
    %                         tracking
    % Aug 18 2021 : Aylin   : removed ROI button (replaced by mask), ROI
    %                         set to default camera image (640 x 512)
    % Sep 15 2021 : Aylin   : added SETUP.trainingSetup to specify which
    %                         frameAcquired function should be used (set in
    %                         myHardwareSetup to either 'box' or 'arena'
    % Sep 22 2021 : Dorian  : added global setup to VideoTracker
    %                         constructor, changed FrameRateMode to
    %                         'Manual' and added read out of FrameRate from
    %                         myHardwareSetup
    % 15.06.2022, Tobias    : replaced strcmp with strcmpi; switched fom else to elseif for frameAcquired function
    %
    %
    % BUGS/ TODO:
    % - add examples
    % - maybe add framerate to preview
    % - change myHardwareSetup to specify which frameAcquired function should be used

    %% PROPERTIES
    properties
        % these can not be changed after startTracker
        camHand                             % camera handle
        camSrc                              % camera source object
        camRoi          = [0 0 640 512];    % region of interest (finds defaults later) %####
        camBright       = 2;                % brightness
        camGain         = 0.1;              % gain
        camGamma        = 1.1;              % gamma
        camShutter      = 2;                % shutter
        trckCol         = 'black';          % object color for tracking (black/white/both) %#### BW Aylin

        % default values (if not yet defined in parameter file)
        blobSizeB       = 100;              % minimal size of hblob
        blobSizeW       = 100;
        blobThresholdB  = 100;              % threshold binary conversion image analysis
        blobThresholdW  = 100;

        % mask as additional property, dimensions = fullsize of video tracker
        noMask          = true(512,640);    % full camera image (default = no mask) %####
        maskID          = 'noMask';         % specifies selected mask (no mask) %####
        mask            = true(512,640);	% mask from previous config file %####

        blobHand

        datOutHand                          % handle data output object
        datInHand                           % handle data input (control) object
        datOutNme       = 'defaultCam_dat.txt';     % file name/ ip address
        datInNme        = 'defaultCam_ctrl.txt';    % tracker-control file
        paramFle        = 'defaultCam_cfg.mat';     % default parameters file

        savData         = false;            % if frameAcquired saves data
        savDatFleNme                        % file name for saving
        savDatFle                           % file handle for saving

        running         = false;            % is the tracker running?

    end

    %% METHODS
    methods

        %% constructor
        function obj = videoTracker(vInNme, vInNr, vInFormat, savFName)

            global SETUP

            % check if required input is provided
            if nargin<3
                error('Not enough input arguments, check help.')
            elseif nargin>3
                % pick up the optional input to save raw data to file
                obj.savData         = true;
                obj.savDatFleNme    = savFName;
            end

            % create video input object
            obj.camHand = videoinput(vInNme, vInNr, vInFormat);

            % configurations for the frame acquired function
            triggerconfig(obj.camHand,'immediate');
            obj.camHand.FramesAcquiredFcnCount  = 1;
            obj.camHand.FramesPerTrigger        = inf;

            % which frameAcquired function should be used defined in myHardwareSetup %#### Aylin
            % SETUP.trainingSetup = 'box' or 'arena';
            if strcmpi(SETUP.trainingSetup,'arena')
                obj.camHand.FramesAcquiredFcn   = {'frameAcquired_Arena',obj};
            elseif strcmpi(SETUP.trainingSetup,'box')
                obj.camHand.FramesAcquiredFcn   = {'frameAcquired_Box',obj};
            end

            % camera source object
            obj.camSrc  = getselectedsource(obj.camHand);

            % set camera to manual so that settings don't change unexpectedly
            if isprop(obj.camSrc,'ExposureMode')
                obj.camSrc.ExposureMode            = 'Manual';
            end
            if isprop(obj.camSrc,'GainMode')
                obj.camSrc.GainMode                = 'Manual';
            end
            if isprop(obj.camSrc,'ShutterMode')
                obj.camSrc.ShutterMode             = 'Manual';
            end
            if isprop(obj.camSrc,'FrameRateMode')
                obj.camSrc.FrameRateMode           = 'Manual';
            end
            if isprop(obj.camSrc,'FrameRatePercentage')
                obj.camSrc.FrameRatePercentage     = 100;       % 1:100
            end
            if isprop(obj.camSrc,'FrameRate')
                obj.camSrc.FrameRate               = SETUP.tracker.frameRate;
            end
            obj.camRoi = obj.camHand.roiPosition; % full camera image as ROI %####

            % use the camera serial to identify the config-file
            obj.paramFle  = [obj.camSrc.SerialNumber '_cfg.mat'];
            obj.datInNme  = [obj.camSrc.SerialNumber '_ctrl.txt']; % mmf
            obj.datOutNme = [obj.camSrc.SerialNumber '_dat.txt'];  % mmf

            % load parameters
            if exist(obj.paramFle,'file')
                load(obj.paramFle);
                % load mask and target color from already existing config file
                % overwrite predefined mask (full screen) and target color (black)
                obj.mask    = mask;
                obj.maskID  = maskID; %####
                obj.trckCol = trckCol; %####
            else
                load('defaultCam_cfg');
                warning('No camera config-file found, creating with default values')
            end

            if exist('thB','var') && exist('thW','var') && exist('szeB','var') && exist('szeW','var')
                obj.blobThresholdB  = thB;
                obj.blobThresholdW  = thW;
                obj.blobSizeB       = szeB;
                obj.blobSizeW       = szeW;
            end
            obj.camBright       = brght;    % brightness
            obj.camGain         = gain;     % gain
            obj.camGamma        = gmma;     % gamma
            obj.camShutter      = shut;     % shutter

            %  apply defaults / loaded values to camera
            obj.camHand.roiPosition       = obj.camRoi; % full camera image
            if isprop(obj.camSrc,'Brightness')
                obj.camSrc.Brightness     = obj.camBright;
            end
            if isprop(obj.camSrc,'Gain')
                obj.camSrc.Gain           = obj.camGain;
            end
            if isprop(obj.camSrc,'Gamma')
                obj.camSrc.Gamma          = obj.camGamma;
            end
            if isprop(obj.camSrc,'Shutter')
                obj.camSrc.Shutter        = obj.camShutter;
            end

            % computer vision toolbox blob analysis object
            obj.blobHand                        = vision.BlobAnalysis;
            obj.blobHand.AreaOutputPort         = false;
            obj.blobHand.BoundingBoxOutputPort  = false;
            obj.blobHand.OrientationOutputPort  = true;
            obj.blobHand.MaximumCount           = 1;
        end

        %% start tracking
        function [] = start(obj)
            obj.running = true;
            % create the memory mapped file for data exchange
            if ~exist(obj.datOutNme,'file')
                f = fopen(obj.datOutNme,'w+');
                fwrite(f,[0 0 0 0 0 0 0 0],'double'); %#### BW Aylin
                fclose(f);
            end
            obj.datOutHand = memmapfile(obj.datOutNme,'Format','double');
            obj.datOutHand.writable = true;

            % create the memory mapped file for data reading
            if ~exist(obj.datInNme,'file')
                f = fopen(obj.datInNme,'w+');
                fwrite(f,[0 -1 0 0],'double');
                fclose(f);
            end
            obj.datInHand = memmapfile(obj.datInNme,'Format','double');
            obj.datInHand.writable  = true;
            obj.datInHand.Data(1)   = 0;

            % open a file for saving 'raw' data
            if obj.savData
                obj.savDatFle = fopen([obj.savDatFleNme '.bin'],'w+');
            end

            % start the video input object
            start(obj.camHand);
        end

        %% stop the tracker (you can still preview)
        function [] = stop(obj)
            obj.running = false;
            % stop video acquisition
            stop(obj.camHand);
        end

        %% delete the tracker
        function [] = destroy(obj)
            obj.running = false;
            % stop video acquisition
            stop(obj.camHand);
            % if 'raw' data was saved read binary and store in mat file
            if obj.savData
                fclose(obj.savDatFle);      % was open only for writing
                obj.savDatFle = fopen([obj.savDatFleNme '.bin'],'r');
                dat = fread(obj.savDatFle,'double');
                fclose(obj.savDatFle);
                % reshape and save 'raw' data in a .mat file
                dat = reshape(dat,10,[])'; %#### BW Aylin - 3 additional columns due to BW tracking
                save([obj.savDatFleNme '.mat'],'dat');
                % delete the binary file
                delete([obj.savDatFleNme '.bin']);
            end
            % delete camera object
            delete(obj.camHand);
        end

        %% test the tracker
        function [] = test(obj)
            plyr        = vision.VideoPlayer;
            obj.running = true;
            % different frame acquired function for testing
            obj.camHand.FramesAcquiredFcn = {'frameAcquiredTst',plyr};
            % show current camera image
            step(plyr,getsnapshot(obj.camHand));
            % start the video input objec
            start(obj.camHand);
        end

        %% video and tracking preview
        function [] = preview(obj)

            % configure and start the video object
            triggerconfig(obj.camHand, 'manual');
            start(obj.camHand);

            % make and place videoPlayer objects side by side %#### BW Aylin
            origVdo             = vision.VideoPlayer;
            origVdo.Position(1) = origVdo.Position(1)-origVdo.Position(3)/2;
            thVdoB              = vision.VideoPlayer; % black target
            thVdoB.Position(1)  = thVdoB.Position(1)+thVdoB.Position(3)/2;
            thVdoW              = vision.VideoPlayer; % white target
            thVdoW.Position(1)  = thVdoW.Position(1)+(thVdoB.Position(3)/2);

            % a figure for input
            fig = figure('position', [origVdo.Position(1), ...
                origVdo.Position(2)-origVdo.Position(4)/2, ...
                origVdo.Position(3)*2, ...
                origVdo.Position(4)/3], ...
                'menubar', 'none', 'visible','off');

            % parameters for button placement
            l   = .03;
            b1  = .1;
            b2  = .25;
            b3  = .35;
            b4  = .55;
            b5  = .7;
            b6  = .8;
            w   = .125;
            h   = .125;

            % text field for camera Brightness
            if isprop(obj.camSrc,'Brightness')
                uicontrol('Style', 'text', 'String', 'Brightness (0-24)',...
                    'Units', 'normalized', 'Position', [l b6 w h]);
                brtBtn = uicontrol('Style', 'edit', 'String', num2str(obj.camBright),...
                    'Units', 'normalized', 'Position', [l b5 w h],...
                    'Callback',  'set(gcbo,''userdata'',true)', 'userdata',false);
            end
            % text field for camera Gain
            if isprop(obj.camSrc,'Gain')
                uicontrol('Style', 'text', 'String', 'Gain (0-18)',...
                    'Units', 'normalized', 'Position', [2*l+w b6 w h]);
                ginBtn = uicontrol('Style', 'edit', 'String', num2str(obj.camGain),...
                    'Units', 'normalized', 'Position', [2*l+w b5 w h],...
                    'Callback',  'set(gcbo,''userdata'',true)', 'userdata',false);
            end
            % text field for camera Gamma
            if isprop(obj.camSrc,'Gamma')
                uicontrol('Style', 'text', 'String', 'Gamma (0.5-4.0)',...
                    'Units', 'normalized', 'Position', [l+2*(l+w) b6 w h]);
                gamBtn = uicontrol('Style', 'edit', 'String', num2str(obj.camGamma),...
                    'Units', 'normalized', 'Position', [l+2*(l+w) b5 w h],...
                    'Callback',  'set(gcbo,''userdata'',true)', 'userdata',false);
            end
            % text field for camera Shutter
            if isprop(obj.camSrc,'Shutter')
                uicontrol('Style', 'text', 'String', 'Shutter (0.01-7.99)',...
                    'Units', 'normalized', 'Position', [l+3*(l+w) b6 w h]);
                sutBtn = uicontrol('Style', 'edit', 'String', num2str(obj.camShutter),...
                    'Units', 'normalized', 'Position', [l+3*(l+w) b5 w h],...
                    'Callback',  'set(gcbo,''userdata'',true)', 'userdata',false);
            end

            % button for mask %#### Sara/Aylin
            uicontrol('Style', 'text', 'String', 'Mask',...
                'Units', 'normalized', 'Position', [l+4.5*(l+w) b6 w h]);
            allMasks = {'noMask' 'maskCam1' 'maskCam2' 'maskN' 'maskS' 'maskHexagon' 'maskCustom'};
            maskNum = find(strcmp(allMasks,obj.maskID));
            maskBtn = uicontrol('Style', 'popupmenu',...
                'Units', 'normalized', 'Position', [l+4.5*(l+w) b5 w h],...
                'Value', maskNum, 'String', allMasks,...
                'Callback',  'set(gcbo,''userdata'',true)', 'userdata',false); % selected mask: maskBtn.Value

            % button for Target object color (black/white/both) %#### BW Aylin
            uicontrol('Style', 'text', 'String', 'Target object color',...
                'Units', 'normalized', 'Position', [l+4.5*(l+w) b4-.06 w h]);
            allCol = {'black' 'white' 'both'};
            colNum = find(strcmp(allCol,obj.trckCol));
            % default: black (bckBtn.UserData = 0, bckBtn.Value = 1)
            % change of target color: white (bckBtn.UserData = 1, bckBtn.Value = 2), both (bckBtn.UserData = 1, bckBtn.Value = 3)
            bckBtn = uicontrol('Style', 'popupmenu',...
                'Units', 'normalized', 'Position', [l+4.5*(l+w) b3+.04 w h],...
                'Value', colNum, 'String', allCol,...
                'Callback', 'set(gcbo,''userdata'',true)', 'userdata',false);

            % text field Threshold
            uicontrol('Style', 'text', 'String', 'Threshold',...
                'Units', 'normalized', 'Position', [l b3 w h]);
            thBtnB = uicontrol('Style', 'edit', 'String', num2str(obj.blobThresholdB),... %#### BLACK
                'Units', 'normalized', 'Position', [l b2 w h],...
                'Callback',  'set(gcbo,''userdata'',true)');
            thBtnW = uicontrol('Style', 'edit', 'String', num2str(obj.blobThresholdW),... %#### WHITE
                'Units', 'normalized', 'Position', [l b1 w h],...
                'Callback',  'set(gcbo,''userdata'',true)');
            % text field for Blob minimum
            uicontrol('Style', 'text', 'String', 'Blob minimum',...
                'Units', 'normalized', 'Position', [l+1*(l+w) b3 w h]);
            minBtnB = uicontrol('Style', 'edit', 'String', num2str(obj.blobSizeB),... %#### BLACK
                'Units', 'normalized', 'Position', [l+1*(l+w) b2 w h],...
                'Callback',  'set(gcbo,''userdata'',true)');
            minBtnW = uicontrol('Style', 'edit', 'String', num2str(obj.blobSizeW),... %#### WHITE
                'Units', 'normalized', 'Position', [l+1*(l+w) b1 w h],...
                'Callback',  'set(gcbo,''userdata'',true)');

            % Target object coordinates
            uicontrol('Style', 'text', 'String', 'Coordinates target object',...
                'Units', 'normalized', 'Position', [l+2*(l+w) b3 w h]);
            % text field X - black
            uicontrol('Style', 'text', 'String', 'X: ',...
                'Units', 'normalized', 'Position', [l+2*(l+w) b2-.04 w/4 h]);
            xTxtB = uicontrol('Style', 'text', 'String', 'no obj',...
                'Units', 'normalized', 'Position', [1.6*l+2*(l+w) b2-.04 w/2 h]);
            % text field Y - black
            uicontrol('Style', 'text', 'String', 'Y: ',...
                'Units', 'normalized', 'Position', [4*l+2*(l+w) b2-.04 w/4 h]);
            yTxtB = uicontrol('Style', 'text', 'String', 'no obj',...
                'Units', 'normalized', 'Position', [4.6*l+2*(l+w) b2-.04 w/2 h]);
            % text field Angle - black
            uicontrol('Style', 'text', 'String', 'a: ',...
                'Units', 'normalized', 'Position', [7*l+2*(l+w) b2-.04 w/4 h]);
            aTxtB = uicontrol('Style', 'text', 'String', 'no obj',...
                'Units', 'normalized', 'Position', [7.6*l+2*(l+w) b2-.04 w/2 h]);
            % text field - black
            uicontrol('Style', 'text', 'String', 'black', 'FontWeight','bold',...
                'Units', 'normalized', 'Position', [11*l+2*(l+w) b2-.04 w/4 h]);

            % text field X - white
            uicontrol('Style', 'text', 'String', 'X: ',...
                'Units', 'normalized', 'Position', [l+2*(l+w) b1-.04 w/4 h]);
            xTxtW = uicontrol('Style', 'text', 'String', 'no obj',...
                'Units', 'normalized', 'Position', [1.6*l+2*(l+w) b1-.04 w/2 h]);
            % text field Y - white
            uicontrol('Style', 'text', 'String', 'Y: ',...
                'Units', 'normalized', 'Position', [4*l+2*(l+w) b1-.04 w/4 h]);
            yTxtW = uicontrol('Style', 'text', 'String', 'no obj',...
                'Units', 'normalized', 'Position', [4.6*l+2*(l+w) b1-.04 w/2 h]);
            % text field Angle - white
            uicontrol('Style', 'text', 'String', 'a: ',...
                'Units', 'normalized', 'Position', [7*l+2*(l+w) b1-.04 w/4 h]);
            aTxtW = uicontrol('Style', 'text', 'String', 'no obj',...
                'Units', 'normalized', 'Position', [7.6*l+2*(l+w) b1-.04 w/2 h]);
            % text field - white
            uicontrol('Style', 'text', 'String', 'white', 'FontWeight','bold',...
                'Units', 'normalized', 'Position', [11*l+2*(l+w) b1-.04 w/4 h]);

            % Create push button for stop
            stopBtn = uicontrol('Style', 'pushbutton', 'String', 'Stop Preview',...
                'Units', 'normalized', 'Position', [l+5*(l+w) b1 w h],...
                'Callback',  'set(gcbo,''userdata'',true)','UserData',false);

            % handle depending on target object color: %#### BW Aylin
            handleBlack = [thBtnB, minBtnB];
            handleWhite = [thBtnW, minBtnW];
            if strcmp(obj.trckCol,'black')% strcmp(vTrck.trckCol,'black') % black
                set(handleBlack, 'Enable', 'on');
                set(handleWhite, 'Enable', 'off');
            elseif strcmp(obj.trckCol,'white') % strcmp(vTrck.trckCol,'white') % white
                set(handleBlack, 'Enable', 'off');
                set(handleWhite, 'Enable', 'on');
            else % obj.trckCol == 'both'
                set(handleBlack, 'Enable', 'on');
                set(handleWhite, 'Enable', 'on');
            end

            % show the gui
            set(fig,'visible','on');

            % process and display each frame until stopped
            while 1
                % is required for ui control
                drawnow;
                % this stops the preview
                if stopBtn.UserData
                    origVdo.release;
                    thVdoB.release; %#### BW
                    thVdoW.release; %#### BW
                    stop(obj.camHand);
                    % reset trigger configuration
                    triggerconfig(obj.camHand,'immediate');
                    close(fig);
                    % save the current values as new defaults, Black/White separately
                    thB     = obj.blobThresholdB;
                    thW     = obj.blobThresholdW;
                    szeB    = obj.blobSizeB;
                    szeW    = obj.blobSizeW;

                    brght   = obj.camBright;
                    gain    = obj.camGain;
                    gmma    = obj.camGamma;
                    shut    = obj.camShutter;

                    % save current mask to configurations memmap file %####
                    mask    = obj.mask;     % logical mask used
                    maskID  = obj.maskID;	% ID of mask used (string)
                    trckCol = obj.trckCol;	% color of target object ('black' 'white' 'both')
                    save(obj.paramFle,'thB','szeB','thW','szeW','brght','gain','gmma','shut','trckCol','mask','maskID'); %####
                    break
                end

                % update camera brightness
                if isprop(obj.camSrc,'Brightness') && brtBtn.UserData
                    obj.camBright           = str2double(brtBtn.String);
                    stop(obj.camHand);
                    obj.camSrc.Brightness   = obj.camBright;
                    start(obj.camHand);
                end
                % update camera gain
                if isprop(obj.camSrc,'Gain') && ginBtn.UserData
                    obj.camGain             = str2double(ginBtn.String);
                    stop(obj.camHand);
                    obj.camSrc.Gain         = obj.camGain;
                    start(obj.camHand);
                end
                % update camera gamma
                if isprop(obj.camSrc,'Gamma') && gamBtn.UserData
                    obj.camGamma            = str2double(gamBtn.String);
                    stop(obj.camHand);
                    obj.camSrc.Gamma        = obj.camGamma;
                    start(obj.camHand);
                end
                % update camera shutter speed
                if isprop(obj.camSrc,'Shutter') && sutBtn.UserData
                    obj.camShutter          = str2double(sutBtn.String);
                    stop(obj.camHand);
                    obj.camSrc.Shutter      = obj.camShutter;
                    start(obj.camHand);
                end
                % update camera region of interest constant, always full camera image
                % ROI selection via mask

                % update the threshold
                if thBtnB.UserData %#### BLACK
                    obj.blobThresholdB  = str2double(thBtnB.String);
                    thBtnB.UserData     = false;
                elseif thBtnW.UserData %#### WHITE
                    obj.blobThresholdW  = str2double(thBtnW.String);
                    thBtnW.UserData     = false;
                end
                % update the minimum blob size
                if minBtnB.UserData %#### BLACK
                    obj.blobSizeB       = str2double(minBtnB.String);
                    minBtnB.UserData    = false;
                elseif minBtnW.UserData %#### WHITE
                    obj.blobSizeW       = str2double(minBtnW.String);
                    minBtnW.UserData    = false;
                end

                % update target object/background color %#### BW Aylin
                if bckBtn.UserData % target color was modified
                    stop(obj.camHand);
                    allCol = {'black' 'white' 'both'};
                    obj.trckCol = allCol{bckBtn.Value}; % bckBtn.Value defines which target color was selected
                    handleBlack = [thBtnB, minBtnB];
                    handleWhite = [thBtnW, minBtnW];
                    if bckBtn.Value == 1 % black
                        set(handleBlack, 'Enable', 'on');
                        set(handleWhite, 'Enable', 'off');
                    elseif bckBtn.Value == 2 % white
                        set(handleBlack, 'Enable', 'off');
                        set(handleWhite, 'Enable', 'on');
                    else % bckBtn.Value == 3
                        set(handleBlack, 'Enable', 'on');
                        set(handleWhite, 'Enable', 'on');
                    end
                    bckBtn.UserData     = false; % reset to get notified for next change
                    start(obj.camHand);
                    % video player need to be released for new size %####
                    release(origVdo);
                    release(thVdoB);
                    release(thVdoW);
                end
                % update mask %#### Sara/Aylin
                if maskBtn.UserData == 1 % change in selection indicated in UserData
                    stop(obj.camHand);
                    % upload and use selected mask:
                    allMasks = {'noMask' 'maskCam1' 'maskCam2' 'maskN' 'maskS' 'maskHexagon' 'maskCustom'};
                    selectedMask = allMasks{maskBtn.Value}; % maskBtn.Value defines which mask was selected

                    if maskBtn.Value == 1 % no mask, use full camera image
                        obj.mask   = obj.noMask; % full image
                        obj.maskID = 'noMask';
                    else
                        customMask = load(strcat(selectedMask,'.mat'));
                        obj.mask   = customMask.mask;
                        obj.maskID = selectedMask; % save name of selected mask
                    end
                    maskBtn.UserData    = false;
                    start(obj.camHand);
                end

                % pick up one frame from camera
                frm  = getsnapshot(obj.camHand);
                frmB = frm; % used to display for black target
                frmW = frm; % used to display for white target
                % threshold the frame for both target colors %#### BW Aylin
                bFrm = sum(frm,3)<obj.blobThresholdB;
                wFrm = sum(frm,3)>obj.blobThresholdW;

                % do the blob-analysis depending on target object color, use mask to specify ROI
                % [x,y angle] of tracked object depend on results of blob analysis, frame threshold and defined mask
                switch bckBtn.Value
                    case 1 % black target object %#### BW Aylin
                        % do the blob-analysis
                        obj.blobHand.MinimumBlobArea = obj.blobSizeB; % min blob size black target %#### BW Aylin
                        [ctrB,angB] = step(obj.blobHand, bFrm & obj.mask); %####

                        if ~isempty(ctrB)
                            % add a yellow line to indicate the beak-direction
                            img = insertShape(frm, 'Line', ...
                                [ctrB ctrB(1)+50*sin(angB) ctrB(2)+50*cos(angB)], ...
                                'LineWidth', 5);
                            % set everything above threshold to white (255) - correct?
                            frmB(~bFrm) = 255; %#### BW Aylin
                            frmB = insertShape(frmB, 'Line', ...
                                [ctrB ctrB(1)+50*sin(angB) ctrB(2)+50*cos(angB)], ...
                                'LineWidth', 5);
                            xTxtB.String = num2str(ctrB(1));
                            yTxtB.String = num2str(ctrB(2));
                            aTxtB.String = num2str(angB);
                        else
                            % text to indicate lack of object
                            img = insertText(frm, [100 100], 'No Object');
                            % set everything above threshold to white (255) - correct?
                            frmB(~bFrm) = 255; %#### BW Aylin
                            frmB = insertText(frmB, [100 100], 'No Object');
                            xTxtB.String = 'no obj';
                            yTxtB.String = 'no obj';
                            aTxtB.String = 'no obj';
                        end

                    case 2 % white target object %#### BW Aylin
                        % do the blob-analysis
                        obj.blobHand.MinimumBlobArea = obj.blobSizeW; % min blob size white target %#### BW Aylin
                        [ctrW,angW] = step(obj.blobHand, wFrm & obj.mask); %####

                        if ~isempty(ctrW)
                            % add a yellow line to indicate the beak-direction
                            img = insertShape(frm, 'Line', ...
                                [ctrW ctrW(1)+50*sin(angW) ctrW(2)+50*cos(angW)], ...
                                'LineWidth', 5);
                            % set everything below threshold to black (0) - correct?
                            frmW(~wFrm) = 0; %#### BW Aylin
                            frmW = insertShape(frmW, 'Line', ...
                                [ctrW ctrW(1)+50*sin(angW) ctrW(2)+50*cos(angW)], ...
                                'LineWidth', 5);
                            xTxtW.String = num2str(ctrW(1));
                            yTxtW.String = num2str(ctrW(2));
                            aTxtW.String = num2str(angW);
                        else
                            % text to indicate lack of object
                            img = insertText(frm, [100 100], 'No Object');
                            % set everything below threshold to black (0) - correct?
                            frmW(~wFrm) = 0; %#### BW Aylin
                            frmW = insertText(frmW, [100 100], 'No Object');
                            xTxtW.String = 'no obj';
                            yTxtW.String = 'no obj';
                            aTxtW.String = 'no obj';
                        end


                    case 3 % both - black and white target object
                        % do the blob-analysis
                        obj.blobHand.MinimumBlobArea = obj.blobSizeB; % min blob size black target %#### BW Aylin
                        [ctrB,angB] = step(obj.blobHand, bFrm & obj.mask);

                        obj.blobHand.MinimumBlobArea = obj.blobSizeW; % min blob size white target %#### BW Aylin
                        [ctrW,angW] = step(obj.blobHand, wFrm & obj.mask);

                        if ~isempty(ctrB) % black object tracked
                            % add a yellow line to indicate the beak-direction
                            img = insertShape(frm, 'Line', ...
                                [ctrB ctrB(1)+50*sin(angB) ctrB(2)+50*cos(angB)], ...
                                'LineWidth', 5);
                            % set everything above threshold to white (255) - correct?
                            frmB(~bFrm) = 255; %#### BW Aylin
                            frmB = insertShape(frmB, 'Line', ...
                                [ctrB ctrB(1)+50*sin(angB) ctrB(2)+50*cos(angB)], ...
                                'LineWidth', 5);
                            xTxtB.String = num2str(ctrB(1));
                            yTxtB.String = num2str(ctrB(2));
                            aTxtB.String = num2str(angB);
                        else % no black object tracked
                            % text to indicate lack of object
                            img = insertText(frm, [100 100], 'No black Object');
                            % set everything above threshold to white (255) - correct?
                            frmB(~bFrm) = 255; %#### BW Aylin
                            frmB = insertText(frmB, [100 100], 'No black Object');
                            xTxtB.String = 'no black obj';
                            yTxtB.String = 'no black obj';
                            aTxtB.String = 'no black obj';
                        end

                        if ~isempty(ctrW) % white object tracked
                            % add a yellow line to indicate the beak-direction
                            img = insertShape(img, 'Line', ...
                                [ctrW ctrW(1)+50*sin(angW) ctrW(2)+50*cos(angW)], ...
                                'LineWidth', 5);
                            % set everything below threshold to black (0) - correct?
                            frmW(~wFrm) = 0; %#### BW Aylin
                            frmW = insertShape(frmW, 'Line', ...
                                [ctrW ctrW(1)+50*sin(angW) ctrW(2)+50*cos(angW)], ...
                                'LineWidth', 5);
                            xTxtW.String = num2str(ctrW(1));
                            yTxtW.String = num2str(ctrW(2));
                            aTxtW.String = num2str(angW);
                        else % no white object tracked
                            % text to indicate lack of object
                            img = insertText(img, [100 100], 'No white Object');
                            % set everything below threshold to black (0) - correct?
                            frmW(~wFrm) = 0; %#### BW Aylin
                            frmW = insertText(frmW, [100 100], 'No white Object');
                            xTxtW.String = 'no white obj';
                            yTxtW.String = 'no white obj';
                            aTxtW.String = 'no white obj';
                        end
                end

                % show the frames
                step(origVdo,img);
                step(thVdoB,frmB); %#### BW Aylin
                step(thVdoW,frmW); %#### BW Aylin
            end
        end
    end
end
