% initOTBR
%
% Call this script to use all functions of the OTBR Toolbox.
%
%
% Copyright:	Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
% Contact:      OTBR-Toolbox@ruhr-uni-bochum.de
% Source Code:  https://gitlab.ruhr-uni-bochum.de/ikn/OTBR
%
% Please cite the OTBR Toolbox where this function is used.
% Cite as:      Otto, T., Rose, J. The open toolbox for behavioral research. Behav Res (2023). 
%               https://doi.org/10.3758/s13428-023-02199-x
%
% See also: mySetup

% VERSION HISTORY:
% Author:  		Tobias Otto, Jonas Rose
% Version: 		1.21
% Last Change:  26.04.2024

% 17.05.2018, Tobias: release version based on start.m
% 07.06.2018, Tobias: split mySetup into hardware and paradigm specific files
% 14.06.2018, Tobias: avoid usage of temp
% 20.12.2018, Tobias: allowing raspberry3 as string
% 31.01.2019, Tobias: bugfix for raspberry3 ip interface
% 13.03.2019, Tobias: Turn off pagination (problem that less waits for user input)
% 16.07.2019, Tobias: define variable interface to avoid problem in deployed pograms
% 12.03.2020, Tobias: added init done at the end
% 14.08.2020, Tobias: added MATLAB version to global struct
% 08.09.2020, Tobias: bugfix for parallel port support
% 12.03.2021, Tobias: changed position of psychstart for new psych toolbox (3.0.16); added GStreamer info
% 18.03.2021, Tobias: added own PsychStartup version for new GStreamr versions
% 19.05.2021, Tobias: minimizing info on Raspberry
% 09.11.2021, Tobias: Try modified version of PsychStartup immediately; cosmetics
% 06.03.2022, Tobias: More cosmetics and added licens information
% 24.03.2022, Tobias: moved computations form myParadigmSetup here
% 24.05.2022, Tobias: using PsychStartup again (for newer toolbox versions)
% 15.06.2022, Tobias: moved ephys setup settings here and introduced new ID for ephys setups
% 20.12.2022, Tobias: added svn remove; always call PsychStartup also in deployed applications
% 02.10.2023, Tobias: removing call of removeSVNEntriesFromString
% 05.10.2023, Tobias: added citation
% 26.04.2024, Tobias: bugfix in check for my*Setup.m files

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           -----------------------------------------
%           | DO NOT edit anything beyond this line |
%           -----------------------------------------
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
year = datestr(now,'yyyy');
clc
disp(' ===================================================================');
disp('                Open Toolbox for Behavioral Research');
disp(' ===================================================================');

%% Add global variables
global SETUP
global WINDOW
global NETWORK
NETWORK.num     = 0;    % Number of active network connections
WINDOW.handle   = [];
on              = 1;
off             = 0;
interface       = -1;

%% Get setup information
if~(exist('myHardwareSetup', 'file') && exist('myParadigmSetup', 'file'))
    disp(' ================================================================');
    disp(' Sorry, but I can''t find the file(s) myHardwareSetup.m')
    disp(' or myParadigmSetup.m !');
    disp(' Please copy them into the current working directory');
    disp(' ================================================================');
    error('Please solve problem and try again');
else
    myHardwareSetup;
    myParadigmSetup;
end

% Order fields
SETUP = orderfields(SETUP);

%% More user info
if(strcmpi(SETUP.platform, 'Windows64Bit'))
    disp('             Developed at the Ruhr University in Bochum, ');
    disp('                Institute of Cognitive Neuroscience');
    disp(' ');
    disp(' Cite the Modular and Open Platform for Behavioral Research Toolbox');
    disp('   where any of the toolbox-functions are used or results obtained ');
    disp('                 using the toolbox are published as:');
    disp('    Otto, T., Rose, J. The open toolbox for behavioral research.')
    disp('    Behav Res (2023). https://doi.org/10.3758/s13428-023-02199-x')
    disp(' ');
    disp(['          Copyright 2007-' year ', CC-GNU GPL by attribution']);
    disp('               https://gitlab.ruhr-uni-bochum.de/ikn/OTBR');
    disp(' ');
    disp(' ');
    disp('Initializing ...');
else
    disp('             Developed at the Ruhr University in Bochum, ');
    disp(' ');
    disp(['          Copyright 2007-' year ', CC-GNU GPL by attribution']);
    disp('               https://gitlab.ruhr-uni-bochum.de/ikn/OTBR');
    disp(' ');
    disp('Initializing ...');
end

%% Initializations and minor computations
% For Monitor dimensions
SETUP.screen.width  = SETUP.screen.width/SETUP.screen.ScreenWidth;
SETUP.screen.height = SETUP.screen.height/SETUP.screen.ScreenHeight;
SETUP.screen.X      = SETUP.screen.X/SETUP.screen.ScreenWidth;
SETUP.screen.Y      = SETUP.screen.Y/SETUP.screen.ScreenHeight;
% For screen color
SETUP.screen.bgColor= SETUP.screen.bgColor*255;

% For touchscreen
SETUP.touchScreen.time              = 0;
SETUP.touchScreen.lastTime          = 0;
SETUP.touchScreen.localValue        = [0 0];
SETUP.touchScreen.lastLocalValue    = [0 0];
SETUP.touchScreen.globalValue       = [0 0];
SETUP.touchScreen.lastGlobalValue   = [0 0];
SETUP.touchScreen.screenIndex       = 0;
SETUP.touchScreen.lastScreenIndex   = 0;
SETUP.touchScreen.found             = 0;
SETUP.touchScreen.offIndex          = -1;
SETUP.touchScreen.deltaP            = SETUP.touchScreen.deltaP/mean([SETUP.screen.ScreenWidth, SETUP.screen.ScreenHeight]);

% For ephys (photo diode)
SETUP.ephys.photodiode = SETUP.ephys.photodiode./ ...
    [SETUP.screen.ScreenWidth SETUP.screen.ScreenHeight SETUP.screen.ScreenWidth SETUP.screen.ScreenHeight];

% Convert strings of feeder type to numbers
switch lower(SETUP.io.feederType)
    case 'time'
        SETUP.io.feederType = 1;
    case 'amount'
        SETUP.io.feederType = 2;
end

% Init variable to control initializations
SETUP.networkDIO.initDone = 0;  % Set this value anyway, even if we use other hardware

% Init raspberry
if(~isfield(SETUP.raspberry, 'num'))
    SETUP.raspberry.num = 0;
end

% Compute values for events
SETUP.events.powersOf2  = 2.^(0:length(SETUP.events.pins)-1);
SETUP.events.pinsLen    = length(SETUP.events.pins);

% General init done
SETUP.initDone = 0;

%% Check inputs in myParadigmSetup & myHardwareSetup
% %%%%%%%%%%%%%%%%%%%%%%%%
if(any(SETUP.screen.X > 1))
    disp(' ================================================================');
    disp(' Check entries in myParadigmSetup.m! ');
    disp(' At least one entry of the left lower corner in X direction is ');
    disp(' bigger than the screen width itself.');
    disp(' Check entries in section 1');
    disp(' ================================================================');
    error('Please solve problem and try again');
end

if(any(SETUP.screen.Y > 1))
    disp(' ================================================================');
    disp(' Check entries in myParadigmSetup.m! ');
    disp(' At least one entry of the left lower corner in Y direction is ');
    disp(' bigger than the screen height itself.');
    disp(' Check entries in section 1');
    disp(' ================================================================');
    error('Please solve problem and try again');
end

if(length(SETUP.screen.X) ~= length(SETUP.screen.Y))
    disp(' ================================================================');
    disp(' Check entries in mySetup.m! ');
    disp(' The number of switch positions in X and Y direction aren''t equal');
    disp(' Correct this problem in section 6');
    disp(' ================================================================');
    error('Please solve problem and try again');
end

if(length(SETUP.screen.X)+length(SETUP.screen.Y) ~= ...
        length(SETUP.screen.width)+length(SETUP.screen.height))
    disp(' ================================================================');
    disp(' Check entries in myParadigmSetup.m! ');
    disp(' The number of entries of switch positions aren''t equal to the');
    disp(' entries in switch width and height. Correct this problem in ');
    disp(' section 1 or 2.');
    disp(' ================================================================');
    error('Please solve problem and try again');
end

if(strcmpi(SETUP.io.interface, 'NetworkBox') && isempty(SETUP.networkDIO.odroidIP))
    disp(' ================================================================');
    disp(' Check entries in myHardwareSetup.m! ');
    disp(' The network IO device (ACN) needs an IP address as string');
    disp(' Correct this problem in section 2.');
    disp(' ================================================================');
    error('Please solve problem and try again');
end

%% Set new state for rand !!!
% Get info about the environment
if(strcmpi(SETUP.platform, 'Windows64Bit'))
    % Get SETUP.platformNum info
    tmpVer = regexp(version, ' ', 'split');
    SETUP.platformNum   = 1;
    SETUP.MATLABVerStr 	= tmpVer{1};
    tmpVer              = regexp(SETUP.MATLABVerStr, '\.', 'split');
    SETUP.MATLABVer     = str2double(tmpVer{1})+str2double(tmpVer{2})*0.1;

    % Check MATLAB version
    tmp0	= version('-release');
    %tmp1    = tmp0(end);
    tmp2    = str2double(tmp0(1:end-1));

    if(tmp2 < 2012 || strcmpi('2012a', tmp0))
        error('This MATLAB version is too old. Please update at least to MATLAB 2012b');
    elseif(tmp2 >= 2012 && tmp2 < 2015)
        RandStream.setGlobalStream(RandStream('mt19937ar','Seed',sum(100*clock)));
    elseif(tmp2 >= 2015)
        rng('shuffle');
    end
elseif(strcmpi(SETUP.platform, 'RaspberryPi2') || strcmpi(SETUP.platform, 'RaspberryPi3'))
    % Get SETUP.platformNum info
    SETUP.platformNum   = 2;
    SETUP.MATLABVerStr 	= '';
    SETUP.MATLABVer     = [];

    % No init necessary!
    % --> By default, the generator is initialized from /dev/urandom if it is available, otherwise
    % from CPU time, wall clock time, and the current fraction of a second. Note that this differs
    % from MATLAB, which always initializes the state to the same state at startup.
else
    disp(' ================================================================');
    disp(' Check entries in myHardwareSetup.m! ');
    disp(' The name of the SETUP.platform is wrong specified!');
    disp(' Correct this problem in section (1c)');
    disp(' ================================================================');
    error('Please solve problem and try again');
end

% Doesn't hurt (for really old functions of the biopsy toolbox)
tic

%% Add path for psychtoolbox toolbox - only if program is not deployed
if(~isdeployed)
    % Check for correct entry of the Open Toolbox for Behavioral Research
    if(exist([SETUP.toolboxPath filesep 'EventCode'],'dir') && ...
            exist([SETUP.toolboxPath filesep 'IO'],'dir') && ...
            exist([SETUP.toolboxPath filesep 'Network'],'dir') && ...
            exist([SETUP.toolboxPath filesep 'Screen'],'dir') && ...
            exist([SETUP.toolboxPath filesep 'Sound'],'dir') && ...
            exist([SETUP.toolboxPath filesep 'Tools'],'dir') && ...
            exist([SETUP.toolboxPath filesep 'Touchscreen'],'dir')  && ...
            exist([SETUP.toolboxPath filesep 'Tracking'],'dir') && ...
            exist([SETUP.toolboxPath filesep 'Video'],'dir'))
        disp(['--> Found Open Toolbox for Behavioral Research in: ' SETUP.toolboxPath])
    else
        disp(' ====================================================================');
        disp(' Check the entry of the OTBR Toolbox path');
        disp(' There is no toolbox in this path or the folder structure is broken');
        disp(' Please correct this problem in the myHardwareSetup.m file in section 1a.');
        disp(' ================================================================');
        error('Please solve problem and try again');
    end

    % Check for correct entry of the PsychToolbox (most important entries)
    if(exist([SETUP.psychToolboxPath filesep 'PsychBasic'],'dir') && ...
            exist([SETUP.psychToolboxPath filesep 'PsychOneliners'],'dir') && ...
            exist([SETUP.psychToolboxPath filesep 'PsychRects'],'dir') && ...
            exist([SETUP.psychToolboxPath filesep 'PsychSound'],'dir') && ...
            exist([SETUP.psychToolboxPath filesep 'PsychJava'],'dir'))
        disp(['--> Found Psychtoolbox in: ' SETUP.psychToolboxPath])
    else
        disp(' ====================================================================');
        disp(' Check the entry of the Psychophysics Toolbox path');
        disp(' There is no toolbox in this path or the folder structure is broken');
        disp(' Please correct this problem in the myHardwareSetup.m file in section 1b.');
        disp(' ================================================================');
        error('Please solve problem and try again');
    end

    % Add path for helper function
    addpath([SETUP.toolboxPath filesep 'Tools']);

    % Get all folders
    psychPath = removeGitEntriesFromString(genpath(SETUP.psychToolboxPath));

    % Set path for psych toolbox
    addpath(psychPath);

    %% Change paths according to the Psychophysics Toolbox PostInstRoutine
    % Is this a Release2007a or later Matlab?
    if(SETUP.platformNum == 1)
        % This is a R2007a or post R2007a Matlab:
        % Add MatlabWindowsFilesR2007a subfolder to Matlab path:
        addpath([SETUP.psychToolboxPath '\PsychBasic\MatlabWindowsFilesR2007a\']);
        disp('--> using Matlab release 2007a or later');
    elseif(SETUP.platformNum == 2)
        % This is a Raspberry system
        % Add Octave3LinuxFilesARM subfolder to Matlab path:
        addpath([SETUP.psychToolboxPath '/PsychBasic/Octave3LinuxFilesARM/']);
        disp('--> using Octave on Raspberry Pi');
    else
        % This is a pre-R2007a Matlab:
        % Add MatlabWindowsFilesR11 subfolder to Matlab path:
        disp(' --------------------------------------------');
        disp(' --> Matlab release prior to R2007a detected.');
        disp(' Sorry this version is no longer supported!');
        disp(' Please upgrade to a newer version of MATLAB');
        disp(' --------------------------------------------');
        error('Please solve problem and try again');
    end

    %% Set path for OTBR toolbox - only if program is not deployed
    % Get all folders
    bioPath = removeGitEntriesFromString(genpath(SETUP.toolboxPath));

    % Set path for biopsy toolbox finally
    addpath(bioPath);

    % Check which interface is used
    switch lower(SETUP.io.interface)
        case 'parallel'
            interface = 2;
        case 'keyboard'
            interface = 3;
        case 'networkbox'
            interface = 7;
        case 'networkboxmatlab'
            interface = 8;
        case 'raspberrypi2'
            interface = 9;
        case 'raspberrypi3'
            interface = 9;
        case 'networkboxacn'
            interface = 10;
        otherwise
            disp(' ================================================================');
            disp(' Check entries in myHardwareSetup.m! ');
            disp(' Please specify correct IO interface in myHardwareSetup. Correct this')
            disp(' problem in section 2')
            disp(' ================================================================');
            error('Please solve problem and try again');
    end

    % Put path entry on top of the path list (like the psycho phsyics toolbox)
    addpath([SETUP.toolboxPath filesep 'IO' filesep 'Interface' num2str(interface)]);

    % Save interface number to SETUP struct
    SETUP.io.interfaceNum = interface;
end

%% Call PsychStartup for Psych toolbox (also in deployed applications)
try
    % Try psych version (newer psych toolbox works fine now)
    PsychStartup;

    % If older versions are used, try the OTBR version
    if(isempty(getenv('GSTREAMER_1_0_ROOT_MSVC_X86_64')))
        % Try modified version of PsychStartup
        PsychStartup_OTBR;
    end
catch
    disp(' ================================================================');
    disp(' Call of PsychStartup didn''t work out! You can use the  Open ');
    disp(' Toolbox for Behavioral Research anyway, but problems may occur!');
    disp(' Please install gstreamer on your system to use the Psychophysics');
    disp(' Toolbox completely!');
    disp(' ');
    disp(' Make sure to install the MSVC gstreamer variant!');
    disp(' --> http://psychtoolbox.org/docs/GStreamer');
    disp(' ================================================================');
end

%% Copy necessary dlls into working directory - only if program is not deployed
% Check for windows and MATLAB systems only (SETUP.platformNum == 1)
foundHW = 0;
if(~isdeployed)
    if(interface ~= 3 && SETUP.platformNum == 1)
        curDir = cd;
        switch interface
            case 2
                if(isempty(dir('inpoutx64.dll')))
                    copyfile([SETUP.toolboxPath '\IO\Interface2\inpoutx64.dll'] , curDir);
                    disp('Copied inpoutx64.dll (InpOut32Drv Driver Interface DLL) into the current working directory');
                    disp('Modified for x64 compatibility and built by Phillip Gibbons (Phil@highrez.co.uk).');
                    disp('See http://www.highrez.co.uk/Downloads/InpOut32 or the Highrez Forums (http://forums.highrez.co.uk) for information.');
                    disp('Based on the original written by Logix4U (www.logix4u.net).');
                    foundHW = 1;
                end
        end
    end
end

% Check for all systems (Windows and MATLAB/Octave and Raspberry), if we
% haven't found something yet
if(interface ~= 3 && foundHW == 0)
    switch interface
        case 7
            SETUP.networkDIO.initDone   = 0;
        case 8
            SETUP.networkDIO.initDone   = 0;
        case 9
            SETUP.raspberryPi2.initDone	= 0;
        case 10
            SETUP.networkDIO.initDone   = 0;
    end
end

%% Set verbosity of Psychtoolbox
if(SETUP.platformNum == 1)
    Screen('Preference', 'Verbosity', SETUP.psychToolbox.verbosity);
elseif(SETUP.platformNum == 2)
    % This is a Raspberry system --> set to zero anyway!
    % Otherwise Octave will get stuck and waits for space press
    Screen('Preference', 'Verbosity', 0);

    % Turn off pagination (problem that less waits for user input)
    more off

    % Since less made problems we use more from now on
    PAGER ('more');
end

%% Remap pin numbers to names for keyboard use during programming
SETUP.io.remapNames = []; % Struct where the names are stored

% Copy field names and delete unnecessary names
names                                   = fieldnames(SETUP.io);
names(strcmpi(names,'inputKeys'))       = [];
names(strcmpi(names,'remapNames'))      = [];
names(strcmpi(names,'interface'))       = [];
names(strcmpi(names,'feederType'))      = [];
names(strcmpi(names,'interfaceNum'))	= [];
counter                                 = 1;

for i=1:length(names)
    if(~isempty(SETUP.io.(names{i})) && ~ischar(SETUP.io.(names{i})))
        len = length(SETUP.io.(names{i}));
        for j=1:len
            if(SETUP.io.(names{i}) > 0)
                SETUP.io.remapNames{1,SETUP.io.(names{i})(j)} = names{i};
                counter = counter + 1;
            else
                disp(' ================================================================');
                disp([' The value entered in SETUP.io.' names{i} ' is not valid !!!']);
                disp(' Please enter values bigger than zero or an empty matrix []');
                disp(' if no device is attached. See myHardwareSetup.m for details');
                disp(' ================================================================');
                error('Please solve problem and try again');
            end
        end
    end
end

%% Init timer (WINDOW only)
if(SETUP.platformNum == 1)
    %% Init timer for manual shaping
    if(SETUP.manualShaping.status == 1)
        disp(' ================================================================');
        disp('               !!! Manual shaping mode is used !!!');
        disp('            Please call closeWindow to stop manual shaping.');
        disp(' ================================================================');

        SETUP.manualShaping.reward  = 0;
        SETUP.manualShaping.timer   = timer('TimerFcn','callback_manualShaping', ...
            'BusyMode','Queue', ...
            'Period',0.1 , ...
            'ExecutionMode','fixedSpacing');
        start(SETUP.manualShaping.timer);
    end
end

%% Add entry in struct for startStopDevice timers
% Let's start with 32 entries. If someone needs more it's added
% automatically. This is done here only for performance reasons
SETUP.startStop.startStopTimer         = [];
SETUP.startStop.startStopTimerIndex    = zeros(1,32);

%% Init sound
% If enabled initialize the psych toolbox sound
if(SETUP.sound.enable == 1)
    disp('--> Initializing PsychToolbox Sound ...');
    % Delete handles if exist -> try catch in case InitializePsychSound
    % wasn't called before
    try
        closeSound;
    end

    % Init audio!
    initSound;
end

%% Init UDP viewer
% open udp communication with control pc
if isfield(SETUP,'udp') && ~isempty(SETUP.udp.ip)
    SETUP.udp.obj = udp(SETUP.udp.ip, SETUP.udp.port, 'localport', SETUP.udp.lclPort);
    fopen(SETUP.udp.obj);
end

%% Init tracking parameters - memory mapped file and tracker ID
% Define setup ID
if(strcmpi(SETUP.trainingSetup, 'arena'))
    SETUP.trainingSetupID = 1;
elseif(strcmpi(SETUP.trainingSetup, 'box'))
    SETUP.trainingSetupID = 2;
else
    SETUP.trainingSetupID = 0;
end

% create memory mapped file(s) for gaze-tracker start the tracker(s)
if ~isempty(SETUP.tracker.bufFleNme)
    for i=1:length(SETUP.tracker.bufFleNme)
        % create the memory mapped file for data exchange
        if ~exist(SETUP.tracker.bufFleNme{i},'file')
            f = fopen(SETUP.tracker.bufFleNme{i},'w+');
            fwrite(f,[0 0 0 0 0],'double');
            fclose(f);
        end
        % map file to memeoy
        SETUP.tracker.bufFle{i} = ...
            memmapfile(SETUP.tracker.bufFleNme{i},'Format','double');
    end
end

% load ConversionMatrix
if strcmpi(SETUP.trainingSetup, 'box')
    load(SETUP.tracker.conversionMatNme);   %specify file path
    SETUP.trackerConversionMatrix1 = C1;    %matrix for camera one
    SETUP.trackerConversionMatrix2 = C2;    %matrix for camera two
end

%% Init naming scheme for keyboard answers
if(strcmpi(SETUP.io.interface, 'Keyboard'))
    disp('--> Init naming scheme for keyboard answers ...');
    KbName('UnifyKeyNames');
    disp('                                                   done!');
end

%% Define setting for Ephys hardware --> deuteron
if strcmpi(SETUP.ephysSetup, 'deuteron')
    SETUP.events.lastPin = 1; % only necessary for event codes using deuteron
end

% Use an ID for special ephys systems (required in sendEvent)
if strcmpi(SETUP.ephysSetup, 'deuteron')
    SETUP.ephysSetupID = 1;
elseif strcmpi(SETUP.ephysSetup, 'intan')
    SETUP.ephysSetupID = 2;
else
    SETUP.ephysSetupID = 0;
end

%% Clear needless variables
clear names counter bioPath i j needless pos startPos interface curDir len
clear tmp0 k ans year tmp2 tmp1 pc psychPath foundHW tmpVer gstr1 gstr2

%% Done
% If we reach this point everything went probably fine ;)
SETUP.initDone = 1;
disp('                 ... done');
